/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

int lwm2_papi_enable = 0;

unsigned long lwm2_hwc_config = 0;
unsigned long lwm2_hwc_registered = 0;

#include "../../lwm2-config.h"

#ifdef LWM2_MEASUREMENT_HWCOUNTER

#include  <papi.h>
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>

/* Logging */
#include "../../util/lwm2-log.h"

#include "../../lwm2-signal.h"

#include "../../lwm2-memory.h"

/* Header with hardware counter event definitions */
#include "../lwm2-hwc-adapter.h"

#include "lwm2-hwc-papi.h"

#include "../../lwm2-pulse-thread.h"
#include "../../lwm2-storage.h"

#ifdef DLWM2_HWC_EXTERN_DEF
#include "../../modules/hwc/lwm2-hwc-extern.h"
#endif


/* PAPI features */
unsigned int feature_multiplexing = 0;



char papi_err_str[PAPI_MAX_STR_LEN];

int lwm2_thread_event_sets[MAX_THREADS];

int lwm2_event_set_index = 0;


/* Private functions */

/**
 * Provides mapping from the general hardware counter definitions to the
 * PAPI specific ones
 */
int lwm2_hwc_papi_translate(int counter)
{
	switch(counter)
	{
	case LWM2_HWCOUNTER_CYCLES:
		return PAPI_TOT_CYC;

	case LWM2_HWCOUNTER_L1_DATA_ACCESS:
		return PAPI_L1_DCA;

	case LWM2_HWCOUNTER_L1_DATA_MISSES:
		return PAPI_L1_DCM;

	case LWM2_HWCOUNTER_L1_INS_MISSES:
		return PAPI_L1_ICM;

	case LWM2_HWCOUNTER_L1_INS_HITS:
		return PAPI_L1_ICH;

	case LWM2_HWCOUNTER_FP_OPS:
		return PAPI_FP_OPS;
		
	case LWM2_HWCOUNTER_FP_INS:
		return PAPI_FP_INS;
		
	case LWM2_HWCOUNTER_LOADS:
		return PAPI_LD_INS;
		
	case LWM2_HWCOUNTER_STORES:
		return PAPI_SR_INS;
		
	case LWM2_HWCOUNTER_INST:
		return PAPI_TOT_INS;
		
	case LWM2_HWCOUNTER_L1_LOAD_MISSES:
		return PAPI_L1_LDM;
		
	case LWM2_HWCOUNTER_L1_STORE_MISSES:
		return PAPI_L1_STM;
			
	case LWM2_HWCOUNTER_LOAD_INST:
		return PAPI_LD_INS;
		
	case LWM2_HWCOUNTER_STORE_INST:
		return PAPI_SR_INS;
			
	case LWM2_HWCOUNTER_LOAD_STORE_INST:
		return PAPI_LST_INS;
	}

	/* Default */
	return -1;
}

unsigned long lwm2_hwc_get_hwc_config()
{
	if(lwm2_hwc_config == 0)
	{
		char* hwc_config = getenv("LWM2_HWC_CONFIG");
		if(hwc_config != NULL)
		{
			if(strcmp(hwc_config, "LWM2_HWC_CPI") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_FLOPS") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_FLOPS;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_FLOINS") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_FLOINS;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_L1_RATIO") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_L1_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_LOAD_STORE_RATIO") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_LOAD_STORE_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_CPU_ONLY") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_FLOINS;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_MEM_ONLY") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_L1_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_CPU_BASIC") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_MEM_ALT") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_LOAD_STORE_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_ALL") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_FLOINS | LWM2_HWC_L1_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_ALL_BASIC") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_L1_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_ALL_ALT") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_FLOINS | LWM2_HWC_LOAD_STORE_RATIO;
			}
			else if(strcmp(hwc_config, "LWM2_HWC_ALL_ALT_BASIC") == 0)
			{
				lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_LOAD_STORE_RATIO;
			}
		}
		else
		{
			//default is ALL_BASIC
			lwm2_hwc_config = LWM2_HWC_CPI | LWM2_HWC_FLOPS | LWM2_HWC_L1_RATIO;
		}
	}
	return lwm2_hwc_config;
}

/* Public functions */

void lwm2_hwc_papi_init(void)
{
	int error;
	
	int papi_ver = PAPI_library_init(PAPI_VER_CURRENT);
	if(papi_ver < 0)
	{
		LWM2LOG(2, "Error while initializing the PAPI library\n");
		lwm2_papi_enable = 0;
		return;
	}
	
#ifdef LWM2_PAPI_THREAD_ENABLED
	error = PAPI_thread_init((unsigned long (*) (void)) (pthread_self));
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Couldn't initialize threading in PAPI\n");
		return;
	}
#endif

	/* Retrieve available number of counters */
	lwm2_num_counters_available = PAPI_num_counters();
	
	/* Try to enable multiplexing */
	feature_multiplexing = 0;
		
	/* Enable multiplexing */
	error = PAPI_multiplex_init();
	if(error != PAPI_OK)
	{
		LWM2LOG(3, "Couldn't get multiplexing status of the system\n");
		feature_multiplexing = 0;
		return;
	}
		
	lwm2_papi_enable = 1;
}

void lwm2_hwc_papi_setup(void* thread_storage)
{
	int error;
	struct lwm2_hwc_storage* hwc_storage = (struct lwm2_hwc_storage*) thread_storage;
	
	if(lwm2_thread_is_pulse_thread == 1)
	{
		hwc_storage->event_set = PAPI_NULL;
		return;
	}
	
#ifdef LWM2_PAPI_THREAD_ENABLED
	error = PAPI_register_thread();
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Couldn't register thread with PAPI\n");
		return;
	}
#endif
	
	memset(hwc_storage->values, 0, sizeof(long_long) * LWM2_HWC_COUNT);
	memset(hwc_storage->old_values, 0, sizeof(long_long) * LWM2_HWC_COUNT);
	hwc_storage->num_counters = 0;
	hwc_storage->event_set = PAPI_NULL;

	/* Create default event set */
	error = PAPI_create_eventset(&hwc_storage->event_set);
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Can't create event set. Disabling PAPI\n");
		lwm2_papi_enable = 0;
		return;
	}

	if(feature_multiplexing != 0)
	{ 
		/* Enable multiplexing on event set */
		error = PAPI_set_multiplex(hwc_storage->event_set);
		if(error != PAPI_OK)
		{
			LWM2LOG(3, "Multiplexing not supported on the system.\n");
			feature_multiplexing = 0;
		}
	}
	
#ifndef LWM2_HWC_EXTERN_DEF	
	/* Add default counters */	
	
	unsigned long hwc_config = lwm2_hwc_get_hwc_config();
	
	int ret = -1;
	
	if((hwc_config & LWM2_HWC_CPI) != 0)
	{
		ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_INST);
		if(ret == LWM2_SUCCESS)
		{
			ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_CYCLES);
		}
		
		if(ret == LWM2_SUCCESS)
		{
			lwm2_hwc_registered |= LWM2_HWC_CPI;
		}
	}
	
	if((hwc_config & LWM2_HWC_FLOPS) != 0)
	{
		ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_FP_OPS);
		
		if(ret == LWM2_SUCCESS)
		{
			lwm2_hwc_registered |= LWM2_HWC_FLOPS;
		}
	}
	
	if((hwc_config & LWM2_HWC_FLOINS) != 0)
	{
		ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_FP_INS);
		
		if(ret == LWM2_SUCCESS)
		{
			lwm2_hwc_registered |= LWM2_HWC_FLOINS;
		}
	}
	
	if((hwc_config & LWM2_HWC_L1_RATIO) != 0)
	{
		ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_L1_DATA_ACCESS);
		
		if(ret == LWM2_SUCCESS)
		{
			ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_L1_DATA_MISSES);
		}
		
		if(ret == LWM2_SUCCESS)
		{
			lwm2_hwc_registered |= LWM2_HWC_L1_RATIO;
		}
	}
	
	if((hwc_config & LWM2_HWC_LOAD_STORE_RATIO) != 0)
	{
		ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_L1_LOAD_MISSES);
		
		if(ret == LWM2_SUCCESS)
		{
			ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_LOAD_INST);
		}

		if(ret == LWM2_SUCCESS)
		{
			ret = lwm2_hwc_papi_add_default_counter(hwc_storage, LWM2_HWCOUNTER_L1_STORE_MISSES);
		}
		
		if(ret == LWM2_SUCCESS)
		{
			ret = lwm2_hwc_papi_add_default_counter(hwc_storage, 	LWM2_HWCOUNTER_STORE_INST);
		}
		
		if(ret == LWM2_SUCCESS)
		{
			lwm2_hwc_registered |= LWM2_HWC_LOAD_STORE_RATIO;
		}
	}

	
#else
	
	int ind;
	for(ind = 0; ind < LWM2_CUSTOM_BASIC_HWC_COUNT; ind++)
	{
//		printf("Adding custom counter %d at index %d\n", lwm2_custom_basic_hwc_codes[ind], ind);
		lwm2_hwc_papi_add_custom_counter(hwc_storage, lwm2_custom_basic_hwc_codes[ind]);
	}
	
#endif
	
	/* Start measurement */
	lwm2_hwc_papi_start(hwc_storage->event_set);
	
	LWM2LOG(2, "Started PAPI with number of counters: %d for event set: %d\n", hwc_storage->num_counters, hwc_storage->event_set);
}

int lwm2_hwc_papi_add_default_counter(struct lwm2_hwc_storage* hwc_storage, int counter)
{
	int papi_event = lwm2_hwc_papi_translate(counter);
	
	return lwm2_hwc_papi_add_counter(hwc_storage, papi_event, counter);
}

int lwm2_hwc_papi_add_custom_counter(struct lwm2_hwc_storage* hwc_storage, int papi_event)
{
	return lwm2_hwc_papi_add_counter(hwc_storage, papi_event, -1);
}

int lwm2_hwc_papi_add_counter(struct lwm2_hwc_storage* hwc_storage, int papi_event, int counter)
{
	if(lwm2_papi_enable != 1)
	{
		LWM2LOG(2, "PAPI is not enabled!\n");
		return LWM2_ERROR;
	}
	
	if(hwc_storage->num_counters + 1 > lwm2_num_counters_available)
	{
		if(feature_multiplexing != 0)
		{
			LWM2LOG(3, "Maximum number of counters reached before subscribing to all counters\n");
			return LWM2_ERROR;
		}
	}

	int error;

	error = PAPI_query_event(papi_event);
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Event %d/%d is not supported on the system! Error: %d\n", counter, papi_event, error);
		return LWM2_ERROR;
	}

	error = PAPI_add_event(hwc_storage->event_set, papi_event);
	
	if(error != PAPI_OK)
	{
		LWM2LOG(3, "Failed to subscribe to counter %d/%d\n", counter, papi_event);
		return LWM2_ERROR;
	}

#ifndef LWM2_HWC_EXTERN_DEF
	/* Adding counter to event set successful, increase num */
	hwc_storage->event_ids[hwc_storage->num_counters] = counter;
#else
	hwc_storage->event_ids[hwc_storage->num_counters] = papi_event;
#endif
	hwc_storage->num_counters++;
//	LWM2LOG(0, "Total number of PAPI counters now: %d for event set: %d\n", hwc_storage->num_counters, hwc_storage->event_set);
	
	return LWM2_SUCCESS;
}



void lwm2_hwc_papi_start(int event_set)
{
	if(lwm2_papi_enable != 1 || event_set == PAPI_NULL)
	{
		return;
	}
	int rtn = PAPI_start(event_set);
	if(rtn != PAPI_OK)
	{
		char* err_msg = PAPI_strerror(rtn);
		LWM2LOG(2, "Failed to start PAPI for event set: %d with error: %d -> %s\n", event_set, rtn, err_msg);
		lwm2_papi_enable = 0;
	}
}

void lwm2_hwc_papi_destroy(int event_set)
{
	
	if(lwm2_papi_enable != 1 || event_set == PAPI_NULL)
	{
		return;
	}
	
	int error;
	long_long v[LWM2_HWC_COUNT];

	/* Stop event set */
	error = PAPI_stop(event_set, NULL);
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Failed to stop PAPI\n");
	}

	/* Cleanup */
	error = PAPI_cleanup_eventset(event_set);
	if(error != PAPI_OK)
	{
		LWM2LOG(3, "Failed to clean up the event set\n");
	}

	error = PAPI_destroy_eventset(&event_set);
	if(error != PAPI_OK)
	{
		LWM2LOG(3, "Failed to destroy PAPI eventset\n");
	}
	
	error = PAPI_unregister_thread();
	if(error != PAPI_OK)
	{
		LWM2LOG(2, "Failed to unregister thread\n");
	}
}
#endif
