/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#define _GNU_SOURCE

#include "lwm2-io-wrapper.h"
#include "../../util/lwm2-log.h"
#include "../../lwm2-storage.h"
#include "../../lwm2-signal.h"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <dlfcn.h>

#include <sys/types.h>

enum {
	LWM2_EVENT_IO_WRITE = 0,
	LWM2_EVENT_IO_READ = 1
};

/* ## concatenates, # quotes */
#define get_original_function(newfunc, name, fptr_type) {          \
    newfunc = (fptr_type)dlsym(RTLD_NEXT, # name);                 \
    if (NULL == newfunc)                                           \
    {                                                              \
      LWM2LOG(0,   "Could not get original address of function %s\n", # name);                                                   \
        LWM2LOG(0, "This will potentially effect the functionality of the application. Exiting!\n");		\
        exit(-1);											\
    }                                                              \
}
    
#ifndef LWM2_NO_DLSYM
#ifdef LWM2_MEASURE_IO

typedef int (*fptr_fread)(void *__restrict__ ptr, size_t size, size_t nitems, FILE *__restrict__ stream);

typedef int (*fptr_fwrite)(const void *__restrict__ ptr, size_t size, size_t nitems, FILE *__restrict__ stream);


fptr_fread Pio_fread = NULL;

size_t fread(void *__restrict__ ptr, size_t size, size_t nitems, FILE *__restrict__ stream)
{
	lwm2_storage_set_flag_fileio();

	/* Call original function */
	size_t ret = Pio_fread(ptr, size, nitems, stream);

	if(lwm2_sys_service_active == 0 || lwm2_app_ended == 0)
	{
		lwm2_storage_add_io_read_bytes(size * nitems);
	}

	lwm2_storage_reset_flag_fileio();
	
	return ret;
}


fptr_fwrite Pio_fwrite = NULL;

size_t fwrite(const void *__restrict__ ptr, size_t size, size_t nitems, FILE *__restrict__ stream)
{
	lwm2_storage_set_flag_fileio();

	/* Call original function */
	size_t ret = Pio_fwrite(ptr, size, nitems, stream);
	if(lwm2_sys_service_active == 0 || lwm2_app_ended == 0)
	{
		lwm2_storage_add_io_write_bytes(size * nitems);
	}

	lwm2_storage_reset_flag_fileio();
	
	return ret;
}

#endif
#endif

void lwm2_wrapper_io_init()
{
#ifndef LWM2_NO_DLSYM
#ifdef LWM2_MEASURE_IO
   /* Retrieve original functions */
   get_original_function(Pio_fwrite, fwrite, fptr_fwrite);

   get_original_function(Pio_fread, fread, fptr_fread);
#endif
#endif
}
