/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "lwm2-cuda-adapter.h"
#include "../util/lwm2-log.h"
#include "../util/util_time_measurement.h"
#include "../modules/measurements/lwm2-measurement-cuda.h"
#include "../lwm2-storage.h"

#include <stdio.h>

int lwm2_cupti_app = 0;

int lwm2_cuda_function_call = 0;

//volatile sig_atomic_t lwm2_cuda_active_function = 0;

#ifdef LWM2_MEASUREMENT_CUDA

#define CHECK_CU_ERROR(err, cufunc)                                     \
  if (err != CUDA_SUCCESS)                                              \
    {                                                                   \
 /*     printf ("%s:%d: error %d for CUDA Driver API function '%s'\n", */   \
 /*             __FILE__, __LINE__, err, cufunc);     */                    \
      /*exit(-1);      */                                                   \
    }

#define CHECK_CUPTI_ERROR(err, cuptifunc)                               \
  if (err != CUPTI_SUCCESS)                                             \
    {                                                                   \
      const char *errstr;                                               \
      cuptiGetResultString(err, &errstr);                               \
 /*     printf ("%s:%d:Error %s for CUPTI API function '%s'.\n",    */      \
 /*             __FILE__, __LINE__, errstr, cuptifunc);             */      \
      /*exit(-1);  */														      \
    }

#define CHECK_CUPTI_ERROR_3(err, cuptifunc, cbid)                               \
  if (err != CUPTI_SUCCESS)                                             \
    {                                                                   \
      const char *errstr;                                               \
      cuptiGetResultString(err, &errstr);                               \
  /*    printf ("%s:%d:Error %s for CUPTI API function '%s'.\n",  */        \
  /*            __FILE__, __LINE__, errstr, cuptifunc);           */        \
      /*exit(-1);  */ printf("Error with cbid %d, name %s\n", cbid, lwm2_cuda_function_names[cbid]);               \
    }

#define CUDA_MEM_CPY_COUNT_ENTRY(param_type)			\
		param_type* func_params = (param_type*) cbInfo->functionParams;		\
				enum cudaMemcpyKind kind = func_params->kind;		\
				if(kind == cudaMemcpyHostToDevice) {		\
					lwm2_storage_set_flag_cuda_to_device();		\
				} else if(kind == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(param_type)		\
		param_type* func_params = (param_type*) cbInfo->functionParams;	\
		enum cudaMemcpyKind kind = func_params->kind;		\
				if(kind == cudaMemcpyHostToDevice) {		\
					lwm2_storage_set_flag_cuda_to_device();		\
				} else if(kind == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_ENTRY(param_type)		\
		param_type* func_params = (param_type*) cbInfo->functionParams;		\
		const struct cudaMemcpy3DParms* params = func_params->p;		\
		enum cudaMemcpyKind kind = params->kind;		\
				if(kind == cudaMemcpyHostToDevice) {		\
					lwm2_storage_set_flag_cuda_to_device();		\
				} else if(kind == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_COUNT_EXIT(param_type)			\
		param_type* func_params1 = (param_type*) cbInfo->functionParams;		\
				enum cudaMemcpyKind kind1 = func_params1->kind;		\
				if(kind1 == cudaMemcpyHostToDevice) {		\
					lwm2_storage_reset_flag_cuda_to_device();		\
				} else if(kind1 == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(param_type)		\
		param_type* func_params1 = (param_type*) cbInfo->functionParams;	\
		enum cudaMemcpyKind kind1 = func_params1->kind;		\
				if(kind1 == cudaMemcpyHostToDevice) {		\
					lwm2_storage_reset_flag_cuda_to_device();		\
				} else if(kind1 == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_EXIT(param_type)		\
		param_type* func_params1 = (param_type*) cbInfo->functionParams;		\
		const struct cudaMemcpy3DParms* params1 = func_params1->p;		\
		enum cudaMemcpyKind kind1 = params1->kind;		\
				if(kind1 == cudaMemcpyHostToDevice) {		\
					lwm2_storage_reset_flag_cuda_to_device();		\
				} else if(kind1 == cudaMemcpyDeviceToHost) {	\
					lwm2_storage_set_flag_cuda_to_host();	\
				}

#define CUDA_MEM_CPY_COUNT(param_type)			\
		param_type* func_params = (param_type*) cbInfo->functionParams;		\
				size_t count = func_params->count;		\
				enum cudaMemcpyKind kind = func_params->kind;		\
				cuda_register_mem_cpy(count, kind);

#define CUDA_MEM_CPY_WIDTH_HEIGHT(param_type)		\
		param_type* func_params = (param_type*) cbInfo->functionParams;	\
		size_t width = func_params->width;		\
		size_t height = func_params->height;		\
		enum cudaMemcpyKind kind = func_params->kind;		\
		cuda_register_mem_cpy(width * height, kind);

#define CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH(param_type)		\
		param_type* func_params = (param_type*) cbInfo->functionParams;		\
		const struct cudaMemcpy3DParms* params = func_params->p;		\
		size_t width = params->extent.width;		\
		size_t height = params->extent.height;		\
		size_t depth = params->extent.depth;		\
		enum cudaMemcpyKind kind = params->kind;		\
		cuda_register_mem_cpy(width * height * depth, kind);

#define EVENT_NAME_TESLA "instructions"
#define EVENT_NAME_FERMI "inst_executed"

typedef struct cupti_eventData_st {
  CUpti_EventGroup eventGroup;
  CUpti_EventID eventId;
} cupti_eventData;


/*
  
 
 typedef enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID                                                        = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit                                                         = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion                                             = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet                                                    = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount                                               = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName                                                = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability                                      = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem                                               = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties                                          = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute                                           = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate                                                    = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy                                                   = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach                                                    = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach                                                    = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent                                               = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent                                                = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice                                                 = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize                                               = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad                                                   = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData                                               = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx                                             = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary                                          = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload                                                 = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction                                            = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal                                              = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal                                            = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef                                              = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo                                                   = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo                                                 = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc                                                     = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc                                                   = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch                                                = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch                                              = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree                                                      = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree                                                    = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange                                           = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange                                         = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost                                                 = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost                                                  = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc                                                 = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer                                      = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer                                    = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags                                              = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD                                                   = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD                                                 = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH                                                   = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH                                                 = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD                                                   = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD                                                 = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA                                                   = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA                                                 = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD                                                   = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD                                                 = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA                                                   = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH                                                   = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA                                                   = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D                                                     = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned                                            = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D                                                     = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D                                                   = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync                                              = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync                                            = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync                                              = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync                                            = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync                                              = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync                                            = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync                                              = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync                                              = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync                                                = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync                                                = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync                                              = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8                                                     = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8                                                   = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16                                                    = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16                                                  = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32                                                    = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32                                                  = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8                                                   = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8                                                 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16                                                  = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16                                                = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32                                                  = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32                                                = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape                                            = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize                                            = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute                                             = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig                                           = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate                                                  = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor                                           = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy                                                 = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate                                                = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor                                         = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate                                                 = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy                                                = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray                                               = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress                                             = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress                                           = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D                                           = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D                                         = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat                                              = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode                                         = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode                                          = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags                                               = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress                                             = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress                                           = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray                                               = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode                                         = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode                                          = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat                                              = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags                                               = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize                                                 = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti                                                    = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf                                                    = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv                                                    = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef                                               = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch                                                       = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid                                                   = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync                                              = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate                                                  = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord                                                  = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery                                                   = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize                                             = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy                                                 = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime                                             = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate                                                 = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery                                                  = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize                                            = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy                                                = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource                                   = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray                            = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer                             = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer                           = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags                                  = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources                                         = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources                                       = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable                                               = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit                                                  = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit                                                  = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice                                               = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate                                               = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource                                = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource                                        = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource                                      = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources                                            = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources                                          = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags                                     = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray                                  = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer                                = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize                                   = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch                                  = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions                            = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice                                               = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate                                               = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource                                = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice                                                = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate                                                = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource                                 = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice                                        = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource                                         = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource                                       = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources                                             = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources                                           = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags                                      = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions                             = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray                                   = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer                                 = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize                                    = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch                                   = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin                                                    = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End                                                      = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer                                     = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer                                          = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer                                        = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer                                   = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate                                                  = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer                                     = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage                                      = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice                                                 = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit                                                       = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject                                       = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject                                            = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject                                          = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject                                     = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags                                    = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync                                       = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync                                     = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice                                               = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate                                               = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface                            = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface                           = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef                                             = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate                                                = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy                                               = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat                                             = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray                                              = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat                                             = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray                                              = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem                                             = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer                              = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize                                 = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch                                = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions                          = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions                           = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer                               = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize                                  = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch                                 = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer                                        = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject                                          = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync                                     = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices                                              = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice                                       = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices                                              = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice                                       = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices                                               = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice                                        = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc                                               = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async                                                = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async                                              = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async                                               = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async                                             = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async                                               = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async                                             = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async                                              = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async                                            = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async                                             = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async                                           = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async                                             = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async                                           = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate                                                = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor                                         = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate                                              = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor                                       = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D                                                   = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned                                          = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync                                              = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2                                                 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2                                            = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2                                            = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2                                             = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2                                               = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2                                            = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2                                           = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2                                                = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2                                                  = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2                                             = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2                                                   = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2                                        = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2                                   = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2                                                    = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2                                                  = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2                                                 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2                                                 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2                                                = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2                                               = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2                                               = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2                                          = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2                                        = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2                                          = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2                          = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2                                            = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2                             = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2                                = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2                               = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2                         = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2                          = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2                              = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2                                 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2                                = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2                                       = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2                                         = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2                                    = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2                                              = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2                                               = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2                                        = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2                                             = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2                                      = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2                                                = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2                                           = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2                                                = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2                                           = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2                                                = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2                                           = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2                                                = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2                                           = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2                                                = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2                                                = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2                                                = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2                                                  = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2                                         = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2                                             = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2                                                  = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2                                             = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2                                                = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2                                           = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2                                              = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent                                              = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion                                             = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice                                       = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice                                       = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig                                            = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig                                            = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister                                              = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister                                            = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent                                                = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent                                                = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy                                                       = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync                                                  = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel                                                 = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart                                                = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop                                                 = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute                                          = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize                                           = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer                                          = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess                                          = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess                                         = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister                                              = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister                                            = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer                                      = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer                                                   = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync                                              = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer                                                 = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync                                            = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2                                                = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2                                            = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2                                             = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2                                              = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2                                             = 326,
    CUPTI_DRIVER_TRACE_CBID_SIZE                                                           = 327,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT                                                      = 0x7fffffff
} CUpti_driver_api_trace_cbid; 
 
 */

//const int LWM2_CUDA_FUNCTION_COUNT = 175;

/*

typedef enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID                                                       = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020                                    = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020                                   = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020                                      = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020                                 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020                                        = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020                                      = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020                                   = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020                                       = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020                                       = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020                                        = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020                                     = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020                                      = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020                                              = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020                                  = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020                                   = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020                                           = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020                                           = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020                                     = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020                                      = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020                                              = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020                                         = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020                                                = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020                                         = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020                                           = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020                                          = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020                                            = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020                                           = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020                                = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020                                        = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020                                          = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020                                              = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020                                            = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020                                       = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020                                     = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020                                     = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020                                   = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020                                  = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020                                = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020                                      = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020                                    = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020                                         = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020                                  = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020                                = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020                                       = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020                                = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020                              = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020                                 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020                               = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020                                              = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020                                            = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020                                         = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020                                       = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020                                    = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020                                       = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020                                         = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020                                       = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020                                  = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020                                       = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020                           = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020                                 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020                                  = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020                                 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020                                       = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020                              = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020                                   = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020                                 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020                            = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020                           = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020                              = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020                            = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020                                        = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020                             = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020                            = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020                          = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020                         = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020                                = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020                              = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020                    = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020                   = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020                                      = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020                                 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020                   = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020                  = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020                                      = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020                                     = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020                              = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020                       = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020                                      = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020                                     = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020                              = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020                       = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020                               = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020                             = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020                                   = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020                                 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020                            = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020                   = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020                         = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020                       = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020                          = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020                         = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020                                       = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020                                      = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020                               = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020                               = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020                        = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020                                = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020                              = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020                                    = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020                                  = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020                             = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020                    = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020                          = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020                        = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020                           = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020                          = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020                                           = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020                                             = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020                            = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020                          = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020                                 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020                               = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020                                          = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020                                  = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020                                    = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020                                   = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020                                      = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020                                      = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020                                        = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020                                       = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020                                   = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020                                         = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020                                         = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020                                = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020                                         = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020                                        = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020                                    = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020                                          = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020                                    = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020                                            = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020                                       = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020                                            = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020                                       = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020                                            = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020                                       = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020                                = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020                                     = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020                              = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020                              = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020                                = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000                                = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000                                        = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000                                      = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000                                 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000                              = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000                             = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000                                        = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000                                      = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000                                = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000                                          = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000                                     = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000                                        = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000                                   = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020                                         = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020                                   = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020                                      = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020                                      = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020                                = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020                                = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000                                  = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000                                       = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000                                        = 172,
    CUPTI_RUNTIME_TRACE_CBID_SIZE                                                          = 173,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT                                                     = 0x7fffffff
} CUpti_runtime_api_trace_cbid;
  
 */
 
 
 char* lwm2_cuda_function_names[] = {
	"INVALID",
    "cudaDriverGetVersion",
    "cudaRuntimeGetVersion",
    "cudaGetDeviceCount",
    "cudaGetDeviceProperties",
    "cudaChooseDevice",
    "cudaGetChannelDesc",
    "cudaCreateChannelDesc",
    "cudaConfigureCall",
    "cudaSetupArgument",
    "cudaGetLastError",
    "cudaPeekAtLastError",
    "cudaGetErrorString",
    "cudaLaunch",
    "cudaFuncSetCacheConfig",
    "cudaFuncGetAttributes",
    "cudaSetDevice",
    "cudaGetDevice",
    "cudaSetValidDevices",
    "cudaSetDeviceFlags",
    "cudaMalloc",
    "cudaMallocPitch",
    "cudaFree",
    "cudaMallocArray",
    "cudaFreeArray",
    "cudaMallocHost",
    "cudaFreeHost",
    "cudaHostAlloc",
    "cudaHostGetDevicePointer",
    "cudaHostGetFlags",
    "cudaMemGetInfo",
    "cudaMemcpy",
    "cudaMemcpy2D",
    "cudaMemcpyToArray",
    "cudaMemcpy2DToArray",
    "cudaMemcpyFromArray",
    "cudaMemcpy2DFromArray",
    "cudaMemcpyArrayToArray",
    "cudaMemcpy2DArrayToArray",
    "cudaMemcpyToSymbol",
    "cudaMemcpyFromSymbol",
    "cudaMemcpyAsync",
    "cudaMemcpyToArrayAsync",
    "cudaMemcpyFromArrayAsync",
    "cudaMemcpy2DAsync",
    "cudaMemcpy2DToArrayAsync",
    "cudaMemcpy2DFromArrayAsync",
    "cudaMemcpyToSymbolAsync",
    "cudaMemcpyFromSymbolAsync",
    "cudaMemset",
    "cudaMemset2D",
    "cudaMemsetAsync",
    "cudaMemset2DAsync",
    "cudaGetSymbolAddress",
    "cudaGetSymbolSize",
    "cudaBindTexture",
    "cudaBindTexture2D",
    "cudaBindTextureToArray",
    "cudaUnbindTexture",
    "cudaGetTextureAlignmentOffset",
    "cudaGetTextureReference",
    "cudaBindSurfaceToArray",
    "cudaGetSurfaceReference",
    "cudaGLSetGLDevice",
    "cudaGLRegisterBufferObject",
    "cudaGLMapBufferObject",
    "cudaGLUnmapBufferObject",
    "cudaGLUnregisterBufferObject",
    "cudaGLSetBufferObjectMapFlags",
    "cudaGLMapBufferObjectAsync",
    "cudaGLUnmapBufferObjectAsync",
    "cudaWGLGetDevice",
    "cudaGraphicsGLRegisterImage",
    "cudaGraphicsGLRegisterBuffer",
    "cudaGraphicsUnregisterResource",
    "cudaGraphicsResourceSetMapFlags",
    "cudaGraphicsMapResources",
    "cudaGraphicsUnmapResources",
    "cudaGraphicsResourceGetMappedPointer",
    "cudaGraphicsSubResourceGetMappedArray",
    "cudaVDPAUGetDevice",
    "cudaVDPAUSetVDPAUDevice",
    "cudaGraphicsVDPAURegisterVideoSurface",
    "cudaGraphicsVDPAURegisterOutputSurface",
    "cudaD3D11GetDevice",
    "cudaD3D11GetDevices",
    "cudaD3D11SetDirect3DDevice",
    "cudaGraphicsD3D11RegisterResource",
    "cudaD3D10GetDevice",
    "cudaD3D10GetDevices",
    "cudaD3D10SetDirect3DDevice",
    "cudaGraphicsD3D10RegisterResource",
    "cudaD3D10RegisterResource",
    "cudaD3D10UnregisterResource",
    "cudaD3D10MapResources",
    "cudaD3D10UnmapResources",
    "cudaD3D10ResourceSetMapFlags",
    "cudaD3D10ResourceGetSurfaceDimensions",
    "cudaD3D10ResourceGetMappedArray",
    "cudaD3D10ResourceGetMappedPointer",
    "cudaD3D10ResourceGetMappedSize",
    "cudaD3D10ResourceGetMappedPitch",
    "cudaD3D9GetDevice",
    "cudaD3D9GetDevices",
    "cudaD3D9SetDirect3DDevice",
    "cudaD3D9GetDirect3DDevice",
    "cudaGraphicsD3D9RegisterResource",
    "cudaD3D9RegisterResource",
    "cudaD3D9UnregisterResource",
    "cudaD3D9MapResources",
    "cudaD3D9UnmapResources",
    "cudaD3D9ResourceSetMapFlags",
    "cudaD3D9ResourceGetSurfaceDimensions",
    "cudaD3D9ResourceGetMappedArray",
    "cudaD3D9ResourceGetMappedPointer",
    "cudaD3D9ResourceGetMappedSize",
    "cudaD3D9ResourceGetMappedPitch",
    "cudaD3D9Begin",
    "cudaD3D9End",
    "cudaD3D9RegisterVertexBuffer",
    "cudaD3D9UnregisterVertexBuffer",
    "cudaD3D9MapVertexBuffer",
    "cudaD3D9UnmapVertexBuffer",
    "cudaThreadExit",
    "cudaSetDoubleForDevice",
    "cudaSetDoubleForHost",
    "cudaThreadSynchronize",
    "cudaThreadGetLimit",
    "cudaThreadSetLimit",
    "cudaStreamCreate",
    "cudaStreamDestroy",
    "cudaStreamSynchronize",
    "cudaStreamQuery",
    "cudaEventCreate",
    "cudaEventCreateWithFlags",
    "cudaEventRecord",
    "cudaEventDestroy",
    "cudaEventSynchronize",
    "cudaEventQuery",
    "cudaEventElapsedTime",
    "cudaMalloc3D",
    "cudaMalloc3DArray",
    "cudaMemset3D",
    "cudaMemset3DAsync",
    "cudaMemcpy3D",
    "cudaMemcpy3DAsync",
    "cudaThreadSetCacheConfig",
    "cudaStreamWaitEvent",
    "cudaD3D11GetDirect3DDevice",
    "cudaD3D10GetDirect3DDevice",
    "cudaThreadGetCacheConfig",
    "cudaPointerGetAttributes",
    "cudaHostRegister",
    "cudaHostUnregister",
    "cudaDeviceCanAccessPeer",
    "cudaDeviceEnablePeerAccess",
    "cudaDeviceDisablePeerAccess",
    "cudaPeerRegister",
    "cudaPeerUnregister",
    "cudaPeerGetDevicePointer",
    "cudaMemcpyPeer",
    "cudaMemcpyPeerAsync",
    "cudaMemcpy3DPeer",
    "cudaMemcpy3DPeerAsync",
    "cudaDeviceReset",
    "cudaDeviceSynchronize",
    "cudaDeviceGetLimit",
    "cudaDeviceSetLimit",
    "cudaDeviceGetCacheConfig",
    "cudaDeviceSetCacheConfig",
    "cudaProfilerInitialize",
    "cudaProfilerStart",
    "cudaProfilerStop",
    "SIZE",
    "FORCE_INT"
};

 char* lwm2_cuda_event_instruction[] = {
		 "instructions",
		 "inst_executed",
		 "divergent_branch"
 };



CUpti_EventID lwm2_cuda_event_ids[LWM2_CUDA_EVENT_COUNT];

//cuda_device_t cuda_devices[LWM2_CUDA_DEV_COUNT];
int lwm2_cuda_device_count;

int lwm2_cupti_init()
{
	CUresult err;
	CUptiResult cuptierr;
	CUpti_SubscriberHandle subscriber;
	CUpti_SubscriberHandle ctx_subscriber;

	CUpti_SubscriberHandle init_subscriber;
	cupti_eventData cuptiEvent;
	
	CUdevice dev;
	 int deviceNum;
	 int deviceCount;
	 char deviceName[32];

	
	char *eventName;
	
	/*
		
	cuptierr = cuptiSubscribe(&init_subscriber, (CUpti_CallbackFunc)handle_cuInit , NULL);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiSubscribe");

	cuptierr = cuptiEnableCallback(1, init_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuInit);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	
	cuptierr = cuptiSubscribe(&ctx_subscriber, (CUpti_CallbackFunc)lwm2_cuda_handle_context , NULL);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiSubscribe");

	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxCreate);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	
	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	
	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxAttach);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");

	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxDetach);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	
	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	
	cuptierr = cuptiEnableCallback(1, ctx_subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableCallback");
	*/
	cuptierr = cuptiSubscribe(&subscriber, (CUpti_CallbackFunc)getTimestampCallback , NULL);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiSubscribe");

	cuptierr = cuptiEnableDomain(1, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEnableDomain");
	
	lwm2_cupti_app = 1;
	
	return 1;
}

char* lwm2_get_event_for_device(int index, CUdevice device)
{
	CUresult err;
	int computeCapabilityMajor=0;
	int computeCapabilityMinor=0;

	err = cuDeviceComputeCapability( &computeCapabilityMajor,
														&computeCapabilityMinor,
														device);
	
	CHECK_CU_ERROR(err, "cuDeviceComputeCapability");
	
	if(index == LWM2_ADAPTER_CUDA_DIV_BRANCH)
	{
		return lwm2_cuda_event_instruction[2];
	}
	
	if(index == LWM2_ADAPTER_CUDA_INSTRUCTION)
	{
		if(computeCapabilityMajor == 1)
		{
			return lwm2_cuda_event_instruction[0];
		}

		if(computeCapabilityMajor == 2)
		{
			return lwm2_cuda_event_instruction[1];
		}
	}
	
	return NULL;
}

void CUPTIAPI handle_cuInit(void *userdata, CUpti_CallbackDomain domain,
                      CUpti_CallbackId cbid, const CUpti_CallbackData *cbInfo)
{
	CUresult err;
	CUptiResult cuptierr;
	CUpti_EventID eventId;
	
	lwm2_cuda_function_call = 1;
	
	if(cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		err = cuDeviceGetCount(&lwm2_cuda_device_count);
		CHECK_CU_ERROR(err, "cuDeviceGetCount");

		int i = 0,j;		

		for(j = 0; j < LWM2_CUDA_EVENT_COUNT; j++)
		{
			cuptierr = cuptiEventGetIdFromName(j, lwm2_get_event_for_device(j, i), &eventId);
			CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGetIdFromName");
			
			lwm2_cuda_event_ids[j] = eventId;
		}		
	}
}

void cuda_register_mem_cpy(size_t count, enum cudaMemcpyKind kind)
{
	if(kind == cudaMemcpyHostToDevice)
	{
		lwm2_storage_add_cuda_bytes_to_device(count);
		lwm2_storage_increment_msgs_to_device();
	}
	
	if(kind == cudaMemcpyDeviceToHost)
	{
		lwm2_storage_add_cuda_bytes_to_host(count);
		lwm2_storage_increment_msgs_to_host();
	}
}

void cuda_handle_mem_cpy_entry(CUpti_CallbackId cbid, const CUpti_CallbackData *cbInfo)
{
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020)
	{
		/*
		cudaMemcpy_v3020_params* func_params = (cudaMemcpy_v3020_params*) cbInfo->functionParams;
		size_t count = func_params->count;
		enum cudaMemcpyKind kind = func_params->kind;
		
		cuda_register_mem_cpy(count, kind);
		*/
		
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpy_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020)
	{
		/*
		cudaMemcpy2D_v3020_params* func_params = (cudaMemcpy2D_v3020_params*) cbInfo->functionParams;
		size_t width = func_params->width;
		size_t height = func_params->height;
		enum cudaMemcpyKind kind = func_params->kind;
		
		cuda_register_mem_cpy(width * height, kind);
		*/
		
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2D_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyToArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DToArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyFromArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DFromArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyArrayToArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DArrayToArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyToSymbol_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyFromSymbol_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyToArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyFromArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DToArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_ENTRY(cudaMemcpy2DFromArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyToSymbolAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_ENTRY(cudaMemcpyFromSymbolAsync_v3020_params);
	}
	    	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020)
	{
		
		/*cudaMemcpy3D_v3020_params* func_params = (cudaMemcpy3D_v3020_params*) cbInfo->functionParams;
		const struct cudaMemcpy3DParms* params = func_params->p;
		
		size_t width = params->extent.width;
		size_t height = params->extent.height;
		size_t depth = params->extent.depth;
		
		enum cudaMemcpyKind kind = params->kind;
		
		cuda_register_mem_cpy(width * height * depth, kind);
		*/
		
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_ENTRY(cudaMemcpy3D_v3020_params);
	}
	        
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_ENTRY(cudaMemcpy3DAsync_v3020_params);
	}
	        
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000
	
}
void cuda_handle_mem_cpy(CUpti_CallbackId cbid, const CUpti_CallbackData *cbInfo)
{
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020)
	{
		/*
		cudaMemcpy_v3020_params* func_params = (cudaMemcpy_v3020_params*) cbInfo->functionParams;
		size_t count = func_params->count;
		enum cudaMemcpyKind kind = func_params->kind;
		
		cuda_register_mem_cpy(count, kind);
		*/
		
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpy_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpy_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020)
	{
		/*
		cudaMemcpy2D_v3020_params* func_params = (cudaMemcpy2D_v3020_params*) cbInfo->functionParams;
		size_t width = func_params->width;
		size_t height = func_params->height;
		enum cudaMemcpyKind kind = func_params->kind;
		
		cuda_register_mem_cpy(width * height, kind);
		*/
		
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2D_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2D_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyToArray_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyToArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DToArray_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DToArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyFromArray_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyFromArray_v3020_params);
	}
	
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DFromArray_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DFromArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyArrayToArray_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyArrayToArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DArrayToArray_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DArrayToArray_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyToSymbol_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyToSymbol_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyFromSymbol_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyFromSymbol_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyAsync_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyToArrayAsync_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyToArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyFromArrayAsync_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyFromArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DAsync_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DToArrayAsync_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DToArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_EXIT(cudaMemcpy2DFromArrayAsync_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT(cudaMemcpy2DFromArrayAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyToSymbolAsync_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyToSymbolAsync_v3020_params);
	}
	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020)
	{
		CUDA_MEM_CPY_COUNT_EXIT(cudaMemcpyFromSymbolAsync_v3020_params);
		CUDA_MEM_CPY_COUNT(cudaMemcpyFromSymbolAsync_v3020_params);
	}
	    	    
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020)
	{
		
		/*cudaMemcpy3D_v3020_params* func_params = (cudaMemcpy3D_v3020_params*) cbInfo->functionParams;
		const struct cudaMemcpy3DParms* params = func_params->p;
		
		size_t width = params->extent.width;
		size_t height = params->extent.height;
		size_t depth = params->extent.depth;
		
		enum cudaMemcpyKind kind = params->kind;
		
		cuda_register_mem_cpy(width * height * depth, kind);
		*/
		
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_EXIT(cudaMemcpy3D_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH(cudaMemcpy3D_v3020_params);
	}
	        
	if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020)
	{
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH_EXIT(cudaMemcpy3DAsync_v3020_params);
		CUDA_MEM_CPY_WIDTH_HEIGHT_DEPTH(cudaMemcpy3DAsync_v3020_params);
	}
	        
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000
//	CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000
	
}

void CUPTIAPI getTimestampCallback(void *userdata, CUpti_CallbackDomain domain,
                     CUpti_CallbackId cbid, const CUpti_CallbackData *cbInfo)
{
	lwm2_cuda_function_call = 1;

	if(cbInfo->callbackSite == CUPTI_API_ENTER)
	{ 
//				lwm2_cuda_active_function++;
		lwm2_storage_increment_flag_cuda();
	}
	
	if(cbInfo->callbackSite == CUPTI_API_EXIT)
	{
//		lwm2_cuda_active_function--;
		lwm2_storage_decrement_flag_cuda();
	}
	
	if(cbInfo->callbackSite == CUPTI_API_ENTER)
	{
		if(cbid >= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 && cbid <= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020)
		{
			cuda_handle_mem_cpy_entry(cbid, cbInfo);
		}
		
		if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 || cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020)
		{
			cuda_handle_mem_cpy_entry(cbid, cbInfo);
		}
		
		if(cbid >= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 && cbid <= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000)
		{
			cuda_handle_mem_cpy_entry(cbid, cbInfo);
		}
	}
	if(cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		if(cbid >= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 && cbid <= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020)
		{
			cuda_handle_mem_cpy(cbid, cbInfo);
		}
		
		if(cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 || cbid == CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020)
		{
			cuda_handle_mem_cpy(cbid, cbInfo);
		}
		
		if(cbid >= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 && cbid <= CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000)
		{
			cuda_handle_mem_cpy(cbid, cbInfo);
		}
	}
}


timestamp_t get_cuda_time(const CUpti_CallbackData *cbInfo, CUpti_CallbackId cbid)
{
	cuda_timestamp_t ntime;
	timestamp_t stime;
	CUptiResult cuptierr;
	cuptierr = cuptiDeviceGetTimestamp(cbInfo->context, &ntime);
	CHECK_CUPTI_ERROR_3(cuptierr, "cuptiDeviceGetTimestamp", cbid);
	if(cuptierr == CUPTI_ERROR_INVALID_CONTEXT)
	{
		ntime = 0;
	}
	stime = ntime * (double) 1e-9;
	return stime;
}

#endif


