/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h> /* sigatomic_t */

#include <mpi.h>
#include <sys/types.h>
#include <unistd.h>

#include <math.h>

#include "../lwm2-signal.h"

#include "../util/util_time_measurement.h"
#include "../util/lwm2-log.h"

#include "lwm2-mpi-adapter.h"

#include "../lwm2-storage.h"
#include "../lwm2-system.h"
#include "../lwm2-signal.h"
#include "../lwm2-pulse-thread.h"
#include "../modules/output/lwm2-output-summary.h"
#include "../modules/output/lwm2-output-file.h"
#include "lwm2-pthread-adapter.h"


/* Indicates current active MPI function */
//volatile sig_atomic_t lwm2_mpi_active = 0;

//volatile sig_atomic_t lwm2_mpi_active_function = 0;
//
//volatile sig_atomic_t lwm2_mpi_active_p2p = 0;
//volatile sig_atomic_t lwm2_mpi_active_coll = 0;

int lwm2_mpi_rank = -1;
int lwm2_mpi_comm_size = 1;


int lwm2_mpi_finalized = 0;
int lwm2_mpi_app = 0;

timestamp_t lwm2_total_merge_time;

#ifdef MPI_DIRECT_SAMPLING
timestamp_t lwm2_mpi_start_time;
timestamp_t lwm2_mpi_end_time;
timestamp_t lwm2_mpi_total_time = 0;
timestamp_t lwm2_reduced_total_lwm2_time;
timestamp_t lwm2_reduced_total_mpi_time;
#endif

/**
 * Function called by adapter when MPI_Init is being called
 */
int lwm2_mpi_init()
{
   /* Make sure main module is initialized */
   lwm2_init();
   
   if(lwm2_sys_service_active != 0)
   {
	   return -1;
   }
   
   lwm2_proc_init();
   
   lwm2_mpi_app =  1;

   return LWM2_SUCCESS;
}

void lwm2_proc_init()
{
	PMPI_Comm_rank(MPI_COMM_WORLD, &lwm2_mpi_rank);
	PMPI_Comm_size(MPI_COMM_WORLD, &lwm2_mpi_comm_size);
	
//	pid_t pid = getpid();
//	pid_t ppid = getppid();
}


/**
 * Function called by MPI_Finalize
 */
void lwm2_mpi_finalize()
{
	if(lwm2_sys_service_active != 0)
	{
		return;
	}
	
#ifdef LWM2_PAPI_THREAD_ENABLED
	lwm2_adapter_pthread_read_hwc();
#endif
	
	lwm2_thread_lock_time_slice_update_lock();

	lwm2_end_pulse_thread();
	lwm2_signal_update_time_slice();
	
	lwm2_thread_unlock_time_slice_update_lock();
   
   lwm2_storage_merge_threads_sum_data();
   
   timestamp_t lwm2_merge_start_time = util_get_timestamp();
   
   lwm2_output_summary_merge_global();
   
   lwm2_output_file_write_file();
   
#ifdef MPI_DIRECT_SAMPLING
   lwm2_mpi_sum_mpi_time();
#endif

   timestamp_t lwm2_merge_end_time = util_get_timestamp();
   
   lwm2_total_merge_time = (lwm2_merge_end_time - lwm2_merge_start_time);
   
   lwm2_mpi_finalized = 1;
}

void lwm2_mpi_sum_mpi_time()
{
#ifdef MPI_DIRECT_SAMPLING
	
	timestamp_t total_time = util_get_timestamp() - lwm2_start_time;
	PMPI_Reduce(&total_time, &lwm2_reduced_total_lwm2_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&lwm2_mpi_total_time, &lwm2_reduced_total_mpi_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
#endif
}

