/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <pthread.h>
#include <limits.h>
#include <unistd.h>
#include <signal.h>

#include "lwm2-pthread-adapter.h"
#include "pthread/lwm2-pthreadwrap.h"

#include "lwm2-mpi-adapter.h"

#include "../util/lwm2-log.h"

#include "../modules/measurements/lwm2-measurement-omp.h"
#include "../modules/measurements/lwm2-measurement-hwcounter.h"

#include "../lwm2-signal.h"

#include "../lwm2-memory.h"

#include "lwm2-omp-adapter.h"

#include "../lwm2-storage.h"
#include "pthread/lwm2-pthreadwrap.h"

#define LWM2_OMP_THREADS 16

pthread_t lwm2_openmp_threads[MAX_THREADS];
char lwm2_openmp_threads_type[MAX_THREADS];

int lwm2_omp_thread_count = 0;

pthread_key_t lwm2_pthread_data_key;

int lwm2_adapter_pthread_init()
{
#ifdef LWM2_MEASUREMENT_PTHREAD
   /* Create pthread Wrapper */
   lwm2_wrapper_pthread_init();
#endif

   
   pthread_key_create(&lwm2_pthread_data_key, NULL);
   
   pthread_rwlock_init(&lwm2_thread_storage_table_lock, NULL);

   return LWM2_SUCCESS;
}

void lwm2_adapter_pthread_destroy()
{
   
}

void lwm2_adapter_pthread_read_hwc()
{
#ifdef LWM2_PAPI_THREAD_ENABLED
	int i;
	unsigned long long* thread_storage;
	pthread_rwlock_t* thread_lock;
	pthread_t thread_id;
	
	union sigval sigh_data;

	for(i = 0; i < lwm2_omp_thread_count; i++)
	{
		thread_id = lwm2_openmp_threads[i];
		
		sigh_data.sival_int = i;
		pthread_sigqueue(thread_id, SIGALRM, sigh_data);
	}
#endif
}

void lwm2_adapter_pthread_custom_leave_pthread_create(pthread_t thread)
{
	if(lwm2_sys_service_active == 1)
	{
		return;
	}
	
	if(lwm2_omp_thread_count < MAX_THREADS)
	{
		pthread_rwlock_wrlock(&lwm2_thread_storage_table_lock);
		
		lwm2_openmp_threads[lwm2_omp_thread_count] = thread;
		
		int thread_index = lwm2_omp_thread_count;		
		
		lwm2_storage_init_thread_storage(thread_index);
		lwm2_omp_thread_count++;

		pthread_rwlock_unlock(&lwm2_thread_storage_table_lock);
		
#ifdef LWM2_PAPI_THREAD_ENABLED
		void* hwc_storage = lwm2_storage_get_thread_storage_pthread(thread_index);
		lwm2_hwc_papi_setup(hwc_storage);
#endif
				
//		pthread_cleanup_push(lwm2_adapter_pthread_custom_enter_pthread_exit, NULL);
	}
}

void lwm2_adapter_pthread_custom_enter_pthread_exit()
{
#ifdef LWM2_PAPI_THREAD_ENABLED
	void* phwc_storage = lwm2_storage_get_my_hwc_storage();
	if(phwc_storage != NULL)
	{
		lwm2_measurement_hwc_papi_read_reset_events((struct lwm2_hwc_storage*) phwc_storage);
		lwm2_storage_reset_my_hwc_storage();
	}
#endif
}

void lwm2_adapter_pthread_custom_enter_pthread_cancel()
{

}

int lwm2_adapter_pthread_is_thread_master()
{
	pthread_t thid = pthread_self();
	int i;
	for(i = 0; i <= lwm2_omp_thread_count; i++)
	{
		if(lwm2_openmp_threads[i] == thid)
		{
			return 0;
		}
	}
	return 1;
}


