/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpireg.h
 *
 * @brief Registration of MPI functions
 */

#ifndef _LWM2_MPIREG_H
#define _LWM2_MPIREG_H

#ifdef __cplusplus
#   define EXTERN extern "C" /**< External C definition*/
#else
#   define EXTERN extern     /**< External definition */
#endif

/******************************************************************************
**
**  EPIK Library (Event Processing Interface Kit)
**
**  - Registration of MPI functions
**
******************************************************************************/

/** function type is not point-to-point */
#define LWM2_MPI_TYPE__NONE                          0
/** function type is receive operation */
#define LWM2_MPI_TYPE__RECV                          1
/** function type is send operation */
#define LWM2_MPI_TYPE__SEND                          2
/** function type is send and receive operation */
#define LWM2_MPI_TYPE__SENDRECV                      3
/** function type is collective */
#define LWM2_MPI_TYPE__COLLECTIVE                    4

/** function has is unknown communication pattern */
#define LWM2_COLL_TYPE__UNKNOWN                      1
/** function is barrier-like operation */
#define LWM2_COLL_TYPE__BARRIER                      2
/** function has 1:n communication pattern */
#define LWM2_COLL_TYPE__ONE2ALL                      3
/** function has n:1 communication pattern */
#define LWM2_COLL_TYPE__ALL2ONE                      4
/** function has n:n communication pattern */
#define LWM2_COLL_TYPE__ALL2ALL                      5
/** function may be partially synchronizing */
#define LWM2_COLL_TYPE__PARTIAL                      6
/** function is implicitely synchronizing */
#define LWM2_COLL_TYPE__IMPLIED                      7

/**
 * Bitpatterns for runtime wrapper enabling
 */
enum epk_mpi_groups
{
  /* pure groups, which can be specified in conf */
  LWM2_MPI_GROUP_NULL	=		0,
  LWM2_MPI_GROUP_CG        =     1,
  LWM2_MPI_GROUP_COLL      =     2,
  LWM2_MPI_GROUP_ENV       =     4,
  LWM2_MPI_GROUP_ERR       =     8,
  LWM2_MPI_GROUP_EXT       =    16,
  LWM2_MPI_GROUP_IO        =    32,
  LWM2_MPI_GROUP_MISC      =    64,
  LWM2_MPI_GROUP_P2P       =   128,
  LWM2_MPI_GROUP_RMA       =   256,
  LWM2_MPI_GROUP_SPAWN     =   512,
  LWM2_MPI_GROUP_TOPO      =  1024,
  LWM2_MPI_GROUP_TYPE      =  2048,
  LWM2_MPI_GROUP_PERF      =  4096,
  LWM2_MPI_GROUP_XNONBLOCK =  8192,
  /* derived groups, which are a combination of existing groups */
  LWM2_MPI_GROUP_CG_ERR    =  LWM2_MPI_GROUP_CG    | LWM2_MPI_GROUP_ERR,
  LWM2_MPI_GROUP_CG_EXT    =  LWM2_MPI_GROUP_CG    | LWM2_MPI_GROUP_EXT,
  LWM2_MPI_GROUP_CG_MISC   =  LWM2_MPI_GROUP_CG    | LWM2_MPI_GROUP_MISC,
  LWM2_MPI_GROUP_IO_ERR    =  LWM2_MPI_GROUP_IO    | LWM2_MPI_GROUP_ERR,
  LWM2_MPI_GROUP_IO_MISC   =  LWM2_MPI_GROUP_IO    | LWM2_MPI_GROUP_MISC,
  LWM2_MPI_GROUP_RMA_ERR   =  LWM2_MPI_GROUP_RMA   | LWM2_MPI_GROUP_ERR,
  LWM2_MPI_GROUP_RMA_EXT   =  LWM2_MPI_GROUP_RMA   | LWM2_MPI_GROUP_EXT,
  LWM2_MPI_GROUP_RMA_MISC  =  LWM2_MPI_GROUP_RMA   | LWM2_MPI_GROUP_MISC,
  LWM2_MPI_GROUP_TYPE_EXT  =  LWM2_MPI_GROUP_TYPE  | LWM2_MPI_GROUP_EXT,
  LWM2_MPI_GROUP_TYPE_MISC =  LWM2_MPI_GROUP_TYPE  | LWM2_MPI_GROUP_MISC,
  /* NOTE: ALL should comprise all pure groups */
  LWM2_MPI_GROUP_ALL       =  LWM2_MPI_GROUP_CG    |
                             LWM2_MPI_GROUP_COLL  |
                             LWM2_MPI_GROUP_ENV   |
                             LWM2_MPI_GROUP_ERR   |
                             LWM2_MPI_GROUP_EXT   |
                             LWM2_MPI_GROUP_IO    |
                             LWM2_MPI_GROUP_MISC  |
                             LWM2_MPI_GROUP_P2P   |
                             LWM2_MPI_GROUP_PERF  |
                             LWM2_MPI_GROUP_RMA   |
                             LWM2_MPI_GROUP_SPAWN |
                             LWM2_MPI_GROUP_TOPO  |
                             LWM2_MPI_GROUP_TYPE  |
                             LWM2_MPI_GROUP_XNONBLOCK,
  /* NOTE: DEFAULT should reflect the default set in 'epk_conf.c' */
  LWM2_MPI_GROUP_DEFAULT =  LWM2_MPI_GROUP_CG    |
                           LWM2_MPI_GROUP_COLL  |
                           LWM2_MPI_GROUP_ENV   |
                           LWM2_MPI_GROUP_IO    |
                           LWM2_MPI_GROUP_P2P   |
                           LWM2_MPI_GROUP_RMA   |
                           LWM2_MPI_GROUP_TOPO
};

#define LWM2_MPI_FUNCCOUNT (310)
#define LWM2_MPI_GROUPCOUNT (14)

/* Flags to mark currently active regions */
extern int lwm2_mpi_region[LWM2_MPI_FUNCCOUNT];

/* Matching region name */
extern char* lwm2_mpi_region_names[LWM2_MPI_FUNCCOUNT + 1];

#define LWM2_MPI_ABORT                              0
#define LWM2_MPI_ACCUMULATE                         1
#define LWM2_MPI_ADD_ERROR_CLASS                    2
#define LWM2_MPI_ADD_ERROR_CODE                     3
#define LWM2_MPI_ADD_ERROR_STRING                   4
#define LWM2_MPI_ADDRESS                            5
#define LWM2_MPI_ALLGATHER                          6
#define LWM2_MPI_ALLGATHERV                         7
#define LWM2_MPI_ALLOC_MEM                          8
#define LWM2_MPI_ALLREDUCE                          9
#define LWM2_MPI_ALLTOALL                          10
#define LWM2_MPI_ALLTOALLV                         11
#define LWM2_MPI_ALLTOALLW                         12
#define LWM2_MPI_ATTR_DELETE                       13
#define LWM2_MPI_ATTR_GET                          14
#define LWM2_MPI_ATTR_PUT                          15
#define LWM2_MPI_BARRIER                           16
#define LWM2_MPI_BCAST                             17
#define LWM2_MPI_BSEND                             18
#define LWM2_MPI_BSEND_INIT                        19
#define LWM2_MPI_BUFFER_ATTACH                     20
#define LWM2_MPI_BUFFER_DETACH                     21
#define LWM2_MPI_CANCEL                            22
#define LWM2_MPI_CART_COORDS                       23
#define LWM2_MPI_CART_CREATE                       24
#define LWM2_MPI_CART_GET                          25
#define LWM2_MPI_CART_MAP                          26
#define LWM2_MPI_CART_RANK                         27
#define LWM2_MPI_CART_SHIFT                        28
#define LWM2_MPI_CART_SUB                          29
#define LWM2_MPI_CARTDIM_GET                       30
#define LWM2_MPI_CLOSE_PORT                        31
#define LWM2_MPI_COMM_ACCEPT                       32
#define LWM2_MPI_COMM_C2F                          33
#define LWM2_MPI_COMM_CALL_ERRHANDLER              34
#define LWM2_MPI_COMM_COMPARE                      35
#define LWM2_MPI_COMM_CONNECT                      36
#define LWM2_MPI_COMM_CREATE                       37
#define LWM2_MPI_COMM_CREATE_ERRHANDLER            38
#define LWM2_MPI_COMM_CREATE_KEYVAL                39
#define LWM2_MPI_COMM_DELETE_ATTR                  40
#define LWM2_MPI_COMM_DISCONNECT                   41
#define LWM2_MPI_COMM_DUP                          42
#define LWM2_MPI_COMM_F2C                          43
#define LWM2_MPI_COMM_FREE                         44
#define LWM2_MPI_COMM_FREE_KEYVAL                  45
#define LWM2_MPI_COMM_GET_ATTR                     46
#define LWM2_MPI_COMM_GET_ERRHANDLER               47
#define LWM2_MPI_COMM_GET_NAME                     48
#define LWM2_MPI_COMM_GET_PARENT                   49
#define LWM2_MPI_COMM_GROUP                        50
#define LWM2_MPI_COMM_JOIN                         51
#define LWM2_MPI_COMM_RANK                         52
#define LWM2_MPI_COMM_REMOTE_GROUP                 53
#define LWM2_MPI_COMM_REMOTE_SIZE                  54
#define LWM2_MPI_COMM_SET_ATTR                     55
#define LWM2_MPI_COMM_SET_ERRHANDLER               56
#define LWM2_MPI_COMM_SET_NAME                     57
#define LWM2_MPI_COMM_SIZE                         58
#define LWM2_MPI_COMM_SPAWN                        59
#define LWM2_MPI_COMM_SPAWN_MULTIPLE               60
#define LWM2_MPI_COMM_SPLIT                        61
#define LWM2_MPI_COMM_TEST_INTER                   62
#define LWM2_MPI_DIMS_CREATE                       63
#define LWM2_MPI_DIST_GRAPH_CREATE                 64
#define LWM2_MPI_DIST_GRAPH_CREATE_ADJACENT        65
#define LWM2_MPI_DIST_GRAPH_NEIGHBORS              66
#define LWM2_MPI_DIST_GRAPH_NEIGHBORS_COUNT        67
#define LWM2_MPI_ERRHANDLER_CREATE                 68
#define LWM2_MPI_ERRHANDLER_FREE                   69
#define LWM2_MPI_ERRHANDLER_GET                    70
#define LWM2_MPI_ERRHANDLER_SET                    71
#define LWM2_MPI_ERROR_CLASS                       72
#define LWM2_MPI_ERROR_STRING                      73
#define LWM2_MPI_EXSCAN                            74
#define LWM2_MPI_FILE_C2F                          75
#define LWM2_MPI_FILE_CALL_ERRHANDLER              76
#define LWM2_MPI_FILE_CLOSE                        77
#define LWM2_MPI_FILE_CREATE_ERRHANDLER            78
#define LWM2_MPI_FILE_DELETE                       79
#define LWM2_MPI_FILE_F2C                          80
#define LWM2_MPI_FILE_GET_AMODE                    81
#define LWM2_MPI_FILE_GET_ATOMICITY                82
#define LWM2_MPI_FILE_GET_BYTE_OFFSET              83
#define LWM2_MPI_FILE_GET_ERRHANDLER               84
#define LWM2_MPI_FILE_GET_GROUP                    85
#define LWM2_MPI_FILE_GET_INFO                     86
#define LWM2_MPI_FILE_GET_POSITION                 87
#define LWM2_MPI_FILE_GET_POSITION_SHARED          88
#define LWM2_MPI_FILE_GET_SIZE                     89
#define LWM2_MPI_FILE_GET_TYPE_EXTENT              90
#define LWM2_MPI_FILE_GET_VIEW                     91
#define LWM2_MPI_FILE_IREAD                        92
#define LWM2_MPI_FILE_IREAD_AT                     93
#define LWM2_MPI_FILE_IREAD_SHARED                 94
#define LWM2_MPI_FILE_IWRITE                       95
#define LWM2_MPI_FILE_IWRITE_AT                    96
#define LWM2_MPI_FILE_IWRITE_SHARED                97
#define LWM2_MPI_FILE_OPEN                         98
#define LWM2_MPI_FILE_PREALLOCATE                  99
#define LWM2_MPI_FILE_READ                        100
#define LWM2_MPI_FILE_READ_ALL                    101
#define LWM2_MPI_FILE_READ_ALL_BEGIN              102
#define LWM2_MPI_FILE_READ_ALL_END                103
#define LWM2_MPI_FILE_READ_AT                     104
#define LWM2_MPI_FILE_READ_AT_ALL                 105
#define LWM2_MPI_FILE_READ_AT_ALL_BEGIN           106
#define LWM2_MPI_FILE_READ_AT_ALL_END             107
#define LWM2_MPI_FILE_READ_ORDERED                108
#define LWM2_MPI_FILE_READ_ORDERED_BEGIN          109
#define LWM2_MPI_FILE_READ_ORDERED_END            110
#define LWM2_MPI_FILE_READ_SHARED                 111
#define LWM2_MPI_FILE_SEEK                        112
#define LWM2_MPI_FILE_SEEK_SHARED                 113
#define LWM2_MPI_FILE_SET_ATOMICITY               114
#define LWM2_MPI_FILE_SET_ERRHANDLER              115
#define LWM2_MPI_FILE_SET_INFO                    116
#define LWM2_MPI_FILE_SET_SIZE                    117
#define LWM2_MPI_FILE_SET_VIEW                    118
#define LWM2_MPI_FILE_SYNC                        119
#define LWM2_MPI_FILE_WRITE                       120
#define LWM2_MPI_FILE_WRITE_ALL                   121
#define LWM2_MPI_FILE_WRITE_ALL_BEGIN             122
#define LWM2_MPI_FILE_WRITE_ALL_END               123
#define LWM2_MPI_FILE_WRITE_AT                    124
#define LWM2_MPI_FILE_WRITE_AT_ALL                125
#define LWM2_MPI_FILE_WRITE_AT_ALL_BEGIN          126
#define LWM2_MPI_FILE_WRITE_AT_ALL_END            127
#define LWM2_MPI_FILE_WRITE_ORDERED               128
#define LWM2_MPI_FILE_WRITE_ORDERED_BEGIN         129
#define LWM2_MPI_FILE_WRITE_ORDERED_END           130
#define LWM2_MPI_FILE_WRITE_SHARED                131
#define LWM2_MPI_FINALIZE                         132
#define LWM2_MPI_FINALIZED                        133
#define LWM2_MPI_FREE_MEM                         134
#define LWM2_MPI_GATHER                           135
#define LWM2_MPI_GATHERV                          136
#define LWM2_MPI_GET                              137
#define LWM2_MPI_GET_ADDRESS                      138
#define LWM2_MPI_GET_COUNT                        139
#define LWM2_MPI_GET_ELEMENTS                     140
#define LWM2_MPI_GET_PROCESSOR_NAME               141
#define LWM2_MPI_GET_VERSION                      142
#define LWM2_MPI_GRAPH_CREATE                     143
#define LWM2_MPI_GRAPH_GET                        144
#define LWM2_MPI_GRAPH_MAP                        145
#define LWM2_MPI_GRAPH_NEIGHBORS                  146
#define LWM2_MPI_GRAPH_NEIGHBORS_COUNT            147
#define LWM2_MPI_GRAPHDIMS_GET                    148
#define LWM2_MPI_GREQUEST_COMPLETE                149
#define LWM2_MPI_GREQUEST_START                   150
#define LWM2_MPI_GROUP_C2F                        151
#define LWM2_MPI_GROUP_COMPARE                    152
#define LWM2_MPI_GROUP_DIFFERENCE                 153
#define LWM2_MPI_GROUP_EXCL                       154
#define LWM2_MPI_GROUP_F2C                        155
#define LWM2_MPI_GROUP_FREE                       156
#define LWM2_MPI_GROUP_INCL                       157
#define LWM2_MPI_GROUP_INTERSECTION               158
#define LWM2_MPI_GROUP_RANGE_EXCL                 159
#define LWM2_MPI_GROUP_RANGE_INCL                 160
#define LWM2_MPI_GROUP_RANK                       161
#define LWM2_MPI_GROUP_SIZE                       162
#define LWM2_MPI_GROUP_TRANSLATE_RANKS            163
#define LWM2_MPI_GROUP_UNION                      164
#define LWM2_MPI_IBSEND                           165
#define LWM2_MPI_INFO_C2F                         166
#define LWM2_MPI_INFO_CREATE                      167
#define LWM2_MPI_INFO_DELETE                      168
#define LWM2_MPI_INFO_DUP                         169
#define LWM2_MPI_INFO_F2C                         170
#define LWM2_MPI_INFO_FREE                        171
#define LWM2_MPI_INFO_GET                         172
#define LWM2_MPI_INFO_GET_NKEYS                   173
#define LWM2_MPI_INFO_GET_NTHKEY                  174
#define LWM2_MPI_INFO_GET_VALUELEN                175
#define LWM2_MPI_INFO_SET                         176
#define LWM2_MPI_INIT                             177
#define LWM2_MPI_INIT_THREAD                      178
#define LWM2_MPI_INITIALIZED                      179
#define LWM2_MPI_INTERCOMM_CREATE                 180
#define LWM2_MPI_INTERCOMM_MERGE                  181
#define LWM2_MPI_IPROBE                           182
#define LWM2_MPI_IRECV                            183
#define LWM2_MPI_IRSEND                           184
#define LWM2_MPI_IS_THREAD_MAIN                   185
#define LWM2_MPI_ISEND                            186
#define LWM2_MPI_ISSEND                           187
#define LWM2_MPI_KEYVAL_CREATE                    188
#define LWM2_MPI_KEYVAL_FREE                      189
#define LWM2_MPI_LOOKUP_NAME                      190
#define LWM2_MPI_OP_C2F                           191
#define LWM2_MPI_OP_COMMUTATIVE                   192
#define LWM2_MPI_OP_CREATE                        193
#define LWM2_MPI_OP_F2C                           194
#define LWM2_MPI_OP_FREE                          195
#define LWM2_MPI_OPEN_PORT                        196
#define LWM2_MPI_PACK                             197
#define LWM2_MPI_PACK_EXTERNAL                    198
#define LWM2_MPI_PACK_EXTERNAL_SIZE               199
#define LWM2_MPI_PACK_SIZE                        200
#define LWM2_MPI_PCONTROL                         201
#define LWM2_MPI_PROBE                            202
#define LWM2_MPI_PUBLISH_NAME                     203
#define LWM2_MPI_PUT                              204
#define LWM2_MPI_QUERY_THREAD                     205
#define LWM2_MPI_RECV                             206
#define LWM2_MPI_RECV_INIT                        207
#define LWM2_MPI_REDUCE                           208
#define LWM2_MPI_REDUCE_LOCAL                     209
#define LWM2_MPI_REDUCE_SCATTER                   210
#define LWM2_MPI_REDUCE_SCATTER_BLOCK             211
#define LWM2_MPI_REGISTER_DATAREP                 212
#define LWM2_MPI_REQUEST_C2F                      213
#define LWM2_MPI_REQUEST_F2C                      214
#define LWM2_MPI_REQUEST_FREE                     215
#define LWM2_MPI_REQUEST_GET_STATUS               216
#define LWM2_MPI_RSEND                            217
#define LWM2_MPI_RSEND_INIT                       218
#define LWM2_MPI_SCAN                             219
#define LWM2_MPI_SCATTER                          220
#define LWM2_MPI_SCATTERV                         221
#define LWM2_MPI_SEND                             222
#define LWM2_MPI_SEND_INIT                        223
#define LWM2_MPI_SENDRECV                         224
#define LWM2_MPI_SENDRECV_REPLACE                 225
#define LWM2_MPI_SIZEOF                           226
#define LWM2_MPI_SSEND                            227
#define LWM2_MPI_SSEND_INIT                       228
#define LWM2_MPI_START                            229
#define LWM2_MPI_STARTALL                         230
#define LWM2_MPI_STATUS_C2F                       231
#define LWM2_MPI_STATUS_F2C                       232
#define LWM2_MPI_STATUS_SET_CANCELLED             233
#define LWM2_MPI_STATUS_SET_ELEMENTS              234
#define LWM2_MPI_TEST                             235
#define LWM2_MPI_TEST_CANCELLED                   236
#define LWM2_MPI_TESTALL                          237
#define LWM2_MPI_TESTANY                          238
#define LWM2_MPI_TESTSOME                         239
#define LWM2_MPI_TOPO_TEST                        240
#define LWM2_MPI_TYPE_C2F                         241
#define LWM2_MPI_TYPE_COMMIT                      242
#define LWM2_MPI_TYPE_CONTIGUOUS                  243
#define LWM2_MPI_TYPE_CREATE_DARRAY               244
#define LWM2_MPI_TYPE_CREATE_F90_COMPLEX          245
#define LWM2_MPI_TYPE_CREATE_F90_INTEGER          246
#define LWM2_MPI_TYPE_CREATE_F90_REAL             247
#define LWM2_MPI_TYPE_CREATE_HINDEXED             248
#define LWM2_MPI_TYPE_CREATE_HVECTOR              249
#define LWM2_MPI_TYPE_CREATE_INDEXED_BLOCK        250
#define LWM2_MPI_TYPE_CREATE_KEYVAL               251
#define LWM2_MPI_TYPE_CREATE_RESIZED              252
#define LWM2_MPI_TYPE_CREATE_STRUCT               253
#define LWM2_MPI_TYPE_CREATE_SUBARRAY             254
#define LWM2_MPI_TYPE_DELETE_ATTR                 255
#define LWM2_MPI_TYPE_DUP                         256
#define LWM2_MPI_TYPE_EXTENT                      257
#define LWM2_MPI_TYPE_F2C                         258
#define LWM2_MPI_TYPE_FREE                        259
#define LWM2_MPI_TYPE_FREE_KEYVAL                 260
#define LWM2_MPI_TYPE_GET_ATTR                    261
#define LWM2_MPI_TYPE_GET_CONTENTS                262
#define LWM2_MPI_TYPE_GET_ENVELOPE                263
#define LWM2_MPI_TYPE_GET_EXTENT                  264
#define LWM2_MPI_TYPE_GET_NAME                    265
#define LWM2_MPI_TYPE_GET_TRUE_EXTENT             266
#define LWM2_MPI_TYPE_HINDEXED                    267
#define LWM2_MPI_TYPE_HVECTOR                     268
#define LWM2_MPI_TYPE_INDEXED                     269
#define LWM2_MPI_TYPE_LB                          270
#define LWM2_MPI_TYPE_MATCH_SIZE                  271
#define LWM2_MPI_TYPE_SET_ATTR                    272
#define LWM2_MPI_TYPE_SET_NAME                    273
#define LWM2_MPI_TYPE_SIZE                        274
#define LWM2_MPI_TYPE_STRUCT                      275
#define LWM2_MPI_TYPE_UB                          276
#define LWM2_MPI_TYPE_VECTOR                      277
#define LWM2_MPI_UNPACK                           278
#define LWM2_MPI_UNPACK_EXTERNAL                  279
#define LWM2_MPI_UNPUBLISH_NAME                   280
#define LWM2_MPI_WAIT                             281
#define LWM2_MPI_WAITALL                          282
#define LWM2_MPI_WAITANY                          283
#define LWM2_MPI_WAITSOME                         284
#define LWM2_MPI_WIN_C2F                          285
#define LWM2_MPI_WIN_CALL_ERRHANDLER              286
#define LWM2_MPI_WIN_COMPLETE                     287
#define LWM2_MPI_WIN_CREATE                       288
#define LWM2_MPI_WIN_CREATE_ERRHANDLER            289
#define LWM2_MPI_WIN_CREATE_KEYVAL                290
#define LWM2_MPI_WIN_DELETE_ATTR                  291
#define LWM2_MPI_WIN_F2C                          292
#define LWM2_MPI_WIN_FENCE                        293
#define LWM2_MPI_WIN_FREE                         294
#define LWM2_MPI_WIN_FREE_KEYVAL                  295
#define LWM2_MPI_WIN_GET_ATTR                     296
#define LWM2_MPI_WIN_GET_ERRHANDLER               297
#define LWM2_MPI_WIN_GET_GROUP                    298
#define LWM2_MPI_WIN_GET_NAME                     299
#define LWM2_MPI_WIN_LOCK                         300
#define LWM2_MPI_WIN_POST                         301
#define LWM2_MPI_WIN_SET_ATTR                     302
#define LWM2_MPI_WIN_SET_ERRHANDLER               303
#define LWM2_MPI_WIN_SET_NAME                     304
#define LWM2_MPI_WIN_START                        305
#define LWM2_MPI_WIN_TEST                         306
#define LWM2_MPI_WIN_UNLOCK                       307
#define LWM2_MPI_WIN_WAIT                         308
#define LWM2_MPI_WTICK                            309
#define LWM2_MPI_WTIME                            310

/* TODO */
//#define IS_EVENT_GEN_ON_FOR(group) LWM2_MPI_GROUP_##group

/* Enable all */
#define IS_EVENT_GEN_ON_FOR(group) 1

#define ELG_MPI_STATUS_SIZE 5

#endif
