/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_coll.c
 *
 * @brief C interface wrappers for collective communication
 */

#include <mpi.h>

#include "../lwm2-mpi-config.h"
#include "lwm2_mpireg.h"

/**
 * @name C wrappers
 * @{
 */

#if defined(HAS_MPI_ALLGATHER) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Allgather
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Allgather(void*        sendbuf,
                  int          sendcount,
                  MPI_Datatype sendtype,
                  void*        recvbuf,
                  int          recvcount,
                  MPI_Datatype recvtype,
                  MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  recvsz, sendsz, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLGATHER,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(sendtype, &sendsz);
    _info.bytes_sent = sendsz * sendcount;
    
    PMPI_Type_size(recvtype, &recvsz);
    _info.bytes_recv = recvsz * recvcount;

    return_val = PMPI_Allgather(sendbuf,
                                sendcount,
                                sendtype,
                                recvbuf,
                                recvcount,
                                recvtype,
                                comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLGATHER],
                     root_loc, EPK_COMM_ID(comm),
             N * sendcount * sendsz,
             N * recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Allgather(sendbuf,
                                sendcount,
                                sendtype,
                                recvbuf,
                                recvcount,
                                recvtype,
                                comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_ALLGATHERV) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Allgatherv
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Allgatherv(void*        sendbuf,
                   int          sendcount,
                   MPI_Datatype sendtype,
                   void*        recvbuf,
                   int*         recvcounts,
                   int*         displs,
                   MPI_Datatype recvtype,
                   MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  recvcount, recvsz, sendsz, i, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLGATHERV,
                                                LWM2_MPI_GROUP_COLL);
    
    int comm_size;
    PMPI_Comm_size(comm, &comm_size);
    
    PMPI_Type_size(sendtype, &sendsz);
    _info.bytes_sent = sendsz * sendcount;

    PMPI_Type_size(recvtype, &recvsz);
    
    for(i = 0; i < comm_size; i++)
    {
    		_info.bytes_recv += recvsz * recvcounts[i];	
    }
    
    return_val = PMPI_Allgatherv(sendbuf,
                                 sendcount,
                                 sendtype,
                                 recvbuf,
                                 recvcounts,
                                 displs,
                                 recvtype,
                                 comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLGATHERV],
                     root_loc, EPK_COMM_ID(comm),
             N * sendcount * sendsz,
             recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Allgatherv(sendbuf,
                                 sendcount,
                                 sendtype,
                                 recvbuf,
                                 recvcounts,
                                 displs,
                                 recvtype,
                                 comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_ALLREDUCE) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Allreduce
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Allreduce(void*        sendbuf,
                  void*        recvbuf,
                  int          count,
                  MPI_Datatype datatype,
                  MPI_Op       op,
                  MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLREDUCE,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    _info.bytes_sent = sz * count;

    return_val = PMPI_Allreduce(sendbuf,
                                recvbuf,
                                count,
                                datatype,
                                op,
                                comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLREDUCE],
                     root_loc, EPK_COMM_ID(comm),
             N * count * sz,
             N * count * sz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Allreduce(sendbuf,
                                recvbuf,
                                count,
                                datatype,
                                op,
                                comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_ALLTOALL) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Alltoall
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Alltoall(void*        sendbuf,
                 int          sendcount,
                 MPI_Datatype sendtype,
                 void*        recvbuf,
                 int          recvcount,
                 MPI_Datatype recvtype,
                 MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  recvsz, sendsz, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLTOALL,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(sendtype, &sendsz);
    _info.bytes_sent = sendsz * sendcount;

    PMPI_Type_size(recvtype, &recvsz);
    _info.bytes_recv = recvsz * recvcount;

    return_val = PMPI_Alltoall(sendbuf,
                               sendcount,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLTOALL],
                     root_loc, EPK_COMM_ID(comm),
             N * sendcount * sendsz,
             N * recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Alltoall(sendbuf,
                               sendcount,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_ALLTOALLV) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Alltoallv
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Alltoallv(void*        sendbuf,
                  int*         sendcounts,
                  int*         sdispls,
                  MPI_Datatype sendtype,
                  void*        recvbuf,
                  int*         recvcounts,
                  int*         rdispls,
                  MPI_Datatype recvtype,
                  MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int recvcount = 0, sendcount = 0, recvsz, sendsz, N,
        i;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLTOALLV,
                                                LWM2_MPI_GROUP_COLL);
    
    int comm_size;
    PMPI_Comm_size(comm, &comm_size);

    PMPI_Type_size(sendtype, &sendsz);

    PMPI_Type_size(recvtype, &recvsz);

    for(i = 0; i < comm_size; i++)
    {
    		_info.bytes_sent += sendsz * sendcounts[i];
    		_info.bytes_recv += recvsz * recvcounts[i];	
    }

    return_val = PMPI_Alltoallv(sendbuf,
                                sendcounts,
                                sdispls,
                                sendtype,
                                recvbuf,
                                recvcounts,
                                rdispls,
                                recvtype,
                                comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLTOALLV],
                     root_loc, EPK_COMM_ID(comm),
             sendcount * sendsz,
             recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Alltoallv(sendbuf,
                                sendcounts,
                                sdispls,
                                sendtype,
                                recvbuf,
                                recvcounts,
                                rdispls,
                                recvtype,
                                comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_ALLTOALLW) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Alltoallw
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup coll
 */
int MPI_Alltoallw(void*        sendbuf,
                  int          sendcounts[],
                  int          sdispls[],
                  MPI_Datatype sendtypes[],
                  void*        recvbuf,
                  int          recvcounts[],
                  int          rdispls[],
                  MPI_Datatype recvtypes[],
                  MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int recvcount = 0, sendcount = 0, recvsz, sendsz, N,
        i;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ALLTOALLW,
                                                LWM2_MPI_GROUP_COLL);
    
    int comm_size;
    PMPI_Comm_size(comm, &comm_size);

    for(i = 0; i < comm_size; i++)
    {
    		PMPI_Type_size(sendtypes[i], &sendsz);
    		_info.bytes_sent += sendsz * sendcounts[i];
    	
    		PMPI_Type_size(recvtypes[i], &recvsz);
    		_info.bytes_recv += recvsz * recvcounts[i];	
    }

    return_val = PMPI_Alltoallw(sendbuf,
                                sendcounts,
                                sdispls,
                                sendtypes,
                                recvbuf,
                                recvcounts,
                                rdispls,
                                recvtypes,
                                comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_ALLTOALLW],
                     root_loc, EPK_COMM_ID(comm),
             sendcount,
             recvcount);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Alltoallw(sendbuf,
                                sendcounts,
                                sdispls,
                                sendtypes,
                                recvbuf,
                                recvcounts,
                                rdispls,
                                recvtypes,
                                comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_BARRIER) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Barrier
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Barrier(MPI_Comm comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_BARRIER,
                                                LWM2_MPI_GROUP_NULL);

    return_val = PMPI_Barrier(comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_BARRIER],
                     root_loc, EPK_COMM_ID(comm),
             0,
             0);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Barrier(comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_BCAST) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Bcast
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Bcast(void*        buffer,
              int          count,
              MPI_Datatype datatype,
              int          root,
              MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, N, me;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_BCAST,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    
    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);
    
    if(comm_rank == root)
    {
    		_info.bytes_sent = sz * count;
    }
    else
    {
    		_info.bytes_recv = sz * count;
    }

    return_val = PMPI_Bcast(buffer, count, datatype, root, comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_BCAST],
                     root_loc, EPK_COMM_ID(comm),
             N*count*sz,
             count*sz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Bcast(buffer, count, datatype, root, comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_EXSCAN) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Exscan
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup coll
 */
int MPI_Exscan(void*        sendbuf,
               void*        recvbuf,
               int          count,
               MPI_Datatype datatype,
               MPI_Op       op,
               MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, me, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_EXSCAN,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    _info.bytes_sent = sz * count;
    
    int comm_rank;
    
    PMPI_Comm_rank(comm, &comm_rank);
    
    if(comm_rank == 0)
    {
    		_info.bytes_recv = sz * count;
    }

    return_val = PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_EXSCAN],
                     root_loc, EPK_COMM_ID(comm),
             (N-me-1) * sz * count,
             me * sz * count);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_GATHER) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Gather
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Gather(void*        sendbuf,
               int          sendcount,
               MPI_Datatype sendtype,
               void*        recvbuf,
               int          recvcount,
               MPI_Datatype recvtype,
               int          root,
               MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sendsz, recvsz, N, me;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GATHER,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(sendtype, &sendsz);
    PMPI_Type_size(recvtype, &recvsz);
    
    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);
    
    if(root != comm_rank)
    {
    		_info.bytes_sent = sendsz * sendcount;
    }
    else
    {
    		int comm_size, i;
    		PMPI_Comm_size(comm, &comm_size);
    
    		for(i = 0; i < comm_size - 1; i++)
    		{
    			_info.bytes_recv += recvsz * recvcount;
    		}
    }

    return_val = PMPI_Gather(sendbuf,
                             sendcount,
                             sendtype,
                             recvbuf,
                             recvcount,
                             recvtype,
                             root,
                             comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_GATHER],
                     root_loc, EPK_COMM_ID(comm),
             sendcount*sendsz,
             N*recvcount*recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Gather(sendbuf,
                             sendcount,
                             sendtype,
                             recvbuf,
                             recvcount,
                             recvtype,
                             root,
                             comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_GATHERV) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Gatherv
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Gatherv(void*        sendbuf,
                int          sendcount,
                MPI_Datatype sendtype,
                void*        recvbuf,
                int*         recvcounts,
                int*         displs,
                MPI_Datatype recvtype,
                int          root,
                MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  recvsz, sendsz, recvcount = 0, me, N, i;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GATHERV,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(sendtype, &sendsz);
    PMPI_Type_size(recvtype, &recvsz);

    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);

    if(root != comm_rank)
    {
    		_info.bytes_sent = sendsz * sendcount;
    }
    else
    {
		int comm_size, i;
		PMPI_Comm_size(comm, &comm_size);

		for(i = 0; i < comm_size - 1; i++)
		{
			_info.bytes_recv += recvsz * recvcounts[i];
		}
    }

    return_val = PMPI_Gatherv(sendbuf,
                              sendcount,
                              sendtype,
                              recvbuf,
                              recvcounts,
                              displs,
                              recvtype,
                              root,
                              comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_GATHERV],
                     root_loc, EPK_COMM_ID(comm),
             sendcount*sendsz,
             recvcount*recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Gatherv(sendbuf,
                              sendcount,
                              sendtype,
                              recvbuf,
                              recvcounts,
                              displs,
                              recvtype,
                              root,
                              comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_REDUCE) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Reduce
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Reduce(void*        sendbuf,
               void*        recvbuf,
               int          count,
               MPI_Datatype datatype,
               MPI_Op       op,
               int          root,
               MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, me, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_REDUCE,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    
    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);

    if(root != comm_rank)
    {
    		_info.bytes_sent = count * sz;
    }
    else
    {
    		_info.bytes_recv = count * sz;
    }

    return_val = PMPI_Reduce(sendbuf,
                             recvbuf,
                             count,
                             datatype,
                             op,
                             root,
                             comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_REDUCE],
                     root_loc, EPK_COMM_ID(comm),
             count*sz,
             (root == me ? N*count*sz : 0 ));*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Reduce(sendbuf,
                             recvbuf,
                             count,
                             datatype,
                             op,
                             root,
                             comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_REDUCE_SCATTER) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Reduce_scatter
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Reduce_scatter(void*        sendbuf,
                       void*        recvbuf,
                       int*         recvcounts,
                       MPI_Datatype datatype,
                       MPI_Op       op,
                       MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  i, sz, me, N, count = 0;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_REDUCE_SCATTER,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    
    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);

    _info.bytes_sent = sz;
    
    	_info.bytes_recv = recvcounts[comm_rank] * sz;

    return_val = PMPI_Reduce_scatter(sendbuf,
                                     recvbuf,
                                     recvcounts,
                                     datatype,
                                     op,
                                     comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_REDUCE_SCATTER],
                     root_loc, EPK_COMM_ID(comm),
             count * sz,
             N * recvcounts[me] * sz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Reduce_scatter(sendbuf,
                                     recvbuf,
                                     recvcounts,
                                     datatype,
                                     op,
                                     comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_REDUCE_SCATTER_BLOCK) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Reduce_scatter_block
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup coll
 */
int MPI_Reduce_scatter_block(void*        sendbuf,
                             void*        recvbuf,
                             int          recvcount,
                             MPI_Datatype datatype,
                             MPI_Op       op,
                             MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, N;

    region_information_t _info = lwm2_mpi_enter(
      LWM2_MPI_REDUCE_SCATTER_BLOCK,
      LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    
    _info.bytes_sent = sz;
    _info.bytes_recv = recvcount * sz;

    return_val = PMPI_Reduce_scatter_block(sendbuf,
                                           recvbuf,
                                           recvcount,
                                           datatype,
                                           op,
                                           comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_REDUCE_SCATTER_BLOCK],
                     root_loc, EPK_COMM_ID(comm),
             N * recvcount * sz,
             N * recvcount * sz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Reduce_scatter_block(sendbuf,
                                           recvbuf,
                                           recvcount,
                                           datatype,
                                           op,
                                           comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_SCAN) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Scan
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Scan(void*        sendbuf,
             void*        recvbuf,
             int          count,
             MPI_Datatype datatype,
             MPI_Op       op,
             MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sz, me, N;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_SCAN,
                                                LWM2_MPI_GROUP_COLL);
    
    PMPI_Type_size(datatype, &sz);
    _info.bytes_sent = sz * count;
    
    int comm_rank;
    PMPI_Comm_rank(comm, &comm_rank);

    if(comm_rank != 0)
    {
		_info.bytes_recv = sz * count;
    }

    return_val = PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_SCAN],
                     root_loc, EPK_COMM_ID(comm),
             (N-me) * count * sz,
             (me+1) * count * sz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_SCATTER) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Scatter
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Scatter(void*        sendbuf,
                int          sendcount,
                MPI_Datatype sendtype,
                void*        recvbuf,
                int          recvcount,
                MPI_Datatype recvtype,
                int          root,
                MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sendsz, recvsz, N, me;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_SCATTER,
                                                LWM2_MPI_GROUP_COLL);
    
    int comm_size, comm_rank;
    PMPI_Comm_size(comm, &comm_size);
    PMPI_Comm_rank(comm, &comm_rank);
    
    PMPI_Type_size(sendtype, &sendsz);
    PMPI_Type_size(recvtype, &recvsz);
    
    if(comm_rank == root)
    {
    		_info.bytes_sent = comm_size * sendcount * sendsz;
    }
    else
    {
    		_info.bytes_recv = recvcount * recvsz;
    }

    return_val = PMPI_Scatter(sendbuf,
                              sendcount,
                              sendtype,
                              recvbuf,
                              recvcount,
                              recvtype,
                              root,
                              comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_SCATTER],
                     root_loc, EPK_COMM_ID(comm),
             N * sendcount * sendsz,
             recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Scatter(sendbuf,
                              sendcount,
                              sendtype,
                              recvbuf,
                              recvcount,
                              recvtype,
                              root,
                              comm);
  }

  return return_val;
}

#endif
#if defined(HAS_MPI_SCATTERV) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Scatterv
 * @note Auto-generated by wrapgen from template: coll.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup coll
 */
int MPI_Scatterv(void*        sendbuf,
                 int*         sendcounts,
                 int*         displs,
                 MPI_Datatype sendtype,
                 void*        recvbuf,
                 int          recvcount,
                 MPI_Datatype recvtype,
                 int          root,
                 MPI_Comm     comm)
{
  int return_val;

  if (IS_EVENT_GEN_ON_FOR(COLL))
  {
    int                  sendcount, sendsz, recvsz, me, N, i;

    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_SCATTERV,
                                                LWM2_MPI_GROUP_COLL);
    
    int comm_size, comm_rank;
    PMPI_Comm_size(comm, &comm_size);
    PMPI_Comm_rank(comm, &comm_rank);

    PMPI_Type_size(sendtype, &sendsz);
    PMPI_Type_size(recvtype, &recvsz);

    if(comm_rank == root)
    {
    		for(i = 0; i < comm_size; i++)
    		{
    			if(i == root)
    			{
    				continue;
    			}
    			_info.bytes_sent += sendcounts[i] * sendsz;
    		}
    }
    else
    {
    		_info.bytes_recv = recvcount * recvsz;
    }

    return_val = PMPI_Scatterv(sendbuf,
                               sendcounts,
                               displs,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               root,
                               comm);

    /*esd_mpi_collexit(epk_mpi_regid[EPK__MPI_SCATTERV],
                     root_loc, EPK_COMM_ID(comm),
             sendcount * sendsz,
             recvcount * recvsz);*/

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Scatterv(sendbuf,
                               sendcounts,
                               displs,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               root,
                               comm);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_REDUCE_LOCAL) && !defined(NO_MPI_COLL)
/**
 * Measurement wrapper for MPI_Reduce_local
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup coll
 */
int MPI_Reduce_local(void*        inbuf,
                     void*        inoutbuf,
                     int          count,
                     MPI_Datatype datatype,
                     MPI_Op       op)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_REDUCE_LOCAL,
                                              LWM2_MPI_GROUP_NULL);

  return_val = PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 */
