/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_err.c
 *
 * @brief C interface wrappers for error handling
 */

#include <mpi.h>

#include "../lwm2-mpi-config.h"
#include "lwm2_mpireg.h"

/**
 * @name C wrappers
 * @{
 */

#if defined(HAS_MPI_ADD_ERROR_CLASS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Add_error_class
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup err
 */
int MPI_Add_error_class(int* errorclass)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ADD_ERROR_CLASS,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Add_error_class(errorclass);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ADD_ERROR_CODE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Add_error_code
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup err
 */
int MPI_Add_error_code(int errorclass, int* errorcode)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ADD_ERROR_CODE,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Add_error_code(errorclass, errorcode);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ADD_ERROR_STRING) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Add_error_string
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup err
 */
int MPI_Add_error_string(int errorcode, char* string)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ADD_ERROR_STRING,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Add_error_string(errorcode, string);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERRHANDLER_CREATE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Errhandler_create
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Errhandler_create(MPI_Handler_function* function,
                          MPI_Errhandler*       errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERRHANDLER_CREATE,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Errhandler_create(function, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERRHANDLER_FREE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Errhandler_free
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Errhandler_free(MPI_Errhandler* errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERRHANDLER_FREE,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Errhandler_free(errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERRHANDLER_GET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Errhandler_get
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler* errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERRHANDLER_GET,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Errhandler_get(comm, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERRHANDLER_SET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Errhandler_set
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERRHANDLER_SET,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Errhandler_set(comm, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERROR_CLASS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Error_class
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Error_class(int errorcode, int* errorclass)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERROR_CLASS,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Error_class(errorcode, errorclass);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_ERROR_STRING) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_Error_string
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup err
 */
int MPI_Error_string(int errorcode, char* string, int* resultlen)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ERROR_STRING,
                                              LWM2_MPI_GROUP_ERR);

  return_val = PMPI_Error_string(errorcode, string, resultlen);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 */
