/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_ext.c
 *
 * @brief C interface wrappers for external interface functions
 */

#include <mpi.h>

#include "../lwm2-mpi-config.h"
#include "lwm2_mpireg.h"

/**
 * @name C wrappers
 * @{
 */

#if defined(HAS_MPI_ABORT)
/**
 * Measurement wrapper for MPI_Abort
 * @note Manually adapted wrapper
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup ext
 */
int MPI_Abort(MPI_Comm comm,
              int      errorcode)
{
  int return_val;

  /*elg_warning("Explicit MPI_Abort call abandoning Scalasca measurement.");*/

  if (IS_EVENT_GEN_ON_FOR(EXT))
  {
    region_information_t _info = lwm2_mpi_enter(LWM2_MPI_ABORT,
                                                LWM2_MPI_GROUP_EXT);

    return_val = PMPI_Abort(comm, errorcode);

    lwm2_mpi_leave(&_info);
  }
  else
  {
    return_val = PMPI_Abort(comm, errorcode);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_GET_COUNT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Get_count
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup ext
 */
int MPI_Get_count(MPI_Status* status, MPI_Datatype datatype, int* count)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GET_COUNT,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Get_count(status, datatype, count);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GET_ELEMENTS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Get_elements
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup ext
 */
int MPI_Get_elements(MPI_Status* status, MPI_Datatype datatype, int* count)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GET_ELEMENTS,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Get_elements(status, datatype, count);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GET_PROCESSOR_NAME) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Get_processor_name
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup ext
 */
int MPI_Get_processor_name(char* name, int* resultlen)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GET_PROCESSOR_NAME,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Get_processor_name(name, resultlen);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GREQUEST_COMPLETE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Grequest_complete
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup ext
 */
int MPI_Grequest_complete(MPI_Request request)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GREQUEST_COMPLETE,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Grequest_complete(request);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GREQUEST_START) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Grequest_start
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup ext
 */
int MPI_Grequest_start(MPI_Grequest_query_function*  query_fn,
                       MPI_Grequest_free_function*   free_fn,
                       MPI_Grequest_cancel_function* cancel_fn,
                       void*                         extra_state,
                       MPI_Request*                  request)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GREQUEST_START,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Grequest_start(query_fn,
                                   free_fn,
                                   cancel_fn,
                                   extra_state,
                                   request);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_STATUS_SET_CANCELLED) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_MISC)
/**
 * Measurement wrapper for MPI_Status_set_cancelled
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup ext
 */
int MPI_Status_set_cancelled(MPI_Status* status, int flag)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_STATUS_SET_CANCELLED,
    LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Status_set_cancelled(status, flag);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_STATUS_SET_ELEMENTS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Status_set_elements
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup ext
 */
int MPI_Status_set_elements(MPI_Status*  status,
                            MPI_Datatype datatype,
                            int          count)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_STATUS_SET_ELEMENTS,
                                              LWM2_MPI_GROUP_EXT);

  return_val = PMPI_Status_set_elements(status, datatype, count);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 */
