/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_io.c
 *
 * @brief C interface wrappers for parallel I/O
 */

#include <mpi.h>

#include "../lwm2-mpi-config.h"
#include "lwm2_mpireg.h"

/**
 * @name C wrappers for administrative functions
 * @{
 */

#if defined(HAS_MPI_FILE_CLOSE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_close
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_close(MPI_File* fh)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_CLOSE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_close(fh);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_DELETE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_delete
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_delete(char* filename, MPI_Info info)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_DELETE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_delete(filename, info);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_AMODE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_amode
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_amode(MPI_File fh, int* amode)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_AMODE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_amode(fh, amode);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_ATOMICITY) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_atomicity
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_atomicity(MPI_File fh, int* flag)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_ATOMICITY,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_atomicity(fh, flag);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_BYTE_OFFSET) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_byte_offset
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_byte_offset(MPI_File    fh,
                             MPI_Offset  offset,
                             MPI_Offset* disp)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_GET_BYTE_OFFSET,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_byte_offset(fh, offset, disp);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_GROUP) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_group
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_group(MPI_File fh, MPI_Group* group)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_GROUP,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_group(fh, group);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_INFO) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_info
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_info(MPI_File fh, MPI_Info* info_used)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_INFO,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_info(fh, info_used);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_POSITION) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_position
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_position(MPI_File fh, MPI_Offset* offset)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_POSITION,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_position(fh, offset);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_POSITION_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_position_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_position_shared(MPI_File fh, MPI_Offset* offset)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_GET_POSITION_SHARED,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_position_shared(fh, offset);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_SIZE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_size(MPI_File fh, MPI_Offset* size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_SIZE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_size(fh, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_TYPE_EXTENT) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_type_extent
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_type_extent(MPI_File     fh,
                             MPI_Datatype datatype,
                             MPI_Aint*    extent)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_GET_TYPE_EXTENT,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_type_extent(fh, datatype, extent);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_VIEW) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_get_view
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_get_view(MPI_File      fh,
                      MPI_Offset*   disp,
                      MPI_Datatype* etype,
                      MPI_Datatype* filetype,
                      char*         datarep)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_VIEW,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_get_view(fh, disp, etype, filetype, datarep);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_OPEN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_open
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_open(MPI_Comm  comm,
                  char*     filename,
                  int       amode,
                  MPI_Info  info,
                  MPI_File* fh)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_OPEN,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_open(comm, filename, amode, info, fh);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_PREALLOCATE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_preallocate
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_preallocate(MPI_File fh, MPI_Offset size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_PREALLOCATE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_preallocate(fh, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SEEK) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_seek
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SEEK,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_seek(fh, offset, whence);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SEEK_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_seek_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SEEK_SHARED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_seek_shared(fh, offset, whence);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SET_ATOMICITY) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_set_atomicity
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_set_atomicity(MPI_File fh, int flag)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SET_ATOMICITY,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_set_atomicity(fh, flag);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SET_INFO) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_set_info
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_set_info(MPI_File fh, MPI_Info info)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SET_INFO,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_set_info(fh, info);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SET_SIZE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_set_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_set_size(MPI_File fh, MPI_Offset size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SET_SIZE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_set_size(fh, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SET_VIEW) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_set_view
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_set_view(MPI_File     fh,
                      MPI_Offset   disp,
                      MPI_Datatype etype,
                      MPI_Datatype filetype,
                      char*        datarep,
                      MPI_Info     info)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SET_VIEW,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SYNC) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_sync
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_sync(MPI_File fh)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SYNC,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_sync(fh);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_REGISTER_DATAREP) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA)
/**
 * Measurement wrapper for MPI_Register_datarep
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_Register_datarep(
  char* datarep,
  MPI_Datarep_conversion_function*
  read_conversion_fn,
  MPI_Datarep_conversion_function*
  write_conversion_fn,
  MPI_Datarep_extent_function*
  dtype_file_extent_fn,
  void* extra_state)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_REGISTER_DATAREP,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_Register_datarep(datarep,
                                     read_conversion_fn,
                                     write_conversion_fn,
                                     dtype_file_extent_fn,
                                     extra_state);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 * @name C wrappers for access functions
 * @{
 */

#if defined(HAS_MPI_FILE_IREAD) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iread
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iread(MPI_File     fh,
                   void*        buf,
                   int          count,
                   MPI_Datatype datatype,
                   MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IREAD,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iread(fh, buf, count, datatype, request);
  
  PMPI_Type_size(datatype, &sz);
  _info.bytes_recv = count * sz;

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_IREAD_AT) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iread_at
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iread_at(MPI_File     fh,
                      MPI_Offset   offset,
                      void*        buf,
                      int          count,
                      MPI_Datatype datatype,
                      MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IREAD_AT,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iread_at(fh,
                                  offset,
                                  buf,
                                  count,
                                  datatype,
                                  request);
  
  PMPI_Type_size(datatype, &sz);
   _info.bytes_recv = count * sz;

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_IREAD_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iread_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iread_shared(MPI_File     fh,
                          void*        buf,
                          int          count,
                          MPI_Datatype datatype,
                          MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IREAD_SHARED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iread_shared(fh, buf, count, datatype, request);

  PMPI_Type_size(datatype, &sz);
   _info.bytes_recv = count * sz;
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_IWRITE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iwrite
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iwrite(MPI_File     fh,
                    void*        buf,
                    int          count,
                    MPI_Datatype datatype,
                    MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IWRITE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iwrite(fh, buf, count, datatype, request);

  PMPI_Type_size(datatype, &sz);
  _info.bytes_sent = count * sz;

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_IWRITE_AT) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iwrite_at
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iwrite_at(MPI_File     fh,
                       MPI_Offset   offset,
                       void*        buf,
                       int          count,
                       MPI_Datatype datatype,
                       MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IWRITE_AT,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iwrite_at(fh,
                                   offset,
                                   buf,
                                   count,
                                   datatype,
                                   request);
  
  PMPI_Type_size(datatype, &sz);
    _info.bytes_sent = count * sz;

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_IWRITE_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_iwrite_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_iwrite_shared(MPI_File     fh,
                           void*        buf,
                           int          count,
                           MPI_Datatype datatype,
                           MPI_Request* request)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_IWRITE_SHARED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_iwrite_shared(fh, buf, count, datatype, request);

  PMPI_Type_size(datatype, &sz);
    _info.bytes_sent = count * sz;
    
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read(MPI_File     fh,
                  void*        buf,
                  int          count,
                  MPI_Datatype datatype,
                  MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
    _info.bytes_recv= count * sz;
    
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ALL) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_all
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_all(MPI_File     fh,
                      void*        buf,
                      int          count,
                      MPI_Datatype datatype,
                      MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_ALL,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_all(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ALL_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_all_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_all_begin(MPI_File     fh,
                            void*        buf,
                            int          count,
                            MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_ALL_BEGIN,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_all_begin(fh, buf, count, datatype);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ALL_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_all_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_all_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_ALL_END,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_all_end(fh, buf, status);
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_AT) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_at
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_at(MPI_File     fh,
                     MPI_Offset   offset,
                     void*        buf,
                     int          count,
                     MPI_Datatype datatype,
                     MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_AT,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_at(fh, offset, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_AT_ALL) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_at_all
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_at_all(MPI_File     fh,
                         MPI_Offset   offset,
                         void*        buf,
                         int          count,
                         MPI_Datatype datatype,
                         MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_AT_ALL,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_at_all(fh,
                                     offset,
                                     buf,
                                     count,
                                     datatype,
                                     status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_AT_ALL_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_at_all_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_at_all_begin(MPI_File     fh,
                               MPI_Offset   offset,
                               void*        buf,
                               int          count,
                               MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_READ_AT_ALL_BEGIN,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_at_all_begin(fh,
                                           offset,
                                           buf,
                                           count,
                                           datatype);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_AT_ALL_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_at_all_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_at_all_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_READ_AT_ALL_END,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_at_all_end(fh, buf, status);
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ORDERED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_ordered
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_ordered(MPI_File     fh,
                          void*        buf,
                          int          count,
                          MPI_Datatype datatype,
                          MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_ORDERED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_ordered(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ORDERED_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_ordered_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_ordered_begin(MPI_File     fh,
                                void*        buf,
                                int          count,
                                MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_READ_ORDERED_BEGIN,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_ordered_begin(fh, buf, count, datatype);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_ORDERED_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_ordered_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_ordered_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_READ_ORDERED_END,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_ordered_end(fh, buf, status);
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_READ_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_read_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_read_shared(MPI_File     fh,
                         void*        buf,
                         int          count,
                         MPI_Datatype datatype,
                         MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_READ_SHARED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_read_shared(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_recv= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write(MPI_File     fh,
                   void*        buf,
                   int          count,
                   MPI_Datatype datatype,
                   MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
      _info.bytes_sent= count * sz;
      
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ALL) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_all
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_all(MPI_File     fh,
                       void*        buf,
                       int          count,
                       MPI_Datatype datatype,
                       MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_ALL,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_all(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ALL_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_all_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_all_begin(MPI_File     fh,
                             void*        buf,
                             int          count,
                             MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_WRITE_ALL_BEGIN,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_all_begin(fh, buf, count, datatype);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ALL_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_all_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_all_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_ALL_END,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_all_end(fh, buf, status);
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_AT) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_at
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_at(MPI_File     fh,
                      MPI_Offset   offset,
                      void*        buf,
                      int          count,
                      MPI_Datatype datatype,
                      MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_AT,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_at(fh, offset, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_AT_ALL) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_at_all
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_at_all(MPI_File     fh,
                          MPI_Offset   offset,
                          void*        buf,
                          int          count,
                          MPI_Datatype datatype,
                          MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_AT_ALL,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_at_all(fh,
                                      offset,
                                      buf,
                                      count,
                                      datatype,
                                      status);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_AT_ALL_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_at_all_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_at_all_begin(MPI_File     fh,
                                MPI_Offset   offset,
                                void*        buf,
                                int          count,
                                MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_WRITE_AT_ALL_BEGIN,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_at_all_begin(fh,
                                            offset,
                                            buf,
                                            count,
                                            datatype);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_AT_ALL_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_at_all_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_at_all_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_WRITE_AT_ALL_END,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_at_all_end(fh, buf, status);
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ORDERED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_ordered
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_ordered(MPI_File     fh,
                           void*        buf,
                           int          count,
                           MPI_Datatype datatype,
                           MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_ORDERED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_ordered(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ORDERED_BEGIN) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_ordered_begin
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_ordered_begin(MPI_File     fh,
                                 void*        buf,
                                 int          count,
                                 MPI_Datatype datatype)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_WRITE_ORDERED_BEGIN,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_ordered_begin(fh, buf, count, datatype);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_ORDERED_END) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_ordered_end
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_ordered_end(MPI_File fh, void* buf, MPI_Status* status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_WRITE_ORDERED_END,
    LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_ordered_end(fh, buf, status);
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_WRITE_SHARED) && !defined(NO_MPI_IO)
/**
 * Measurement wrapper for MPI_File_write_shared
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io
 */
int MPI_File_write_shared(MPI_File     fh,
                          void*        buf,
                          int          count,
                          MPI_Datatype datatype,
                          MPI_Status*  status)
{
  int                  return_val;
  int					sz;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_WRITE_SHARED,
                                              LWM2_MPI_GROUP_IO);

  return_val = PMPI_File_write_shared(fh, buf, count, datatype, status);

  PMPI_Type_size(datatype, &sz);
        _info.bytes_sent= count * sz;
        
  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 * @name C wrappers for error handling functions
 * @{
 */

#if defined(HAS_MPI_FILE_CALL_ERRHANDLER) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_File_call_errhandler
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_err
 */
int MPI_File_call_errhandler(MPI_File fh, int errorcode)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_CALL_ERRHANDLER,
    LWM2_MPI_GROUP_IO_ERR);

  return_val = PMPI_File_call_errhandler(fh, errorcode);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_CREATE_ERRHANDLER) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_File_create_errhandler
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_err
 */
int MPI_File_create_errhandler(MPI_File_errhandler_fn* function,
                               MPI_Errhandler*         errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_FILE_CREATE_ERRHANDLER,
    LWM2_MPI_GROUP_IO_ERR);

  return_val = PMPI_File_create_errhandler(function, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_GET_ERRHANDLER) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_File_get_errhandler
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_err
 */
int MPI_File_get_errhandler(MPI_File file, MPI_Errhandler* errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_GET_ERRHANDLER,
                                              LWM2_MPI_GROUP_IO_ERR);

  return_val = PMPI_File_get_errhandler(file, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_SET_ERRHANDLER) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_ERR)
/**
 * Measurement wrapper for MPI_File_set_errhandler
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_err
 */
int MPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_SET_ERRHANDLER,
                                              LWM2_MPI_GROUP_IO_ERR);

  return_val = PMPI_File_set_errhandler(file, errhandler);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 * @name Fortran wrappers for handle conversion functions
 * @{
 */

#if defined(HAS_MPI_FILE_C2F) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_MISC)
/**
 * Measurement wrapper for MPI_File_c2f
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_misc
 */
MPI_Fint MPI_File_c2f(MPI_File file)
{
  MPI_Fint             return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_C2F,
                                              LWM2_MPI_GROUP_IO_MISC);

  return_val = PMPI_File_c2f(file);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_FILE_F2C) && !defined(NO_MPI_IO) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_MISC)
/**
 * Measurement wrapper for MPI_File_f2c
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup io_misc
 */
MPI_File MPI_File_f2c(MPI_Fint file)
{
  MPI_File             return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_FILE_F2C,
                                              LWM2_MPI_GROUP_IO_MISC);

  return_val = PMPI_File_f2c(file);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
