/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_topo.c
 *
 * @brief C interface wrappers for topologies
 */

#include <stdlib.h>
#include <mpi.h>

#include "../lwm2-mpi-config.h"
#include "lwm2_mpireg.h"

/**
 * @name C wrappers
 * @{
 */

#if defined(HAS_MPI_CART_CREATE)
/**
 * Measurement wrapper for MPI_Cart_create
 * @note Manually adapted wrapper
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_create(MPI_Comm  comm_old,
                    int       ndims,
                    int*      dims,
                    int*      periodv,
                    int       reorder,
                    MPI_Comm* comm_cart)
{
  const int            event_gen_active = IS_EVENT_GEN_ON_FOR(TOPO);
  int                  return_val;

  region_information_t _info;

  if (event_gen_active)
  {
    _info = lwm2_mpi_enter(LWM2_MPI_CART_CREATE, LWM2_MPI_GROUP_TOPO);
  }

  return_val = PMPI_Cart_create(comm_old,
                                ndims,
                                dims,
                                periodv,
                                reorder,
                                comm_cart);

  if (event_gen_active)
  {
    lwm2_mpi_leave(&_info);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_CART_SUB)
/**
 * Measurement wrapper for MPI_Cart_sub
 * @note Auto-generated by wrapgen from template: comm_mgnt.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_sub(MPI_Comm comm, int* remain_dims, MPI_Comm* newcomm)
{
  const int            event_gen_active = IS_EVENT_GEN_ON_FOR(TOPO);
  int                  return_val;

  region_information_t _info;

  if (event_gen_active)
  {
    _info = lwm2_mpi_enter(LWM2_MPI_CART_SUB, LWM2_MPI_GROUP_TOPO);
  }

  return_val = PMPI_Cart_sub(comm, remain_dims, newcomm);
  if (*newcomm != MPI_COMM_NULL)
  {
    /*epk_comm_create(*newcomm);*/
  }

  if (event_gen_active)
  {
    lwm2_mpi_leave(&_info);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_GRAPH_CREATE)
/**
 * Measurement wrapper for MPI_Graph_create
 * @note Auto-generated by wrapgen from template: comm_mgnt.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graph_create(MPI_Comm  comm_old,
                     int       nnodes,
                     int*      index,
                     int*      edges,
                     int       reorder,
                     MPI_Comm* newcomm)
{
  const int            event_gen_active = IS_EVENT_GEN_ON_FOR(TOPO);
  int                  return_val;

  region_information_t _info;

  if (event_gen_active)
  {
    _info = lwm2_mpi_enter(LWM2_MPI_GRAPH_CREATE, LWM2_MPI_GROUP_TOPO);
  }

  return_val = PMPI_Graph_create(comm_old,
                                 nnodes,
                                 index,
                                 edges,
                                 reorder,
                                 newcomm);
  if (*newcomm != MPI_COMM_NULL)
  {
    /*epk_comm_create(*newcomm);*/
  }

  if (event_gen_active)
  {
    lwm2_mpi_leave(&_info);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_DIST_GRAPH_CREATE) && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Dist_graph_create
 * @note Auto-generated by wrapgen from template: comm_mgnt.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup topo
 */
int MPI_Dist_graph_create(MPI_Comm  comm_old,
                          int       n,
                          int       sources[],
                          int       degrees[],
                          int       destinations[],
                          int       weights[],
                          MPI_Info  info,
                          int       reorder,
                          MPI_Comm* newcomm)
{
  const int            event_gen_active = IS_EVENT_GEN_ON_FOR(TOPO);
  int                  return_val;

  region_information_t _info;

  if (event_gen_active)
  {
    _info = lwm2_mpi_enter(LWM2_MPI_DIST_GRAPH_CREATE, LWM2_MPI_GROUP_TOPO);
  }

  return_val = PMPI_Dist_graph_create(comm_old,
                                      n,
                                      sources,
                                      degrees,
                                      destinations,
                                      weights,
                                      info,
                                      reorder,
                                      newcomm);
  if (*newcomm != MPI_COMM_NULL)
  {
    /*epk_comm_create(*newcomm);*/
  }

  if (event_gen_active)
  {
    lwm2_mpi_leave(&_info);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_DIST_GRAPH_CREATE_ADJACENT) && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Dist_graph_create_adjacent
 * @note Auto-generated by wrapgen from template: comm_mgnt.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup topo
 */
int MPI_Dist_graph_create_adjacent(MPI_Comm  comm_old,
                                   int       indegree,
                                   int       sources[],
                                   int       sourceweights[],
                                   int       outdegree,
                                   int       destinations[],
                                   int       destweights[],
                                   MPI_Info  info,
                                   int       reorder,
                                   MPI_Comm* newcomm)
{
  const int            event_gen_active = IS_EVENT_GEN_ON_FOR(TOPO);
  int                  return_val;

  region_information_t _info;

  if (event_gen_active)
  {
    _info = lwm2_mpi_enter(LWM2_MPI_DIST_GRAPH_CREATE_ADJACENT,
                           LWM2_MPI_GROUP_TOPO);
  }

  return_val = PMPI_Dist_graph_create_adjacent(comm_old,
                                               indegree,
                                               sources,
                                               sourceweights,
                                               outdegree,
                                               destinations,
                                               destweights,
                                               info,
                                               reorder,
                                               newcomm);
  if (*newcomm != MPI_COMM_NULL)
  {
    /*epk_comm_create(*newcomm);*/
  }

  if (event_gen_active)
  {
    lwm2_mpi_leave(&_info);
  }

  return return_val;
}

#endif

#if defined(HAS_MPI_CART_COORDS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cart_coords
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int* coords)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CART_COORDS,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cart_coords(comm, rank, maxdims, coords);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_CART_GET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cart_get
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_get(MPI_Comm comm,
                 int      maxdims,
                 int*     dims,
                 int*     periods,
                 int*     coords)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CART_GET,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cart_get(comm, maxdims, dims, periods, coords);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_CART_MAP) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cart_map
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_map(MPI_Comm comm,
                 int      ndims,
                 int*     dims,
                 int*     periods,
                 int*     newrank)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CART_MAP,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cart_map(comm, ndims, dims, periods, newrank);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_CART_RANK) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cart_rank
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_rank(MPI_Comm comm, int* coords, int* rank)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CART_RANK,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cart_rank(comm, coords, rank);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_CART_SHIFT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cart_shift
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cart_shift(MPI_Comm comm,
                   int      direction,
                   int      disp,
                   int*     rank_source,
                   int*     rank_dest)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CART_SHIFT,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cart_shift(comm,
                               direction,
                               disp,
                               rank_source,
                               rank_dest);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_CARTDIM_GET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Cartdim_get
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Cartdim_get(MPI_Comm comm, int* ndims)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_CARTDIM_GET,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Cartdim_get(comm, ndims);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_DIST_GRAPH_NEIGHBORS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Dist_graph_neighbors
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup topo
 */
int MPI_Dist_graph_neighbors(MPI_Comm comm,
                             int      maxindegree,
                             int      sources[],
                             int      sourceweights[],
                             int      maxoutdegree,
                             int      destinations[],
                             int      destweights[])
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_DIST_GRAPH_NEIGHBORS,
    LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Dist_graph_neighbors(comm,
                                         maxindegree,
                                         sources,
                                         sourceweights,
                                         maxoutdegree,
                                         destinations,
                                         destweights);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_DIST_GRAPH_NEIGHBORS_COUNT) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Dist_graph_neighbors_count
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.2
 * @ingroup topo
 */
int MPI_Dist_graph_neighbors_count(MPI_Comm comm,
                                   int*     indegree,
                                   int*     outdegree,
                                   int*     weighted)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_DIST_GRAPH_NEIGHBORS_COUNT,
    LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Dist_graph_neighbors_count(comm,
                                               indegree,
                                               outdegree,
                                               weighted);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GRAPH_GET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Graph_get
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graph_get(MPI_Comm comm,
                  int      maxindex,
                  int      maxedges,
                  int*     index,
                  int*     edges)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GRAPH_GET,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Graph_get(comm, maxindex, maxedges, index, edges);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GRAPH_MAP) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Graph_map
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graph_map(MPI_Comm comm,
                  int      nnodes,
                  int*     index,
                  int*     edges,
                  int*     newrank)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GRAPH_MAP,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Graph_map(comm, nnodes, index, edges, newrank);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GRAPH_NEIGHBORS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Graph_neighbors
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graph_neighbors(MPI_Comm comm,
                        int      rank,
                        int      maxneighbors,
                        int*     neighbors)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GRAPH_NEIGHBORS,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GRAPH_NEIGHBORS_COUNT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Graph_neighbors_count
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int* nneighbors)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_GRAPH_NEIGHBORS_COUNT,
    LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Graph_neighbors_count(comm, rank, nneighbors);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_GRAPHDIMS_GET) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Graphdims_get
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Graphdims_get(MPI_Comm comm, int* nnodes, int* nedges)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_GRAPHDIMS_GET,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Graphdims_get(comm, nnodes, nedges);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TOPO_TEST) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Topo_test
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Topo_test(MPI_Comm comm, int* status)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TOPO_TEST,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Topo_test(comm, status);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_DIMS_CREATE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TOPO)
/**
 * Measurement wrapper for MPI_Dims_create
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup topo
 */
int MPI_Dims_create(int nnodes, int ndims, int* dims)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_DIMS_CREATE,
                                              LWM2_MPI_GROUP_TOPO);

  return_val = PMPI_Dims_create(nnodes, ndims, dims);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
/**
 * @}
 */
