/****************************************************************************
**  Wrapper originally built for:										
**  **
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  LWM2
**  **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2010                                                     **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpiwrap_type.c
 *
 * @brief C interface wrappers for datatype functions
 */

#include <mpi.h>

#include "../lwm2-mpi-config.h"

/**
 * @name C wrappers for general functions
 * @{
 */

#if defined(HAS_MPI_PACK) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Pack
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Pack(void*        inbuf,
             int          incount,
             MPI_Datatype datatype,
             void*        outbuf,
             int          outsize,
             int*         position,
             MPI_Comm     comm)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_PACK,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Pack(inbuf,
                         incount,
                         datatype,
                         outbuf,
                         outsize,
                         position,
                         comm);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_PACK_EXTERNAL) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Pack_external
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Pack_external(char*        datarep,
                      void*        inbuf,
                      int          incount,
                      MPI_Datatype datatype,
                      void*        outbuf,
                      MPI_Aint     outsize,
                      MPI_Aint*    position)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_PACK_EXTERNAL,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Pack_external(datarep,
                                  inbuf,
                                  incount,
                                  datatype,
                                  outbuf,
                                  outsize,
                                  position);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_PACK_EXTERNAL_SIZE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Pack_external_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Pack_external_size(char*        datarep,
                           int          incount,
                           MPI_Datatype datatype,
                           MPI_Aint*    size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_PACK_EXTERNAL_SIZE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Pack_external_size(datarep, incount, datatype, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_PACK_SIZE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Pack_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Pack_size(int          incount,
                  MPI_Datatype datatype,
                  MPI_Comm     comm,
                  int*         size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_PACK_SIZE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Pack_size(incount, datatype, comm, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_SIZEOF) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Sizeof
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Sizeof(MPI_Datatype x, int size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_SIZEOF,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Sizeof(x, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_COMMIT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_commit
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_commit(MPI_Datatype* datatype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_COMMIT,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_commit(datatype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CONTIGUOUS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_contiguous
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_contiguous(int           count,
                        MPI_Datatype  oldtype,
                        MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CONTIGUOUS,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_contiguous(count, oldtype, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_DARRAY) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_darray
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_darray(int           size,
                           int           rank,
                           int           ndims,
                           int           array_of_gsizes[],
                           int           array_of_distribs[],
                           int           array_of_dargs[],
                           int           array_of_psizes[],
                           int           order,
                           MPI_Datatype  oldtype,
                           MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CREATE_DARRAY,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_darray(size,
                                       rank,
                                       ndims,
                                       array_of_gsizes,
                                       array_of_distribs,
                                       array_of_dargs,
                                       array_of_psizes,
                                       order,
                                       oldtype,
                                       newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_F90_COMPLEX) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_f90_complex
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_f90_complex(int p, int r, MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_F90_COMPLEX,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_f90_complex(p, r, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_F90_INTEGER) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_f90_integer
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_f90_integer(int r, MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_F90_INTEGER,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_f90_integer(r, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_F90_REAL) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_f90_real
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_f90_real(int p, int r, MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_F90_REAL,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_f90_real(p, r, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_HINDEXED) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_hindexed
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_hindexed(int           count,
                             int           array_of_blocklengths[],
                             MPI_Aint      array_of_displacements[],
                             MPI_Datatype  oldtype,
                             MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_HINDEXED,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_hindexed(count,
                                         array_of_blocklengths,
                                         array_of_displacements,
                                         oldtype,
                                         newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_HVECTOR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_hvector
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_hvector(int           count,
                            int           blocklength,
                            MPI_Aint      stride,
                            MPI_Datatype  oldtype,
                            MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CREATE_HVECTOR,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_hvector(count,
                                        blocklength,
                                        stride,
                                        oldtype,
                                        newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_INDEXED_BLOCK) \
  && !defined(NO_MPI_EXTRA) && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_indexed_block
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_indexed_block(int           count,
                                  int           blocklength,
                                  int           array_of_displacements[],
                                  MPI_Datatype  oldtype,
                                  MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_INDEXED_BLOCK,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_indexed_block(count,
                                              blocklength,
                                              array_of_displacements,
                                              oldtype,
                                              newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_RESIZED) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_resized
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_resized(MPI_Datatype  oldtype,
                            MPI_Aint      lb,
                            MPI_Aint      extent,
                            MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CREATE_RESIZED,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_resized(oldtype, lb, extent, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_STRUCT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_struct
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_struct(int           count,
                           int           array_of_blocklengths[],
                           MPI_Aint      array_of_displacements[],
                           MPI_Datatype  array_of_types[],
                           MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CREATE_STRUCT,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_struct(count,
                                       array_of_blocklengths,
                                       array_of_displacements,
                                       array_of_types,
                                       newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_CREATE_SUBARRAY) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_create_subarray
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_create_subarray(int           ndims,
                             int           array_of_sizes[],
                             int           array_of_subsizes[],
                             int           array_of_starts[],
                             int           order,
                             MPI_Datatype  oldtype,
                             MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_CREATE_SUBARRAY,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_create_subarray(ndims,
                                         array_of_sizes,
                                         array_of_subsizes,
                                         array_of_starts,
                                         order,
                                         oldtype,
                                         newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_DUP) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_dup
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_dup(MPI_Datatype type, MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_DUP,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_dup(type, newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_EXTENT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_extent
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_extent(MPI_Datatype datatype, MPI_Aint* extent)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_EXTENT,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_extent(datatype, extent);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_FREE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_free
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_free(MPI_Datatype* datatype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_FREE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_free(datatype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_CONTENTS) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_get_contents
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_get_contents(MPI_Datatype datatype,
                          int          max_integers,
                          int          max_addresses,
                          int          max_datatypes,
                          int          array_of_integers[],
                          MPI_Aint     array_of_addresses[],
                          MPI_Datatype array_of_datatypes[])
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_GET_CONTENTS,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_get_contents(datatype,
                                      max_integers,
                                      max_addresses,
                                      max_datatypes,
                                      array_of_integers,
                                      array_of_addresses,
                                      array_of_datatypes);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_ENVELOPE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_get_envelope
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_get_envelope(MPI_Datatype datatype,
                          int*         num_integers,
                          int*         num_addresses,
                          int*         num_datatypes,
                          int*         combiner)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_GET_ENVELOPE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_get_envelope(datatype,
                                      num_integers,
                                      num_addresses,
                                      num_datatypes,
                                      combiner);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_EXTENT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_get_extent
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_get_extent(MPI_Datatype datatype,
                        MPI_Aint*    lb,
                        MPI_Aint*    extent)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_GET_EXTENT,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_get_extent(datatype, lb, extent);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_TRUE_EXTENT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_get_true_extent
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_get_true_extent(MPI_Datatype datatype,
                             MPI_Aint*    true_lb,
                             MPI_Aint*    true_extent)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(
    LWM2_MPI_TYPE_GET_TRUE_EXTENT,
    LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_get_true_extent(datatype, true_lb, true_extent);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_HINDEXED) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_hindexed
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_hindexed(int           count,
                      int*          array_of_blocklengths,
                      MPI_Aint*     array_of_displacements,
                      MPI_Datatype  oldtype,
                      MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_HINDEXED,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_hindexed(count,
                                  array_of_blocklengths,
                                  array_of_displacements,
                                  oldtype,
                                  newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_HVECTOR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_hvector
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_hvector(int           count,
                     int           blocklength,
                     MPI_Aint      stride,
                     MPI_Datatype  oldtype,
                     MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_HVECTOR,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_hvector(count,
                                 blocklength,
                                 stride,
                                 oldtype,
                                 newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_INDEXED) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_indexed
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_indexed(int           count,
                     int*          array_of_blocklengths,
                     int*          array_of_displacements,
                     MPI_Datatype  oldtype,
                     MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_INDEXED,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_indexed(count,
                                 array_of_blocklengths,
                                 array_of_displacements,
                                 oldtype,
                                 newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_LB) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_lb
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_lb(MPI_Datatype datatype, MPI_Aint* displacement)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_LB,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_lb(datatype, displacement);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_MATCH_SIZE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_match_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Type_match_size(int typeclass, int size, MPI_Datatype* type)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_MATCH_SIZE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_match_size(typeclass, size, type);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_SIZE) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_size
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_size(MPI_Datatype datatype, int* size)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_SIZE,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_size(datatype, size);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_STRUCT) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_struct
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_struct(int           count,
                    int*          array_of_blocklengths,
                    MPI_Aint*     array_of_displacements,
                    MPI_Datatype* array_of_types,
                    MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_STRUCT,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_struct(count,
                                array_of_blocklengths,
                                array_of_displacements,
                                array_of_types,
                                newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_UB) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_ub
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_ub(MPI_Datatype datatype, MPI_Aint* displacement)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_UB,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_ub(datatype, displacement);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_VECTOR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Type_vector
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Type_vector(int           count,
                    int           blocklength,
                    int           stride,
                    MPI_Datatype  oldtype,
                    MPI_Datatype* newtype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_VECTOR,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Type_vector(count,
                                blocklength,
                                stride,
                                oldtype,
                                newtype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_UNPACK) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Unpack
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 1.0
 * @ingroup type
 */
int MPI_Unpack(void*        inbuf,
               int          insize,
               int*         position,
               void*        outbuf,
               int          outcount,
               MPI_Datatype datatype,
               MPI_Comm     comm)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_UNPACK,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Unpack(inbuf,
                           insize,
                           position,
                           outbuf,
                           outcount,
                           datatype,
                           comm);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_UNPACK_EXTERNAL) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE)
/**
 * Measurement wrapper for MPI_Unpack_external
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type
 */
int MPI_Unpack_external(char*        datarep,
                        void*        inbuf,
                        MPI_Aint     insize,
                        MPI_Aint*    position,
                        void*        outbuf,
                        int          outcount,
                        MPI_Datatype datatype)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_UNPACK_EXTERNAL,
                                              LWM2_MPI_GROUP_TYPE);

  return_val = PMPI_Unpack_external(datarep,
                                    inbuf,
                                    insize,
                                    position,
                                    outbuf,
                                    outcount,
                                    datatype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 * @name C wrappers for external interface functions
 * @{
 */

#if defined(HAS_MPI_TYPE_CREATE_KEYVAL) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_create_keyval
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_create_keyval(
  MPI_Type_copy_attr_function* type_copy_attr_fn,
  MPI_Type_delete_attr_function*
  type_delete_attr_fn,
  int*                         type_keyval,
  void*                        extra_state)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_CREATE_KEYVAL,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_create_keyval(type_copy_attr_fn,
                                       type_delete_attr_fn,
                                       type_keyval,
                                       extra_state);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_DELETE_ATTR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_delete_attr
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_delete_attr(MPI_Datatype type, int type_keyval)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_DELETE_ATTR,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_delete_attr(type, type_keyval);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_FREE_KEYVAL) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_free_keyval
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_free_keyval(int* type_keyval)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_FREE_KEYVAL,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_free_keyval(type_keyval);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_ATTR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_get_attr
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_get_attr(MPI_Datatype type,
                      int          type_keyval,
                      void*        attribute_val,
                      int*         flag)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_GET_ATTR,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_GET_NAME) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_get_name
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_get_name(MPI_Datatype type, char* type_name, int* resultlen)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_GET_NAME,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_get_name(type, type_name, resultlen);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_SET_ATTR) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_set_attr
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_set_attr(MPI_Datatype type,
                      int          type_keyval,
                      void*        attribute_val)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_SET_ATTR,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_set_attr(type, type_keyval, attribute_val);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_SET_NAME) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_EXT)
/**
 * Measurement wrapper for MPI_Type_set_name
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_ext
 */
int MPI_Type_set_name(MPI_Datatype type, char* type_name)
{
  int                  return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_SET_NAME,
                                              LWM2_MPI_GROUP_TYPE_EXT);

  return_val = PMPI_Type_set_name(type, type_name);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 * @name C wrappers for handle conversion functions
 * @{
 */

#if defined(HAS_MPI_TYPE_C2F) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_MISC)
/**
 * Measurement wrapper for MPI_Type_c2f
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_misc
 */
MPI_Fint MPI_Type_c2f(MPI_Datatype datatype)
{
  MPI_Fint             return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_C2F,
                                              LWM2_MPI_GROUP_TYPE_MISC);

  return_val = PMPI_Type_c2f(datatype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif
#if defined(HAS_MPI_TYPE_F2C) && !defined(NO_MPI_EXTRA) \
  && !defined(NO_MPI_TYPE) && !defined(NO_MPI_MISC)
/**
 * Measurement wrapper for MPI_Type_f2c
 * @note Auto-generated by wrapgen from template: std.w
 * @note C interface
 * @note Introduced with MPI 2.0
 * @ingroup type_misc
 */
MPI_Datatype MPI_Type_f2c(MPI_Fint datatype)
{
  MPI_Datatype         return_val;

  region_information_t _info = lwm2_mpi_enter(LWM2_MPI_TYPE_F2C,
                                              LWM2_MPI_GROUP_TYPE_MISC);

  return_val = PMPI_Type_f2c(datatype);

  lwm2_mpi_leave(&_info);

  return return_val;
}

#endif

/**
 * @}
 */
