/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/



#define _GNU_SOURCE

#include "../../lwm2-config.h"

#include <stdio.h>
#include <stdint.h>
#include <bits/pthreadtypes.h>
#include <time.h>
#include <dlfcn.h>
#include <stdlib.h>

#include "../../util/lwm2-log.h"

#include "lwm2-pthreadwrap.h"
#include "../lwm2-pthread-config.h"
#include "../lwm2-pthread-adapter.h"

/* ## concatenates, # quotes */
#define get_original_function(newfunc, name, fptr_type) {          \
    newfunc = (fptr_type)dlsym(RTLD_NEXT, # name);                 \
    if (NULL == newfunc)                                           \
    {                                                              \
      LWM2LOG(0, "Could not get original address of function %s\n", # name);                    \
        LWM2LOG(0, "This will potentially effect the functionality of the application. Exiting!\n");		\
        exit(-1);											\
    }                                                              \
}

#ifndef LWM2_NO_DLSYM
#ifdef LWM2_MEASUREMENT_PTHREAD

#define LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_CREATE
#define LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_EXIT
#define LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_CANCEL

/* pthread start function. defined as ftpr to use existing capabilities
   of scalasca wrapgen */

/* Wrap all defined functions */

/* Management */
typedef int (*fptr_pthread_create)(pthread_t* thread,
                                   const pthread_attr_t* attr,
                                   fptr start_function, void* arg);
fptr_pthread_create Ppthread_create = NULL;

typedef void (*fptr_pthread_exit)(void* retval);
fptr_pthread_exit Ppthread_exit = NULL;

typedef int (*fptr_pthread_cancel)(pthread_t thread);
fptr_pthread_cancel Ppthread_cancel = NULL;



/**
 * Measurement wrapper for pthread_create
 * @note Auto-generated by wrapgen from template: lwm2_pthread_std.w
 * @ingroup management
 */
int pthread_create(pthread_t*            thread,
                   const pthread_attr_t* attr,
                   fptr                  start_function,
                   void*                 arg)
{
  int return_val;

  if (NULL != Ppthread_create)
  {
    /* Original call */
    return_val = Ppthread_create(thread, attr, start_function, arg);
  }


  #ifdef LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_CREATE
  /* Call our custom handler */
  lwm2_adapter_pthread_custom_leave_pthread_create(
    *thread
    );
  #endif

  return return_val;
} /* pthread_create */

void pthread_exit(void* retval)
{
	
#ifdef LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_EXIT
	lwm2_adapter_pthread_custom_enter_pthread_exit();
#endif
	
	if(Ppthread_exit != NULL)
	{
		Ppthread_exit(retval);
	}
}

int pthread_cancel(pthread_t thread)
{
	int ret = -1;
#ifdef LWM2_ADAPTER_PTHREAD_CUSTOM_PTHREAD_CANCEL
	lwm2_adapter_pthread_custom_enter_pthread_cancel();
#endif
	
	if(Ppthread_cancel != NULL)
	{
		ret = Ppthread_cancel(thread);
	}
	return ret;
}

#endif
#endif

void lwm2_wrapper_pthread_init()
{
#ifndef LWM2_NO_DLSYM
#ifdef LWM2_MEASUREMENT_PTHREAD
  get_original_function(Ppthread_create,
                        pthread_create,
                        fptr_pthread_create);
  
  get_original_function(Ppthread_exit, pthread_exit, fptr_pthread_exit);
  
  get_original_function(Ppthread_cancel, pthread_cancel, fptr_pthread_cancel);
#endif
#endif
}

