
#include "../include/lwm2-l2f-reader.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../../lwm2file/include/lwm2-file-common.h"
#include "../../lwm2file/include/lwm2-file-reader.h"
#include "../../lwm2file/include/lwm2-file-reader-def.h"

file_def_t file_definition;
sensor_def_t *sensors;

#define FILE_LEN 256

char def_file[FILE_LEN];
char slc_file[FILE_LEN];
char sum_file[FILE_LEN];

int main(int argc, char* argv[])
{
	if(argc < 2)
	{
		printf("Usage: l2f_reader <file_name> [--list-sensors] [--file-def] [--time-slices] [--summary] [--proc-metric-tslice #proc #metric]\n"  
				"[--proc-metric-summary #proc #metric] [--metric-tslice #metric] [--metric-summary #metric] [--metric-proc #metric]\n" 
				"[--proc-summary #proc] [--allproc-metric-tslice #metric] \n");
		return -1;
	}
	
	int len = strlen(argv[1]);
	if(len + 10 > FILE_LEN)
	{
		printf("File name larger than buffer\n");
		return -1;
	}
	
//	sprintf(def_file, "%s.definition", argv[1]);
//	sprintf(sum_file, "%s.digest", argv[1]);
//	sprintf(slc_file, "%s.slices", argv[1]);
	
	lwm2_file_read_init(argv[1]);
	lwm2_l2f_read_desc_file();
	
	int ind;
	
	printf("\n");
	
	for(ind = 2; ind < argc; ind++)
	{
		if(strncmp("--list-sensors", argv[ind], 14) == 0)
		{
			lwm2_l2f_show_sensors();
			continue;
		}
		
		if(strncmp("--file-def", argv[ind], 10) == 0)
		{
			lwm2_l2f_show_file_def();
			continue;
		}
		
		if(strncmp("--time-slices", argv[ind], 13) == 0)
		{
			lwm2_l2f_show_all_metrics_by_tslices();
			continue;
		}
		
		if(strncmp("--summary", argv[ind], 9) == 0)
		{
			lwm2_l2f_show_summary();
			continue;
		}
		
		if(strncmp("--allproc-metric-tslice", argv[ind], 24) == 0)
		{
			if(ind + 1 < argc)
			{
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);
				
				lwm2_l2f_show_metric_in_all_processes_per_tslice(metric);
			}
			continue;
		}
		
		if(strncmp("--proc-metric-tslice", argv[ind], 20) == 0)
		{
			if(ind + 2 < argc)
			{
				long proc = strtol(argv[++ind], (char **)NULL, 10);
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);
				
				lwm2_l2f_show_metric_in_process_per_tslice(proc, metric);
			}
			continue;
		}
		
		if(strncmp("--proc-metric-summary", argv[ind], 21) == 0)
		{
			if(ind + 2 < argc)
			{
				long proc = strtol(argv[++ind], (char **)NULL, 10);
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);

				lwm2_l2f_show_metric_in_process_in_summary(proc, metric);
			}
			continue;
		}
		
		if(strncmp("--metric-tslice", argv[ind], 15) == 0)
		{
			if(ind + 1 < argc)
			{
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);

				lwm2_l2f_show_metric_per_tslice(metric);
			}
			continue;
		}
		
		if(strncmp("--metric-summary", argv[ind], 16) == 0)
		{
			if(ind + 1 < argc)
			{
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);

				lwm2_l2f_show_metric_in_summary(metric);
			}
			continue;
		}
		
		if(strncmp("--metric-proc", argv[ind], 13) == 0)
		{
			if(ind + 1 < argc)
			{
				int metric = (int) strtol(argv[++ind], (char **)NULL, 10);

				lwm2_l2f_show_metric_by_processes(metric);
			}
			continue;
		}
		
		if(strncmp("--proc-summary", argv[ind], 14) == 0)
		{
			if(ind + 1 < argc)
			{
				long proc = strtol(argv[++ind], (char **)NULL, 10);

				lwm2_l2f_show_process_in_summary(proc);
			}
			continue;
		}
	}
	
	lwm2_l2f_cleanup();
	
	return 1;
}

void lwm2_l2f_read_desc_file()
{
	int i;
	
	lwm2_file_read_init_definition();
	file_definition = lwm2_file_get_definition();
//	file_definition.time_slice_count++;
//	lwm2_file_reader_def_file_def(def_file, &file_definition);
	
	sensors = (sensor_def_t*) malloc(file_definition.summary_metric_count * sizeof(sensor_def_t));
	
	for(i = 0; i < file_definition.summary_metric_count; i++)
	{
		lwm2_file_reader_def_sensor(&sensors[i]);
	}
}

void lwm2_l2f_cleanup()
{
	lwm2_file_read_finish_definition();
	free(sensors);
}

void lwm2_l2f_show_sensors()
{
	int i;
	int tslice_limit = file_definition.summary_metric_count - file_definition.time_slice_metric_count;
	
	char* only_summary = "Summary only";
	char* both_type = "Summary and time-slice";
	
	char* sensor_type = only_summary;
	
	
	printf("%-30s%-8s%-25s\n", "Sensor name", "ID", "Type");
	printf("------------------------------------------------------------------------\n");
	for(i = 0; i < file_definition.summary_metric_count; i++)
	{
		if(i >= tslice_limit)
		{
			sensor_type = both_type;
		}
		printf("%-30s%-8d%-25s\n", sensors[i].sensor_name, sensors[i].sensor_id, sensor_type);
	}
}

void lwm2_l2f_show_file_def()
{
	printf("Profiling meta data\n");
	printf("------------------------------------------------\n");
	
	printf("%25s: %-4d\n", "Version", file_definition.version);
	printf("%25s: %-4d[secs]\n", "Time-slice length", file_definition.time_slice_length);
	printf("%25s: %-4d\n", "Time-slice metric count", file_definition.time_slice_metric_count);
	printf("%25s: %-4d\n", "Summary metric count", file_definition.summary_metric_count);
	printf("%25s: %-10d\n", "Job id", file_definition.job_id);
	printf("%25s: %-7d\n", "Process count", file_definition.process_count);
	printf("%25s: %-5ld\n", "Time-slice count", file_definition.time_slice_count);
	printf("%25s: %-10ld\n", "Time-slice start time", file_definition.time_slice_start_time);
}

void lwm2_l2f_show_all_metrics_by_tslices()
{
	int no_time_slice = file_definition.summary_metric_count - file_definition.time_slice_metric_count;

	lwm2_file_read_init_time_slice_file();

	int proc_ind, tslice_ind, k;
	unsigned long long metric_data[file_definition.time_slice_count][file_definition.time_slice_metric_count];
	unsigned long long metric_tslice[file_definition.time_slice_metric_count];

	for(proc_ind = 0; proc_ind < file_definition.process_count; proc_ind++)
	{
		for(tslice_ind = 0; tslice_ind < file_definition.time_slice_count; tslice_ind++)
		{
			if(proc_ind == 0)
			{
				for(k = 0; k < file_definition.time_slice_metric_count; k++)
				{
					metric_data[tslice_ind][k] = 0;
				}
			}

			lwm2_file_read_process_time_slice(metric_tslice);
			for(k = 0; k < file_definition.time_slice_metric_count; k++)
			{
				metric_data[tslice_ind][k] += metric_tslice[k];
			}
		}
	}

	for(tslice_ind = 0; tslice_ind < file_definition.time_slice_count; tslice_ind++)
	{
		for(k = 0; k < file_definition.time_slice_metric_count; k++)
		{
			printf("%12llu", metric_data[tslice_ind][k]);
		}
		printf("\n");
	}

	lwm2_file_read_finish_time_slice_file();
}

void lwm2_l2f_show_summary()
{
	lwm2_file_read_init_summary_file();

	unsigned long long metrics[file_definition.summary_metric_count];
	unsigned long long sum_metrics[file_definition.summary_metric_count];
	int proc_ind, k;
	
	for(proc_ind = 0; proc_ind < file_definition.process_count; proc_ind++)
	{
		lwm2_file_read_process_summary_metrics(metrics);

		for(k = 0; k < file_definition.summary_metric_count; k++)
		{
			if(proc_ind == 0)
			{
				sum_metrics[k] = 0;
			}
			sum_metrics[k] += metrics[k];
		}
	}

	printf("\tSummary:\n-------------------------------------------------\n");
	for(k = 0; k < file_definition.summary_metric_count; k++)
	{
		printf("%25s: %llu\n", sensors[k].sensor_name, sum_metrics[k]);
	}

	lwm2_file_read_finish_summary_file();
}

void lwm2_l2f_show_metric_in_all_processes_per_tslice(int metric)
{
	int no_time_slice = file_definition.summary_metric_count - file_definition.time_slice_metric_count;
	if(metric < no_time_slice || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number: %d\n", metric);
		return;
	}
	
	lwm2_file_read_init_time_slice_file();
	
	int i, proc;
	unsigned long long metrics[file_definition.time_slice_metric_count];
	
	printf("\n");
	for(proc = 0; proc < file_definition.process_count; proc++)
	{
		for(i = 0; i < file_definition.time_slice_count; i++)
		{
			lwm2_file_read_process_time_slice(metrics);
			printf("%llu\t", metrics[metric - no_time_slice]);
		}
		printf("\n");
	}
	
	lwm2_file_read_finish_time_slice_file();
}

void lwm2_l2f_show_metric_in_process_per_tslice(long proc, int metric)
{
	if(proc < 0 || proc > file_definition.process_count - 1)
	{
		printf("Invalid process number\n");
		return;
	}
	
	int no_time_slice = file_definition.summary_metric_count - file_definition.time_slice_metric_count;
	if(metric < no_time_slice || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number\n");
		return;
	}
	
	lwm2_file_read_init_time_slice_file();
	
	if(proc >0)
	{
		lwm2_file_skip_process_from_start(proc - 1);
	}
	
	int i;
	unsigned long long metrics[file_definition.time_slice_metric_count];
	
	for(i = 0; i < file_definition.time_slice_count; i++)
	{
		lwm2_file_read_process_time_slice(metrics);
		printf("%llu\n", metrics[metric - no_time_slice]);
	}
	
	lwm2_file_read_finish_time_slice_file();
}

void lwm2_l2f_show_metric_in_process_in_summary(long proc, int metric)
{
	if(proc < 0 || proc > file_definition.process_count - 1)
	{
		printf("Invalid process number\n");
		return;
	}

	if(metric < 0 || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number\n");
		return;
	}
		
	lwm2_file_read_init_summary_file();
	
	lwm2_file_skip_process_summaries(proc);
	
	unsigned long long metrics[file_definition.summary_metric_count];
	lwm2_file_read_process_summary_metrics(metrics);
	
	printf("%25s: %llu\n", sensors[metric].sensor_name, metrics[metric]);
	
	lwm2_file_read_finish_summary_file();
}

void lwm2_l2f_show_metric_per_tslice(int metric)
{
	int no_time_slice = file_definition.summary_metric_count - file_definition.time_slice_metric_count;
	if(metric < no_time_slice || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number\n");
		return;
	}
	
	lwm2_file_read_init_time_slice_file();
	
	int proc_ind, tslice_ind;
	unsigned long long metric_data[file_definition.time_slice_count];
	unsigned long long metric_tslice[file_definition.time_slice_metric_count];
	
	for(proc_ind = 0; proc_ind < file_definition.process_count; proc_ind++)
	{
		for(tslice_ind = 0; tslice_ind < file_definition.time_slice_count; tslice_ind++)
		{
			if(proc_ind == 0)
			{
				metric_data[tslice_ind] = 0;
			}
			
			lwm2_file_read_process_time_slice(metric_tslice);
			metric_data[tslice_ind] += metric_tslice[metric - no_time_slice];
		}
	}
	
	for(tslice_ind = 0; tslice_ind < file_definition.time_slice_count; tslice_ind++)
	{
		printf("%llu\n", metric_data[tslice_ind]);
	}

	lwm2_file_read_finish_time_slice_file();
}

void lwm2_l2f_show_metric_in_summary(int metric)
{
	if(metric < 0 || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number\n");
		return;
	}
	
	unsigned long long metric_val = 0;
	int k;

	lwm2_file_read_init_summary_file();

	unsigned long long metrics[file_definition.summary_metric_count];
	
	for(k = 0; k < file_definition.process_count; k++)
	{
		lwm2_file_read_process_summary_metrics(metrics);
		metric_val += metrics[metric];
	}

	printf("%25s: %llu\n", sensors[metric].sensor_name, metrics[metric]);

	lwm2_file_read_finish_summary_file();
}

void lwm2_l2f_show_metric_by_processes(int metric)
{
	if(metric < 0 || metric > file_definition.summary_metric_count - 1)
	{
		printf("Invalid metric number\n");
		return;
	}

	unsigned long long metric_val = 0;
	int k;

	lwm2_file_read_init_summary_file();

	unsigned long long metrics[file_definition.summary_metric_count];

	printf("%s:\n----------------------------------\n", sensors[metric].sensor_name);
	
	for(k = 0; k < file_definition.process_count; k++)
	{
		lwm2_file_read_process_summary_metrics(metrics);
		printf("%llu\n", metrics[metric]);
	}

	lwm2_file_read_finish_summary_file();
}

void lwm2_l2f_show_process_in_summary(long proc)
{
	if(proc < 0 || proc > file_definition.process_count - 1)
	{
		printf("Invalid process number\n");
		return;
	}

	lwm2_file_read_init_summary_file();

	lwm2_file_skip_process_summaries(proc);

	unsigned long long metrics[file_definition.summary_metric_count];
	int k;
	
	lwm2_file_read_process_summary_metrics(metrics);

	for(k = 0; k < file_definition.summary_metric_count; k++)
	{
		printf("%25s: %llu\n", sensors[k].sensor_name, metrics[k]);
	}

	lwm2_file_read_finish_summary_file();
}

