/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "../include/lwm2-file-reader.h"

#include "../include/lwm2-file-reader-def.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CHECK_HANDLE(X) if((X) == NULL) { return; }
#define CHECK_HANDLE_RETURN(X, Y) if((X) == NULL) { return Y; }

#define SENSOR_NAME_LEN (34)
#define NODE_NAME_LEN (64)
#define FILE_NAME_LEN (1024)

FILE* rsummary_file_handle = NULL;
FILE* rslices_file_handle = NULL;

file_def_t rfile_definition;

unsigned long long* proc_buff = NULL;

char def_reader_file_name[FILE_NAME_LEN];
char def_summary_file_name[FILE_NAME_LEN];
char def_time_slice_file_name[FILE_NAME_LEN];

void lwm2_file_read_init(char* file_name)
{
	sprintf(def_reader_file_name, "%s.definition", file_name);
	sprintf(def_summary_file_name, "%s.digest", file_name);
	sprintf(def_time_slice_file_name, "%s.slices", file_name);
}

int lwm2_file_read_init_definition()
{	
	int rtn = lwm2_file_reader_def_file_def(def_reader_file_name, &rfile_definition);
	return rtn;
}

void lwm2_file_read_finish_definition()
{
	lwm2_file_reader_def_finish();
}

int lwm2_file_get_time_slice_length()
{
	return rfile_definition.time_slice_length;
}

unsigned long lwm2_file_get_time_slice_count()
{
	return rfile_definition.time_slice_count;
}

unsigned long lwm2_file_get_time_slice_start_time()
{
	return rfile_definition.time_slice_start_time;
}

unsigned short lwm2_file_get_summary_sensor_count()
{
	return rfile_definition.summary_metric_count;
}

unsigned short lwm2_file_get_time_slice_sensor_count()
{
	return rfile_definition.time_slice_metric_count;
}

unsigned int lwm2_file_get_job_id()
{
	return rfile_definition.job_id;
}

unsigned int lwm2_file_get_process_count()
{
	return rfile_definition.process_count;
}

file_def_t lwm2_file_get_definition()
{
	return rfile_definition;
}

int lwm2_file_read_sensor_def(unsigned short *sensor_id, char* sensor_name)
{
	sensor_def_t sensor_definition;
	
	int rtn = lwm2_file_reader_def_sensor(&sensor_definition);
	
	if(rtn == 0)
	{
		return 0;
	}
	
	*sensor_id = sensor_definition.sensor_id;
	strncpy(sensor_name, sensor_definition.sensor_name, SENSOR_NAME_LEN);
	
	return 1;
}

int lwm2_file_read_process_mapping(unsigned int *process_id, char* node_name)
{
	process_map_t proc_mapping;
	
	int rtn = lwm2_file_reader_def_process_mapping(&proc_mapping);
	
	if(rtn == 0)
	{
		return 0;
	}
	
	*process_id = proc_mapping.proc_id;
	strncpy(node_name, proc_mapping.node_name, NODE_NAME_LEN);
	
	return 1;
}

int lwm2_file_read_init_summary_file()
{
	rsummary_file_handle = fopen(def_summary_file_name, "rb");
	if(rsummary_file_handle == NULL)
	{
		return 0;
	}
	return 1;
}

void lwm2_file_read_finish_summary_file()
{
	CHECK_HANDLE(rsummary_file_handle);
	fclose(rsummary_file_handle);
}

void lwm2_file_read_process_summary_metrics(unsigned long long* metric_values)
{
	CHECK_HANDLE(rsummary_file_handle);
	fread(metric_values, sizeof(unsigned long long), rfile_definition.summary_metric_count, rsummary_file_handle);
}

void lwm2_file_skip_process_summaries(unsigned int num_processes)
{
	CHECK_HANDLE(rsummary_file_handle);
	
	long skip_bytes = sizeof(unsigned long long) * rfile_definition.summary_metric_count * num_processes;
	
	fseek(rsummary_file_handle, skip_bytes, SEEK_SET);
}

int lwm2_file_read_init_time_slice_file()
{
	rslices_file_handle = fopen(def_time_slice_file_name, "rb");
	if(rslices_file_handle == NULL)
	{
		return 0;
	}
	return 1;
}

void lwm2_file_read_finish_time_slice_file()
{
	CHECK_HANDLE(rslices_file_handle);
	fclose(rslices_file_handle);
}

void lwm2_file_read_process_mertric_val(unsigned long long *metric_value)
{
	CHECK_HANDLE(rslices_file_handle);

	fread(metric_value, sizeof(unsigned long long), 1, rslices_file_handle);
}

void lwm2_file_read_process_time_slice(unsigned long long *time_slice_metrics)
{
	CHECK_HANDLE(rslices_file_handle);
	
	fread(time_slice_metrics, sizeof(unsigned long long), rfile_definition.time_slice_metric_count, rslices_file_handle);
}

void lwm2_file_read_process_time_slice_with_count(int count, unsigned long long *time_slice_metrics)
{
	CHECK_HANDLE(rslices_file_handle);
	
	int read_count = count * rfile_definition.time_slice_metric_count;
	
	fread(time_slice_metrics, sizeof(unsigned long long), read_count, rslices_file_handle);
}

void lwm2_file_skip_time_slices(long num_time_slices)
{
	CHECK_HANDLE(rslices_file_handle);
	long skip_bytes = sizeof(unsigned long long) * num_time_slices * rfile_definition.time_slice_metric_count;
	fseek(rslices_file_handle, skip_bytes, SEEK_CUR);
}

void lwm2_file_skip_process_from_start(unsigned int num_processes)
{
	CHECK_HANDLE(rslices_file_handle);
	long skip_bytes = sizeof(unsigned long long) * rfile_definition.time_slice_count * rfile_definition.time_slice_metric_count * num_processes;
	fseek(rslices_file_handle, skip_bytes, SEEK_SET);
}

void lwm2_file_read_process_data(unsigned long long *proc_data, int time_slice_count, int time_slice_offset)
{
	proc_buff = (unsigned long long*) malloc(sizeof(unsigned long long) * time_slice_count * rfile_definition.time_slice_metric_count);
	
	long offset = time_slice_offset * rfile_definition.time_slice_metric_count * sizeof(unsigned long long);
	
	if(proc_buff == NULL)
	{
		return;
	}
	
	long proc_offset = rfile_definition.time_slice_count * rfile_definition.time_slice_metric_count * sizeof(unsigned long long);
	
	long eff_offset = proc_offset - (rfile_definition.time_slice_metric_count * time_slice_count * sizeof(unsigned long long));
	
	fseek(rslices_file_handle, offset, SEEK_SET);
	
	int i;
	for(i = 0; i < rfile_definition.process_count; i++)
	{
		fread(proc_buff, sizeof(unsigned long long), rfile_definition.time_slice_metric_count * time_slice_count, rslices_file_handle);
		
		lwm2file_cube_copy_time_slice_to_proc(proc_buff, proc_data, rfile_definition.process_count, time_slice_count, i);
		
		fseek(rslices_file_handle, eff_offset, SEEK_CUR);
	}
	
	free(proc_buff);
}

void lwm2file_cube_copy_time_slice_to_proc(unsigned long long* pbuff, unsigned long long* proc_data, int proc_count, int time_slice_count, int proc)
{
	int i;
	
	int metrics = time_slice_count * rfile_definition.time_slice_metric_count;
	
	for(i = 0; i < metrics; i++)
	{
		proc_data[proc + i * proc_count] = pbuff[i];
	}
}


