/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include "../include/lwm2-file-writer.h"
#include "../include/lwm2-file-common.h"
#include "../include/lwm2-file-writer-def.h"

#include <stdio.h>
#include <string.h>

#define SENSOR_NAME_LEN (34)
#define NODE_NAME_LEN (64)
#define LWM2_FORMAT_VERSION 1
#define FILE_NAME_LEN (1024)

#define CHECK_HANDLE(X) if((X) == NULL) { return; }

file_def_t file_definition;
FILE* time_slice_file_handle = NULL;
FILE* summary_file_handle = NULL;
char definition_file_name[FILE_NAME_LEN];
char time_slice_file_name[FILE_NAME_LEN];
char summary_file_name[FILE_NAME_LEN];

void lwm2_file_init_writer(char* file_name)
{
	sprintf(definition_file_name, "%s.definition", file_name);
	sprintf(time_slice_file_name, "%s.slices", file_name);
	sprintf(summary_file_name, "%s.digest", file_name);
}

void lwm2_file_close_definition_file()
{
	lwm2_file_write_def_finish();
}

void lwm2_file_init_time_slice_file()
{
	time_slice_file_handle = fopen(time_slice_file_name, "wb");
}

void lwm2_file_close_time_slice_file()
{
	if(time_slice_file_handle != NULL)
	{
		fclose(time_slice_file_handle);
	}
}

void lwm2_file_init_summary_file()
{
	summary_file_handle = fopen(summary_file_name, "wb");
}

void lwm2_file_close_summary_file()
{
	if(summary_file_handle != NULL)
	{
		fclose(summary_file_handle);
	}
}

void lwm2_file_set_time_slice_length(int time_slice_length)
{
	file_definition.time_slice_length = time_slice_length;
}

void lwm2_file_set_time_slice_metric_count(unsigned short time_slice_metric_count)
{
	file_definition.time_slice_metric_count = time_slice_metric_count;
}

void lwm2_file_set_summary_metric_count(unsigned short summary_metric_count)
{
	file_definition.summary_metric_count = summary_metric_count;
}

void lwm2_file_set_job_id(unsigned int job_id)
{
	file_definition.job_id = job_id;
}

void lwm2_file_set_process_count(unsigned int process_count)
{
	file_definition.process_count = process_count;
}

void lwm2_file_set_time_slice_count(unsigned long time_slice_count)
{
	file_definition.time_slice_count = time_slice_count;
}

void lwm2_file_set_time_slice_start_time(unsigned long time_slice_start_time)
{
	file_definition.time_slice_start_time = time_slice_start_time;
}

void lwm2_file_write_definition()
{
	file_definition.version = LWM2_FORMAT_VERSION;
	lwm2_file_write_def_file_def(definition_file_name, file_definition);
}

void lwm2_file_write_sensor_def(unsigned short sensor_id, char* sensor_name)
{
	sensor_def_t sensor_definition;
	
	sensor_definition.sensor_id = sensor_id;
	strncpy(sensor_definition.sensor_name, sensor_name, SENSOR_NAME_LEN);
	
	lwm2_file_write_def_sensor(sensor_definition);
}

void lwm2_file_write_process_mapping(unsigned int proc_id, char* node_name)
{
	process_map_t proc_mapping;
	
	proc_mapping.proc_id = proc_id;
	strncpy(proc_mapping.node_name, node_name, NODE_NAME_LEN);
	
	lwm2_file_write_def_process_mapping(proc_mapping);
}

void lwm2_file_write_summary_process_data(unsigned long long* summary_data)
{
	CHECK_HANDLE(summary_file_handle);
	
	fwrite(summary_data, sizeof(unsigned long long), file_definition.summary_metric_count, summary_file_handle);
}

void lwm2_file_append_process_metric_value(unsigned long long metric_val)
{
	CHECK_HANDLE(time_slice_file_handle);

	fwrite(&metric_val, sizeof(unsigned long long), 1, time_slice_file_handle);
}

void lwm2_file_append_process_time_slice(unsigned long long *time_slice_metrics)
{
	CHECK_HANDLE(time_slice_file_handle);

	fwrite(time_slice_metrics, sizeof(unsigned long long), file_definition.time_slice_metric_count, time_slice_file_handle);
}
