/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef LWM2_MEMORY_H_
#define LWM2_MEMORY_H_

#include "lwm2-config.h"

/*
 * Define thread group size and count. Thread specific data storage will be initialized using
 * these parameters.
 */
#define THREAD_GROUP_SIZE 4
#define THREAD_GROUP_COUNT 3
#define THREAD_SLICE_DEFAULT_SIZE 4096

#define MAX_THREADS (THREAD_GROUP_SIZE * THREAD_GROUP_COUNT)

/*
 * Defines the parameters for time-slice storage
 */
#define TIME_SLICE_COUNT (6 * 60 * 24)
#define TIME_SLICE_SIZE (LWM2_WRAPPER_COUNT * sizeof(struct metric_node))

/*
 * The memory allocated for the temporary buffer.
 */
#define TEMP_MEM_CHUNK (1048576)

#define END_TEMP_MEM_CHUNK (134217728)

int lwm2_memory_init(void);
void lwm2_memory_destroy(void);

void lwm2_memory_init_buffer();

void* lwm2_memory_new_end_temp_buffer_bytes(int bytes);

void lwm2_memory_free_end_temp_buffer();

unsigned long long* lwm2_memory_new_time_slice_storage();

void* lwm2_memory_new_time_slice_storage_bytes(int bytes);

void* lwm2_memory_new_temp_buffer_bytes(int bytes);

void lwm2_memory_free_temp_buffer();

void* lwm2_new_thread_storage();

#endif /* LWM2_MEMORY_H_ */
