/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "lwm2-pulse-thread.h"
#include "util/util_time_measurement.h"
#include "lwm2-signal.h"
#include "adapter/lwm2-mpi-adapter.h"

#include <pthread.h>
#include <time.h>
#include <math.h>

int lwm2_pulse_thread_cont = 1;
int is_new_time_slice = 0;

unsigned long lwm2_first_time_slice_time = 0;
unsigned long lwm2_last_time_slice_time = 0;

unsigned long lwm2_first_pulse_time_slice_time = 0;
unsigned long lwm2_last_pulse_time_slice_time = 0;

int lwm2_thread_is_pulse_thread = 0;

pthread_rwlock_t pulse_cont_lock;
pthread_rwlock_t time_slice_update_lock;

/*
 * Returns the time remaining till the next time-slice
 */
double lwm2_thread_get_time_till_time_slice()
{
	timestamp_t now_time = util_get_timestamp();
	
	double dsecs_now = floor(now_time);
	double dmilli_secs_now = now_time - dsecs_now;
	
	long lsecs_now = (long) dsecs_now;
	lsecs_now = lsecs_now + (is_new_time_slice * lwm2_time_slice_duration);
	int spent_secs = lsecs_now % lwm2_time_slice_duration;
	double sleep_time = lwm2_time_slice_duration - spent_secs - dmilli_secs_now;
	
	is_new_time_slice = 0;
	
	return sleep_time;
}

/*
 * Returns the time remaining till the next time-slice
 */
struct timespec lwm2_thread_get_timespec_till_time_slice()
{
//	timestamp_t now_time = util_get_timestamp();
//		
//	double dsecs_now = floor(now_time);
//	double dmilli_secs_now = now_time - dsecs_now;
//
//	long lsecs_now = (long) dsecs_now;
//	lsecs_now = lsecs_now + (is_new_time_slice * lwm2_time_slice_duration);
//	int rem_secs = lsecs_now % lwm2_time_slice_duration;
//	
//	struct timespec timespec_time;
//	timespec_time.tv_sec = lwm2_time_slice_duration - rem_secs - 1;
//	timespec_time.tv_nsec = (1 - dmilli_secs_now) * 1e9;
//	
//	is_new_time_slice = 0;
//	
//	return timespec_time;
	
	struct timespec curr_time = lwm2_util_get_timespec_time();
	
	struct timespec time_slice_time;
	
	int done_secs = (curr_time.tv_sec  % lwm2_time_slice_duration);
	
//	printf("Done secs are %d\n", done_secs);
	
	if(done_secs >= lwm2_time_slice_duration - 1)
	{
		done_secs = 0;
	}
	
//	printf("Done secs are now %d\n", done_secs);
	
	time_slice_time.tv_sec  = lwm2_time_slice_duration - done_secs - 1;
	time_slice_time.tv_nsec = 1e9 - curr_time.tv_nsec;
	
	return time_slice_time;
}

void lwm2_end_pulse_thread()
{
	pthread_rwlock_wrlock(&pulse_cont_lock);
	lwm2_pulse_thread_cont = 0;
	pthread_rwlock_unlock(&pulse_cont_lock);
}

/*
 * The function which handles time-slices
 */
void* lwm2_thread_pulse_function(void* pnull)
{
	struct timespec sleep_time, remaining_time;
	
	int time_slice_cnt = 0;
	
	while(lwm2_pulse_thread_cont)
	{
		sleep_time = lwm2_thread_get_timespec_till_time_slice();
		
		//Sleep can be interrupted in between. In that case, sleep again!
		while(nanosleep(&sleep_time, &remaining_time) != 0)
		{
			sleep_time = remaining_time;
		}
		pthread_rwlock_rdlock(&pulse_cont_lock);
		if(lwm2_pulse_thread_cont == 0)
		{
			pthread_rwlock_unlock(&pulse_cont_lock);
			break;
		}
		pthread_rwlock_unlock(&pulse_cont_lock);
		
		time_slice_cnt++;
		if(lwm2_first_pulse_time_slice_time == 0)
		{
			lwm2_first_pulse_time_slice_time = lwm2_last_pulse_time_slice_time;
		}

		lwm2_thread_lock_time_slice_update_lock();
		pthread_rwlock_rdlock(&pulse_cont_lock);
		if(lwm2_pulse_thread_cont == 0)
		{
			pthread_rwlock_unlock(&pulse_cont_lock);
			lwm2_thread_unlock_time_slice_update_lock();
			break;
		}
		pthread_rwlock_unlock(&pulse_cont_lock);
		lwm2_last_pulse_time_slice_time = lwm2_util_get_long_time();
		lwm2_signal_update_time_slice();
		lwm2_thread_unlock_time_slice_update_lock();

		is_new_time_slice = 1;
	}
	
	return NULL;
}

void lwm2_thread_lock_time_slice_update_lock()
{
	pthread_rwlock_wrlock(&time_slice_update_lock);
}

void lwm2_thread_unlock_time_slice_update_lock()
{
	pthread_rwlock_unlock(&time_slice_update_lock);
}

/*
 * Create and start the pulse-thread
 */
void lwm2_thread_start_pulse_thread()
{
	pthread_t pulse_thread;

	pthread_rwlock_init(&pulse_cont_lock, NULL);
	pthread_rwlock_init(&time_slice_update_lock, NULL);
	
	lwm2_thread_is_pulse_thread = 1;
	pthread_create(&pulse_thread, NULL, lwm2_thread_pulse_function, NULL);
	
	lwm2_thread_is_pulse_thread = 0;
}

