/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>

#include "lwm2-measurement-cuda.h"
#include "../../adapter/lwm2-cuda-adapter.h"

#include <signal.h>

#ifdef LWM2_MEASUREMENT_CUDA


#define CHECK_CU_ERROR(err, cufunc)                                     \
  if (err != CUDA_SUCCESS)                                              \
    {                                                                   \
      /*printf ("%s:%d: error %d for CUDA Driver API function '%s'\n", */   \
      /*        __FILE__, __LINE__, err, cufunc);        */              \
      /*exit(-1);      */                                               \
	  /*cuda_active = 0;	*/												\
    }

#define CHECK_CUPTI_ERROR(err, cuptifunc)                               \
  if (err != CUPTI_SUCCESS)                                             \
    {                                                                   \
      const char *errstr;                                               \
      cuptiGetResultString(err, &errstr);                               \
      /*printf ("%s:%d:Error %s for CUPTI API function '%s'.\n", */         \
      /*        __FILE__, __LINE__, errstr, cuptifunc);  */                 \
      /*exit(-1);  */													\
	  /*cuda_active = 0;		*/											\
    }

dev_data_t event_data[LWM2_CUDA_DEV_COUNT];

CUcontext lwm2_current_context[LWM2_CUDA_DEV_COUNT];

#endif

char cuda_active = 0;
      
void lwm2_measurement_setup_cuda()
{
#ifdef LWM2_MEASUREMENT_CUDA
	lwm2_cupti_init();
	cuda_active = 1;
	int i;
	for(i = 0; i < LWM2_CUDA_DEV_COUNT; i++)
	{
		lwm2_current_context[i] = 0;
	}
#endif
}

void lwm2_measurement_teardown_cuda()
{
	
}

void lwm2_measurement_handle_signal_cuda()
{
#ifdef LWM2_MEASUREMENT_CUDA
	if(lwm2_storage_get_flag_cuda())
	{
		lwm2_storage_increment_signal_cuda();
		if(lwm2_storage_get_flag_cuda_to_device())
		{
			lwm2_storage_increment_signal_cuda_to_device();
		}
		if(lwm2_storage_get_flag_cuda_to_host())
		{
			lwm2_storage_increment_signal_cuda_to_host();
		}
	}
#endif
	
#ifdef TREAT_CUDA_AS_MPI
	if(lwm2_storage_get_flag_cuda())
	{
		lwm2_storage_increment_signal_cuda();
	}
#endif
}

void lwm2_measurement_read_cuda_counters()
{
/*
#ifdef LWM2_MEASUREMENT_CUDA
	only_read_context_events_for_current();
	CUresult err;
	CUptiResult cuptierr;
	CUpti_EventID eventId;

	int device, event;		

	for(device = 0; device < lwm2_cuda_device_count; device++)
	{
		for(event = 0; event < LWM2_CUDA_EVENT_COUNT; event++)
		{
			if(event == LWM2_ADAPTER_CUDA_INSTRUCTION)
			{
				lwm2_storage_add_cuda_instructions(event_data[device].values[event]);
			}
			
			if(event == LWM2_ADAPTER_CUDA_DIV_BRANCH)
			{
				lwm2_storage_add_cuda_divergent_branches(event_data[device].values[event]);
			}
			
			event_data[device].values[event] = 0;
		}
	}
	
#endif
*/
}


#ifdef LWM2_MEASUREMENT_CUDA

int check_device(CUdevice dev)
{
	if(dev < 0 || dev > LWM2_CUDA_DEV_COUNT)
	{
		return 0;
	}
	return 1;
}

void get_context_device(CUcontext* context, CUdevice* device)
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	CUresult err;

	err = cuCtxGetCurrent(context);
	CHECK_CU_ERROR(err, "cuCtxGetCurrent");
	
	if(err != CUDA_SUCCESS)
	{
		*device = -1;
	}

	err = cuCtxGetDevice(device);
	CHECK_CU_ERROR(err, "cuCtxGetDevice");
	
	if(err != CUDA_SUCCESS)
	{
		*device = -1;
	}
	*/
}

void unsubscribe_context(CUdevice device)
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	CUptiResult cuptierr;

	if(lwm2_current_context[device] == 0)
	{
		return;
	}
	
	cuptierr = cuptiEventGroupDisable(event_data[device].event_group);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupDisable");

	cuptierr = cuptiEventGroupDestroy(event_data[device].event_group);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupDestroy");

	lwm2_current_context[device] = 0;
	*/
}

void read_context_events(CUcontext context, CUdevice device)
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	int i;
	CUptiResult cuptierr;
	size_t bytes_read;
	
	uint64_t values;
	
	for(i = 0; i < LWM2_CUDA_EVENT_COUNT; i++)
	{
		values = 0;
		cuptierr = cuptiEventGroupReadEvent(event_data[device].event_group, 
				CUPTI_EVENT_READ_FLAG_NONE, 
				lwm2_cuda_event_ids[i], &bytes_read, &values);
		
		CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupReadEvent");

		printf("Index %d, Event %d, Value: %lu\n",i, lwm2_cuda_event_ids[i], values); 

		if(cuptierr != CUPTI_SUCCESS)
		{
			return;
		}

		event_data[device].values[i] += values;
	}
	
	cuptierr = cuptiEventGroupResetAllEvents(event_data[device].event_group);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupResetAllEvents");
	*/
}

void only_read_context_events_for_current()
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	CUcontext cntx;
	CUdevice dev;

	get_context_device(&cntx, &dev);

	if(dev < 0 || check_device(dev) == 0)
	{
		return;
	}
	
	if(lwm2_current_context[dev] == 0)
	{
		return;
	}

	read_context_events(cntx, dev);
	*/
}

void read_context_events_for_current()
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	CUcontext cntx;
	CUdevice dev;

	get_context_device(&cntx, &dev);

	if(dev < 0 || check_device(dev) == 0)
	{
		return;
	}
	
	if(lwm2_current_context[dev] == 0)
	{
		return;
	}

	read_context_events(cntx, dev);

	unsubscribe_context(dev);
	*/
}

void subscribe_to_events_for_context(CUcontext context, CUdevice device)
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	int i;
	CUptiResult cuptierr;
	CUpti_EventID eventId;

	cuptierr = cuptiEventGroupCreate(context, &event_data[device].event_group, 0);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupCreate");

	for(i = 0; i < LWM2_CUDA_EVENT_COUNT; i++)
	{
		cuptierr = cuptiEventGetIdFromName(i, lwm2_get_event_for_device(i, device), &eventId);
		CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGetIdFromName");
					
		cuptierr = cuptiEventGroupAddEvent(event_data[device].event_group, eventId);
		CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupAddEvent");

			if(cuda_active == 0)
			{
				return;
			}

	}

	cuptierr = cuptiEventGroupEnable(event_data[device].event_group);
	CHECK_CUPTI_ERROR(cuptierr, "cuptiEventGroupEnable");

	if(cuptierr != CUPTI_SUCCESS)
	{
		unsubscribe_context(device);
		printf("Event group enable failed. Unsubscribed from the event group\n");
	}
	*/
}

void subscribe_to_events_for_current_context()
{
	if(cuda_active == 0)
	{
		return;
	}

	/*
	CUcontext cntx;
	CUdevice dev;
	
	get_context_device(&cntx, &dev);

	if(dev < 0 || check_device(dev) == 0)
	{
		return;
	}
	
	subscribe_to_events_for_context(cntx, dev);
	
	lwm2_current_context[dev] = cntx;
	*/
}


void CUPTIAPI lwm2_cuda_handle_context(void *userdata, CUpti_CallbackDomain domain,
                      CUpti_CallbackId cbid, const CUpti_CallbackData *cbInfo)
{
	/*
	CUresult err;
	CUcontext cntx;

	lwm2_cuda_function_call = 1;

	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxAttach && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
//		read_context_events_for_current();
	}
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxAttach && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
//		subscribe_to_events_for_current_context();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxDetach && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
//		read_context_events_for_current();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxDetach && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
//		read_context_events_for_current();
	}

	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		subscribe_to_events_for_current_context();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
		read_context_events_for_current();
	}

	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
//		subscribe_to_events_for_current_context();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxCreate && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
//		read_context_events_for_current();
	}
	
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxCreate && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		subscribe_to_events_for_current_context();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy && cbInfo->callbackSite == CUPTI_API_ENTER)
	{
		read_context_events_for_current();
	}
	
	if(cbid == CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy && cbInfo->callbackSite == CUPTI_API_EXIT)
	{
		
	}
	*/
}
#endif


