/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#ifdef LWM2_MEASUREMENT_HWCOUNTER
#include  <papi.h>
#endif

/* Utils */
#include "../../util/util_time_measurement.h"

#include "lwm2-measurement-hwcounter.h"

#include "../../adapter/lwm2-hwc-adapter.h"
#include "../../adapter/hwc/lwm2-hwc-papi.h"

#include "../../util/lwm2-log.h"

#include "../../lwm2-signal.h"
#include "../../lwm2-storage.h"

/* PAPI hardware counter interface */
#ifdef LWM2_MEASUREMENT_HWCOUNTER
#include "../../adapter/hwc/lwm2-hwc-papi.h"
#endif

/* OProfile hardware counter interface */
#ifdef LWM2_MEASUREMENT_HWCOUNTER_OPROFILE
/* TODO */
#endif

#ifdef LWM2_MEASUREMENT_HWCOUNTER

#endif

#ifdef LWM2_HWC_EXTERN_DEF
#include "../hwc/lwm2-hwc-extern.h"
#endif

unsigned long lwm2_hwc_last_read_duration;
unsigned long lwm2_hwc_last_read_time;

void lwm2_measurement_setup_hwcounter(void)
{
#ifdef LWM2_MEASUREMENT_HWCOUNTER
	lwm2_hwc_papi_init();
	
	lwm2_hwc_papi_setup((void*) &lwm2_proc_hwc_storage);
	
	lwm2_hwc_last_read_time = lwm2_util_get_long_time();

#endif
}

void lwm2_measurement_teardown_hwcounter(void)
{
#ifdef LWM2_MEASUREMENT_HWCOUNTER
	lwm2_hwc_papi_destroy(lwm2_proc_hwc_storage.event_set);
#endif
}

void lwm2_measurement_handle_signal_hwcounter()
{

}

#ifdef LWM2_MEASUREMENT_HWCOUNTER
void lwm2_measurement_hwc_papi_read_reset_events(struct lwm2_hwc_storage* hwc_storage)
{
//#ifdef LWM2_MEASUREMENT_HWCOUNTER
	if(hwc_storage->event_set == PAPI_NULL)
	{
		return;
	}
	
//	printf("Reading hardware counters from event set: %d, with counters: %d\n", hwc_storage->event_set, hwc_storage->num_counters);
	
	PAPI_read(hwc_storage->event_set, &hwc_storage->values[0]);
	
	unsigned long tmp_time = lwm2_util_get_long_time();
	lwm2_hwc_last_read_duration = tmp_time - lwm2_hwc_last_read_time;
	lwm2_hwc_last_read_time = tmp_time;
	
	int i;
	long_long val;
	
	unsigned long long l1_misses = 0;
	unsigned long long l1_accesses = 0;
	
#ifdef LWM2_HWC_EXTERN_DEF
	unsigned long long custom_hwc_values[LWM2_CUSTOM_BASIC_HWC_COUNT];
	unsigned long long storage_hwc_values[LWM2_CUSTOM_STORAGE_HWC_COUNT];
#endif
	
	for(i = 0; i < hwc_storage->num_counters; i++)
	{
		val = hwc_storage->values[i] - hwc_storage->old_values[i];
		
#ifndef LWM2_HWC_EXTERN_DEF
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_CYCLES)
		{
			lwm2_storage_set_cpu_cycles(val);
		}
		
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_FP_OPS)
		{
			lwm2_storage_set_cpu_flops(val);
		}
		
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_FP_INS)
		{
			lwm2_storage_set_cpu_floins(val);
		}

		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_L1_DATA_ACCESS)
		{
			lwm2_storage_set_l1_data_accesses(val);
		}
		
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_L1_DATA_MISSES)
		{
			lwm2_storage_set_l1_data_misses(val);
		}
		
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_INST)
		{
			lwm2_storage_set_cpu_instructions(val);
		}
		
		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_L1_LOAD_MISSES)
		{
			l1_misses += val;
		}

		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_L1_STORE_MISSES)
		{
			l1_misses += val;
		}

		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_LOAD_INST)
		{
			l1_accesses += val;
		}

		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_STORE_INST)
		{
			l1_accesses += val;
		}

		if(hwc_storage->event_ids[i] == LWM2_HWCOUNTER_LOAD_STORE_INST)
		{
			lwm2_storage_set_l1_data_accesses(val);
		}

#else
		custom_hwc_values[i] = val;
#endif
		
		hwc_storage->old_values[i] = hwc_storage->values[i];
	}
	
#ifndef LWM2_HWC_EXTERN_DEF
	if(l1_misses != 0)
	{
		lwm2_storage_set_l1_data_misses(l1_misses);
	}
	
	if(l1_accesses != 0)
	{
		lwm2_storage_set_l1_data_accesses(l1_accesses);
	}
#endif
	
#ifdef LWM2_HWC_EXTERN_DEF
	lwm2_hwc_custom_convert_for_storage(custom_hwc_values, storage_hwc_values);
	lwm2_storage_set_custom_hwcs(storage_hwc_values);
#endif
	
//#endif
}
#endif
