/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "lwm2-output-file.h"

#include "../../lwm2-config.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <mpi.h>	
#include <stdlib.h>

#include "../../adapter/lwm2-mpi-adapter.h"
#include "../../lwm2-memory.h"
#include "../../external/lwm2file/include/lwm2-file-writer.h"
#include "../../util/util_time_measurement.h"
#include "../../lwm2-storage.h"
#include "../../lwm2-signal.h"
#include "../../lwm2-pulse-thread.h"

#ifdef LWM2_HWC_EXTERN_DEF
#include "../hwc/lwm2-hwc-extern.h"
#endif

char lwm2_output_l2f_filename[FILENAME_LEN];
char* lwm2_output_filename_p;

int time_slice_count = 0;
long max_time_slice_count = 0;
long lwm2_max_time_slices_written = 0;

unsigned long min_first_time_slice_time = 0;
unsigned long max_last_time_slice_time = 0;
int total_time_slices = 0;

char lwm2_host_name[HOSTNAME_LENGTH];
char* lwm2_host_table = NULL;

int time_slice_metrics_count = LWM2_TOTAL_METRIC_COUNT - LWM2_THREAD_METRIC_COUNT;

void lwm2_output_file_init()
{
#ifdef LWM2_OUTPUT_LWM2FILE
	lwm2_output_file_get_l2f_filename();
	
	lwm2_file_init_writer(lwm2_output_l2f_filename);

	lwm2_file_set_time_slice_length(lwm2_time_slice_duration);

	lwm2_file_set_time_slice_metric_count(LWM2_TOTAL_METRIC_COUNT - LWM2_THREAD_METRIC_COUNT);
	
	lwm2_file_set_summary_metric_count(LWM2_TOTAL_METRIC_COUNT);

	lwm2_file_set_job_id(lwm2_job_id);

	lwm2_file_set_process_count(lwm2_mpi_comm_size);

	lwm2_file_set_time_slice_count(lwm2_max_time_slices_written);

	unsigned long first_time_slice = (unsigned long) lwm2_first_time_slice;
	lwm2_file_set_time_slice_start_time(first_time_slice);
	
	lwm2_file_write_definition();
	
	unsigned short i;
	for(i = 0; i < LWM2_SIGNAL_METRIC_COUNT; i++)
	{
		lwm2_file_write_sensor_def(i, lwm2_signal_metric_names[i]);
	}
	
#ifndef LWM2_HWC_EXTERN_DEF
	for(i = LWM2_SIGNAL_METRIC_COUNT; i < LWM2_HWC_METRIC_COUNT; i++)
	{
		lwm2_file_write_sensor_def(i, lwm2_hwc_metric_names[i - LWM2_SIGNAL_METRIC_COUNT]);
	}
	
#else
	
	for(i = LWM2_SIGNAL_METRIC_COUNT; i < LWM2_HWC_METRIC_COUNT; i++)
	{
		lwm2_file_write_sensor_def(i, lwm2_custom_storage_hwc_names[i - LWM2_SIGNAL_METRIC_COUNT]);
	}
#endif
	
	for(i = LWM2_HWC_METRIC_COUNT; i < LWM2_TOTAL_METRIC_COUNT; i++)
	{
		lwm2_file_write_sensor_def(i, lwm2_tsliced_metric_names[i - LWM2_HWC_METRIC_COUNT]);
	}
	
	for(i = 0; i < lwm2_mpi_comm_size; i++)
	{
		lwm2_file_write_process_mapping(i, &lwm2_host_table[i * HOSTNAME_LENGTH]);
	}
	lwm2_file_close_definition_file();
#endif
}

void lwm2_output_file_write_data(int count, unsigned long long *metrics_data)
{
#ifdef LWM2_OUTPUT_LWM2FILE
	int i, j;
	for(i = 0; i < count; i++)
	{
		for(j = 0; j < time_slice_metrics_count; j++)
		{
			lwm2_file_append_process_metric_value(metrics_data[i * time_slice_metrics_count + j]);
		}
	}
#endif
}

int lwm2_get_starting_time_slice_count(unsigned long my_time, unsigned long min_time)
{
	//Start time will be on time-slice boundry, except for runs smaller than a time-slice
	if(my_time % lwm2_time_slice_duration != 0)
	{
		my_time = my_time + (lwm2_time_slice_duration - (my_time % lwm2_time_slice_duration));
	}
	if(min_time % lwm2_time_slice_duration != 0)
	{
		min_time = min_time + (lwm2_time_slice_duration - (min_time % lwm2_time_slice_duration));
	}
	int starting_count = (my_time - min_time) / lwm2_time_slice_duration;
	return starting_count;
}

int lwm2_get_ending_time_slice_count(unsigned long my_time, unsigned long max_time)
{
	//Ending time will never be on time-slice boundry, except when the application ended at
	//that time. But even then, there will be an extra time-slice for that small time after
	//time-slice time.
	if(my_time % lwm2_time_slice_duration != 0)
	{
		my_time = my_time + (lwm2_time_slice_duration - (my_time % lwm2_time_slice_duration));
	}
	else
	{
		my_time = my_time + lwm2_time_slice_duration;
	}

	if(max_time % lwm2_time_slice_duration != 0)
	{
		max_time = max_time + (lwm2_time_slice_duration - (max_time % lwm2_time_slice_duration));
	}
	else
	{
		max_time = max_time + lwm2_time_slice_duration;
	}
	int ending_count = (max_time - my_time) / lwm2_time_slice_duration;
	return ending_count;
}

int lwm2_get_new_time_slice_count(unsigned long start_time, unsigned long end_time)
{
	int start_rem = start_time % lwm2_time_slice_duration;
	int end_rem = end_time % lwm2_time_slice_duration;

	unsigned long start_time_end_tslice = start_time;
	unsigned long end_time_end_tslice = end_time;

	if(start_rem != 0)
	{
		start_time_end_tslice = start_time + (lwm2_time_slice_duration - start_rem);
	}

	if(end_rem != 0)
	{
		end_time_end_tslice = end_time + (lwm2_time_slice_duration - end_rem);
	}

	int tslice_count = (end_time_end_tslice - start_time_end_tslice) / lwm2_time_slice_duration;

	return tslice_count;
}

unsigned long lwm2_get_starting_slice_end_time()
{
	unsigned long start_time = lwm2_first_pulse_time_slice_time;
	//if no time-slice was recorded by the pulse thread. This means the application duration was shorter
	//than the time-slice
	if(start_time == 0)
	{
		start_time = lwm2_first_time_slice_time + lwm2_time_slice_duration - (lwm2_first_time_slice_time % lwm2_time_slice_duration);
	}
	//If there was a time-slice, make sure the pulse-thread executed on the right time
	else
	{
		int start_rem = start_time % lwm2_time_slice_duration;
		//if there was a bit of variance in time, get to the nearest time-slice boundry
		if(start_rem != 0)
		{
			if(start_rem < (lwm2_time_slice_duration / 2))
			{
				start_time = start_time - start_rem;
			}
			else
			{
				start_time = start_time + lwm2_time_slice_duration - start_rem;
			}
		}
	}
	return start_time;
}

unsigned long lwm2_get_ending_slice_end_time()
{
	unsigned long end_time = lwm2_last_time_slice_time;
	//There will always be a time-slice at the end, which was not triggered by the pulse thread.
	//Therefore, the time has to be adjusted for the ending time-slice boundary
	if(end_time % lwm2_time_slice_duration == 0)
	{
		end_time = end_time + lwm2_time_slice_duration;
	}
	else
	{
		end_time = lwm2_last_time_slice_time + lwm2_time_slice_duration - (lwm2_last_time_slice_time % lwm2_time_slice_duration);
	}

	unsigned long tslice_end_time = lwm2_last_pulse_time_slice_time;
		int end_rem = tslice_end_time % lwm2_time_slice_duration;
		//if there was a bit of variance in time, get to the nearest time-slice boundry
		if(end_rem != 0)
		{
			if(end_rem < (lwm2_time_slice_duration / 2))
			{
				tslice_end_time = tslice_end_time - end_rem;
			}
			else
			{
				tslice_end_time = tslice_end_time + lwm2_time_slice_duration - end_rem;
			}
		}
		if(tslice_end_time == end_time)
		{
			end_time = end_time + lwm2_time_slice_duration;
		}
	return end_time;
}

int lwm2_get_time_slice_count(unsigned long start_time, unsigned long end_time)
{
	unsigned long total_time = end_time - start_time;
	int tslices = total_time / lwm2_time_slice_duration;
	tslices++;
	return tslices;
}

void lwm2_output_file_write_definition_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE

	struct metric_node* iter_node;

	timestamp_t d_time;

	if(lwm2_mpi_rank == 0)
	{
		lwm2_host_table = lwm2_memory_new_end_temp_buffer_bytes(lwm2_mpi_comm_size * HOSTNAME_LENGTH * sizeof(char));
	}

	PMPI_Gather(lwm2_host_name, HOSTNAME_LENGTH, MPI_CHAR, lwm2_host_table, HOSTNAME_LENGTH, MPI_CHAR, 0, MPI_COMM_WORLD);

	lwm2_first_time_slice_time = lwm2_get_starting_slice_end_time();
	lwm2_last_time_slice_time = lwm2_get_ending_slice_end_time();
	PMPI_Reduce(&lwm2_first_time_slice_time, &min_first_time_slice_time, 1, MPI_UNSIGNED_LONG, MPI_MIN, 0 , MPI_COMM_WORLD);
	PMPI_Reduce(&lwm2_last_time_slice_time, &max_last_time_slice_time, 1, MPI_UNSIGNED_LONG, MPI_MAX, 0 , MPI_COMM_WORLD);

	PMPI_Bcast(&min_first_time_slice_time, 1, MPI_UNSIGNED_LONG, 0, MPI_COMM_WORLD);
	PMPI_Bcast(&max_last_time_slice_time, 1, MPI_UNSIGNED_LONG, 0, MPI_COMM_WORLD);
	
	total_time_slices = lwm2_get_time_slice_count(min_first_time_slice_time, max_last_time_slice_time);

	lwm2_max_time_slices_written = total_time_slices + 1;
	if(lwm2_mpi_rank == 0)
	{
		lwm2_output_file_init();
	}
#endif
}

void lwm2_output_file_write_non_mpi_definition_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE

	struct metric_node* iter_node;


	lwm2_host_table = lwm2_memory_new_temp_buffer_bytes(HOSTNAME_LENGTH * sizeof(char));

	strncpy(lwm2_host_table, lwm2_host_name, HOSTNAME_LENGTH);

	int total_time_slices = lwm2_get_time_slice_count(lwm2_first_time_slice_time, lwm2_last_time_slice_time);

	lwm2_mpi_comm_size = 1;

	lwm2_max_time_slices_written = total_time_slices + 1;
	lwm2_output_file_init();
#endif
}

void lwm2_output_file_write_summary_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE
	MPI_Status stat;

	if(lwm2_mpi_rank == 0)
	{
		lwm2_file_init_summary_file();
		
		lwm2_file_write_summary_process_data(lwm2_metric_total);

		int i;
		unsigned long long lwm2_proc_summary[LWM2_TOTAL_METRIC_COUNT];
		for(i = 1; i < lwm2_mpi_comm_size; i++)
		{
			MPI_Recv(lwm2_proc_summary, LWM2_TOTAL_METRIC_COUNT, MPI_UNSIGNED_LONG_LONG, i, 42, MPI_COMM_WORLD, &stat);
			lwm2_file_write_summary_process_data(lwm2_proc_summary);
		}

		lwm2_file_close_summary_file();
	}
	else
	{
		MPI_Ssend(lwm2_metric_total, LWM2_TOTAL_METRIC_COUNT, MPI_UNSIGNED_LONG_LONG, 0, 42, MPI_COMM_WORLD);
	}
#endif
}

void lwm2_output_file_write_non_mpi_summary_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE
	
	lwm2_file_init_summary_file();
		
	lwm2_file_write_summary_process_data(lwm2_metric_total);

		
	lwm2_file_close_summary_file();
#endif
}

void lwm2_output_file_write_time_slice_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE
	MPI_Status stat;

		int starting_tslices = lwm2_get_time_slice_count(min_first_time_slice_time, lwm2_first_time_slice_time);
		starting_tslices--;
		int ending_tslices = lwm2_get_time_slice_count(lwm2_last_time_slice_time, max_last_time_slice_time);

	time_slice_count = 0;
	if(lwm2_mpi_rank == 0)
	{
		lwm2_file_init_time_slice_file();

		struct metric_node dummy_node;
		int i;
		for(i = 0; i < LWM2_WRAPPER_COUNT; i++)
		{
			dummy_node.metrics_data[i] = 0;
		}

		for(i = 0; i < starting_tslices; i++)
		{
			lwm2_output_file_write_data(1, dummy_node.metrics_data);
			time_slice_count++;
		}

		struct metric_node* iter_node;
		for(iter_node = list_head; iter_node != NULL && iter_node->next != NULL; iter_node = iter_node->next)
		{
			lwm2_output_file_write_data(1, iter_node->metrics_data);
			time_slice_count++;
		}

		while(time_slice_count <= total_time_slices)
		{
			lwm2_output_file_write_data(1, dummy_node.metrics_data);
			time_slice_count++;
		}

		int j;
		for(i = 1; i < lwm2_mpi_comm_size; i++)
		{
			for(j = 0; j <= total_time_slices; j++)
			{

				MPI_Recv(dummy_node.metrics_data, time_slice_metrics_count, MPI_UNSIGNED_LONG_LONG, i, 42, MPI_COMM_WORLD, &stat);
				lwm2_output_file_write_data(1, dummy_node.metrics_data);
			}
		}

		lwm2_file_close_time_slice_file();

	}
	else
	{
		
		struct metric_node dummy_node;
        int i;
        for(i = 0; i < LWM2_WRAPPER_COUNT; i++)
        {   
            dummy_node.metrics_data[i] = 0;
        }

		for(i = 0; i < starting_tslices; i++)
        {   
			MPI_Ssend(dummy_node.metrics_data, time_slice_metrics_count, MPI_UNSIGNED_LONG_LONG, 0, 42, MPI_COMM_WORLD);
			time_slice_count++;
        }

		struct metric_node* iter_node;
		
		for(iter_node = list_head; iter_node != NULL && iter_node->next != NULL; iter_node = iter_node->next)
		{
			MPI_Ssend(iter_node->metrics_data, time_slice_metrics_count, MPI_UNSIGNED_LONG_LONG, 0, 42, MPI_COMM_WORLD);
			time_slice_count++;
		}

		while(time_slice_count <= total_time_slices)
        {   
			MPI_Ssend(dummy_node.metrics_data, time_slice_metrics_count, MPI_UNSIGNED_LONG_LONG, 0, 42, MPI_COMM_WORLD);
			time_slice_count++;
        }
	}
#endif
}

void lwm2_output_file_write_non_mpi_time_slice_file()
{
#ifdef LWM2_OUTPUT_LWM2FILE
		lwm2_file_init_time_slice_file();
		time_slice_count = 0;
		struct metric_node* iter_node;
		
		for(iter_node = list_head; iter_node != NULL; iter_node = iter_node->next)
		{
			lwm2_output_file_write_data(1, iter_node->metrics_data);
			time_slice_count++;
		}

		lwm2_file_close_time_slice_file();
#endif
}

void lwm2_output_file_write_file()
{
//	gethostname(lwm2_host_name, HOSTNAME_LENGTH);
#ifdef LWM2_OUTPUT_LWM2FILE
	//if environment variable says don't write, then don't.
	char* should_write = getenv("LWM2_WRITE_FILE");
	if(should_write != NULL)
	{
		if(strncmp(should_write, "NO", 2) == 0)
		{
			return;
		}
		
		lwm2_output_file_write_definition_file();
		lwm2_output_file_write_summary_file();
		
		if(strncmp(should_write, "TIME_SLICES", 11) == 0)
		{
			lwm2_output_file_write_time_slice_file();
		}
	}
	else
	{
		lwm2_output_file_write_definition_file();
		lwm2_output_file_write_summary_file();
	}
#endif
}

void lwm2_output_file_write_non_mpi_app_file()
{
//	gethostname(lwm2_host_name, HOSTNAME_LENGTH);
#ifdef LWM2_OUTPUT_LWM2FILE
	//if environment variable says don't write, then don't.
	char* should_write = getenv("LWM2_WRITE_FILE");
	if(should_write != NULL)
	{
		if(strncmp(should_write, "NO", 2) == 0)
		{
			return;
		}
		
		lwm2_output_file_write_non_mpi_definition_file();
		lwm2_output_file_write_non_mpi_summary_file();
		
		if(strncmp(should_write, "TIME_SLICES", 11) == 0)
		{
			lwm2_output_file_write_non_mpi_time_slice_file();
		}
	}
	else
	{
		lwm2_output_file_write_non_mpi_definition_file();
		lwm2_output_file_write_non_mpi_summary_file();
	}
#endif
}

void lwm2_output_file_get_l2f_filename()
{
	gethostname(lwm2_host_name, HOSTNAME_LENGTH);
#ifdef LWM2_OUTPUT_LWM2FILE
	

	
	char tmp_filename[256];
	char* hopsa_file_name;
	
#ifdef LWM2_CLUSTRX_SOCKET	
	sprintf(tmp_filename, "hopsa-lwm2-%s-%ld", lwm2_host_name, lwm2_long_start_time);
	hopsa_file_name = getenv("HOPSA_STORE_LWM2");
#else
	sprintf(tmp_filename, "lwm2.j%d.t%ld", lwm2_job_id, lwm2_long_start_time);
	hopsa_file_name = getenv("LWM2_OUTPUT_FILENAME");
#endif
	
	if(hopsa_file_name != NULL)
	{
		if(hopsa_file_name[0] != '/')
		{
			strncpy(tmp_filename, hopsa_file_name, 256);
		}
		else
		{
			strncpy(lwm2_output_l2f_filename, hopsa_file_name, FILENAME_LEN);
			return;
		}
	}
	
	char* hopsa_file_dir;
#ifdef LWM2_CLUSTRX_SOCKET	
	hopsa_file_dir = getenv("HOPSA_STORE_DIR");
#else
	hopsa_file_dir = getenv("LWM2_OUTPUT_DIR");
#endif
	
	if(hopsa_file_dir != NULL)
	{
		int len = strlen(hopsa_file_dir);
		
		//If the path length is too large, save in current directory
		if(len > FILENAME_LEN - strlen(tmp_filename))
		{
			strcpy(lwm2_output_l2f_filename, tmp_filename);
		}
		else
		{
			if(hopsa_file_dir[len - 1] == '/')
			{
				sprintf(lwm2_output_l2f_filename, "%s%s", hopsa_file_dir, tmp_filename);
			}
			else
			{
				sprintf(lwm2_output_l2f_filename, "%s/%s", hopsa_file_dir, tmp_filename);
			}
		}
	}
	else
	{
		strcpy(lwm2_output_l2f_filename, tmp_filename);
	}	
	
#endif //LWM2_OUTPUT_LWM2FILE
}
