/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <arpa/inet.h>

#include "../../lwm2-storage.h"
#include "../../lwm2-signal.h"
#include "../../adapter/lwm2-mpi-adapter.h"

#include "lwm2-output-socket.h"

int lwm2_is_client_connected = 0;


int client_socket;

int msg_len;
short msg_type;

#define SHOULD_RECONNECT(x) if((x) == -1) { set_reconnect(); return; }

int should_reconnect = 1;

void set_reconnect()
{
	should_reconnect = 1;
	lwm2_is_client_connected = 0;
	printf("Disconnecting because of error\n");
}

unsigned long long lwm2_htonllu(unsigned long long data)
{
	unsigned int test_int = 123456;
	if(test_int != htonl(test_int))
	{
		unsigned long long tmp;
		unsigned int p1, p2;

		p1 = (data & 0xFFFFFFFF00000000) >> (sizeof(int) * 8);
		p2 = (data & 0x00000000FFFFFFFF);

		p1 = htonl(p1);
		p2 = htonl(p2);

		tmp = p2;
		tmp = tmp << (sizeof(int) * 8);
		tmp = tmp | p1;

		return tmp;
	}
	return data;
}

void lwm2_socket_connect()
{
	
#ifdef LWM2_CLUSTRX_SOCKET
	int len, flags;
	struct sockaddr_un server_addr;
	
//	printf("Connecting to node server\n");
	
	should_reconnect = 0;
	
	client_socket = socket(AF_UNIX, SOCK_STREAM, 0);
	
	if(client_socket == -1)
	{
		lwm2_is_client_connected = 0;
		return;
	}
	
	flags = fcntl(client_socket, F_GETFL, 0);
	if(flags < 0)
	{
		lwm2_is_client_connected = 0;
		close(client_socket);
		return;
	}
	
	if(fcntl(client_socket, F_SETFL, flags | O_NONBLOCK) < 0)
	{
		lwm2_is_client_connected = 0;
		close(client_socket);
		return;
	}
	
//	printf("Made socket non-blocking\n");
	
	server_addr.sun_family = AF_UNIX;
	strcpy(server_addr.sun_path, "/lustre/jhome5/grs200/grs20011/comm_socket/clustrx.socket");
//	strcpy(server_addr.sun_path, "/var/run/nmhopsad.socket");
	len = strlen(server_addr.sun_path) + sizeof(server_addr.sun_family);
	
	if(connect(client_socket, (struct sockaddr *) &server_addr, len) == -1)
	{
		lwm2_is_client_connected = 0;
		close(client_socket);
		
		printf("Connection failed!\n");
		perror("connect: ");
		return;
	}
	
//	printf("Connected to server\n");
	lwm2_is_client_connected = 1;
	
	msg_len = sizeof(short) + sizeof(int) + sizeof(int);
	msg_len = htonl(msg_len);
	msg_type = 2;
	msg_type = htons(msg_type);

	unsigned int net_job_id = htonl(lwm2_job_id);
	unsigned int net_mpi_rank = htonl(lwm2_mpi_rank);

	SHOULD_RECONNECT(send(client_socket, &msg_len, sizeof(msg_len), 0))
	SHOULD_RECONNECT(send(client_socket, &msg_type, sizeof(msg_type), 0))
	SHOULD_RECONNECT(send(client_socket, &net_job_id, sizeof(net_job_id), 0))
	SHOULD_RECONNECT(send(client_socket, &net_mpi_rank, sizeof(net_mpi_rank), 0))
#endif
}

void lwm2_socket_send_data()
{
#ifdef LWM2_CLUSTRX_SOCKET
//	printf("Send data for time-slice called\n");
	if(lwm2_is_client_connected == 0 && should_reconnect != 1)
	{
		return;
	}
	
	if(should_reconnect == 1)
	{
		lwm2_socket_connect();
	}
	
//	printf("Sending time-slice data to node server\n");
	
	unsigned long long metrics_to_send[LWM2_WRAPPER_COUNT];
	lwm2_storage_copy_metrics(metrics_to_send);
	
	msg_len = sizeof(short) + sizeof(metrics_to_send);
	msg_type = 1;
	msg_len = htonl(msg_len);
	msg_type = htons(msg_type);

	SHOULD_RECONNECT(send(client_socket, &msg_len, sizeof(msg_len), 0))
	SHOULD_RECONNECT(send(client_socket, &msg_type, sizeof(msg_type), 0))
	
	printf("Sending %d metrics\n", LWM2_WRAPPER_COUNT);
	
	int i;
//	for(i = 0; i < LWM2_METRIC_COUNT; i++)
//	{
		unsigned long long net_metric;
		for(i = 0; i < LWM2_WRAPPER_COUNT; i++)
		{
			net_metric = lwm2_htonllu(metrics_to_send[i]);
			SHOULD_RECONNECT(send(client_socket, &net_metric, sizeof(net_metric),0))
			printf("[%d], sensor %d, value: %llu\n", lwm2_mpi_rank, (i+1), metrics_to_send[i]);
		}
//	}
	
//	printf("Time-slice data sent!\n");
#endif
}

void lwm2_socket_disconnect()
{
#ifdef LWM2_CLUSTRX_SOCKET
//	printf("****************** Closing the Socket ********************\n");
	if(lwm2_is_client_connected == 0)
	{
		return;
	}
	close(client_socket);
#endif
}
