/****************************************************************************
**  Copyright (c) 2011-2012                                                **
**  German Research School for Simulation Sciences GmbH					   **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "lwm2-output-summary.h"
#include "../../lwm2-storage.h"
#include "../../util/util_time_measurement.h"
#include "../../lwm2-signal.h"
#include "../../adapter/lwm2-mpi-adapter.h"
#include "../../adapter/hwc/lwm2-hwc-papi.h"
#include "../../adapter/lwm2-cuda-adapter.h"

#ifdef LWM2_HWC_EXTERN_DEF
#include "../hwc/lwm2-hwc-extern.h"
#endif

#include <mpi.h>
#include <stdlib.h>
#include <string.h>

unsigned long long lwm2_metric_sum[LWM2_TOTAL_METRIC_COUNT];
unsigned long long lwm2_metric_min[LWM2_TOTAL_METRIC_COUNT];
unsigned long long lwm2_metric_max[LWM2_TOTAL_METRIC_COUNT];

#define MPI_TIME_THRESHOLD 20

enum {
	SIGNAL_SERIAL = 0,
	SIGNAL_MPI,
	SIGNAL_FILE_IO,
	SIGNAL_MPI_IO,
	SIGNAL_MPI_P2P,
	SIGNAL_MPI_COLL,
	SIGNAL_CUDA,
	SIGNAL_CUDA_TO_DEVICE,
	SIGNAL_CUDA_TO_HOST,
	RATIO_CNT
};

double local_signal_ratios[RATIO_CNT];
double sum_signal_ratios[RATIO_CNT];
double min_signal_ratios[RATIO_CNT];
double max_signal_ratios[RATIO_CNT];

unsigned long long local_metric_p2p_total_msg;
unsigned long long sum_metric_p2p_total_msg;
unsigned long long min_metric_p2p_total_msg;
unsigned long long max_metric_p2p_total_msg;

#ifndef LWM2_HWC_EXTERN_DEF

enum {
	HWC_CPI = 0,
	HWC_FLOPS,
	HWC_FLOINS,
	HWC_MEM,
	HWC_L1,
	HWC_TOTAL
};

double local_hwc_ratios[HWC_TOTAL];
double sum_hwc_ratios[HWC_TOTAL];
double min_hwc_ratios[HWC_TOTAL];
double max_hwc_ratios[HWC_TOTAL];

#else

double local_custom_summary_hwc[LWM2_CUSTOM_SUMMARY_HWC_COUNT];
double sum_custom_summary_hwc[LWM2_CUSTOM_SUMMARY_HWC_COUNT];
double min_custom_summary_hwc[LWM2_CUSTOM_SUMMARY_HWC_COUNT];
double max_custom_summary_hwc[LWM2_CUSTOM_SUMMARY_HWC_COUNT];
#endif

unsigned long long min_mpi_comm_data[COMM_DATA_TOTAL];
unsigned long long max_mpi_comm_data[COMM_DATA_TOTAL];

enum {
	MPI_TRANS_RATE_P2P = 0,
	MPI_TRANS_RATE_COLL,
	MPI_FREQ_P2P,
	MPI_FREQ_COLL,
	OMP_EFF_THRDS,
	TOTAL_EXEC_TIME,
	MPI_RATIO_TOTAL,
};

double local_mpi_trans_rate[MPI_RATIO_TOTAL];
double sum_mpi_trans_rate[MPI_RATIO_TOTAL];
double min_mpi_trans_rate[MPI_RATIO_TOTAL];
double max_mpi_trans_rate[MPI_RATIO_TOTAL];

enum {
	CUDA_DATA_TO_DEVICE,
	CUDA_DATA_TO_HOST,
	CUDA_DATA_COUNT
};

unsigned long long local_cuda_data[CUDA_DATA_COUNT];
unsigned long long sum_cuda_data[CUDA_DATA_COUNT];
unsigned long long min_cuda_data[CUDA_DATA_COUNT];
unsigned long long max_cuda_data[CUDA_DATA_COUNT];

enum {
	CUDA_DATA_PER_MSG_TO_DEVICE,
	CUDA_DATA_PER_MSG_TO_HOST,
	CUDA_RATE_MSG_TO_DEVICE,
	CUDA_RATE_MSG_TO_HOST,
	CUDA_DATA_DETAIL_COUNT
};

double local_cuda_data_det[CUDA_DATA_DETAIL_COUNT];
double sum_cuda_data_det[CUDA_DATA_DETAIL_COUNT];
double min_cuda_data_det[CUDA_DATA_DETAIL_COUNT];
double max_cuda_data_det[CUDA_DATA_DETAIL_COUNT];

int lwm2_output_should_print_summary()
{
	char* should_summary = getenv("LWM2_CONSOLE_SUMMARY");

	if(should_summary == NULL)
	{
		return 0;
	}
	return 1;
}

int lwm2_output_should_print_cuda()
{
	return lwm2_cuda_function_call;
}

int lwm2_output_should_print_verbose()
{
	char* print_verbose = getenv("LWM2_CONSOLE_SUMMARY");	
	if(print_verbose != NULL)
	{
		if(strcmp(print_verbose, "VERBOSE") == 0)
		{
			return 1;
		}
	}
	return 0;
}

void lwm2_output_summary_calculate_ratios_non_mpi()
{
	if(lwm2_output_should_print_summary() == 0)
	{
		return;
	}

//	local_signal_ratios[SIGNAL_MPI] = lwm2_metric_total[LWM2_SIGNAL_MPI] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_FILE_IO] = lwm2_metric_total[LWM2_SIGNAL_FILE_IO] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
//	local_signal_ratios[SIGNAL_MPI_IO] = lwm2_metric_total[LWM2_SIGNAL_MPI_IO] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
//	local_signal_ratios[SIGNAL_MPI_P2P] = lwm2_metric_total[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
//	local_signal_ratios[SIGNAL_MPI_COLL] = lwm2_metric_total[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA] = lwm2_metric_total[LWM2_SIGNAL_CUDA] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA_TO_DEVICE] = lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_DEVICE] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA_TO_HOST] = lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_HOST] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	
//	local_metric_p2p_total_msg = lwm2_metric_total[LWM2_METRIC_P2P_SENT_MSGS] + lwm2_metric_total[LWM2_METRIC_P2P_RECV_MSGS];
	
#ifndef LWM2_HWC_EXTERN_DEF
	local_hwc_ratios[HWC_CPI] = lwm2_metric_total[LWM2_METRIC_CYCLES] / (double) lwm2_metric_total[LWM2_METRIC_INSTRUCTIONS];
	local_hwc_ratios[HWC_FLOPS] = (double) lwm2_metric_total[LWM2_METRIC_FLOPS];
	local_hwc_ratios[HWC_FLOINS] = lwm2_metric_total[LWM2_METRIC_FLOINS] / (double) lwm2_metric_total[LWM2_METRIC_INSTRUCTIONS] * 100;

	unsigned long long l1_cache_hits = lwm2_metric_total[LWM2_METRIC_L1_DATA_ACCESS] - lwm2_metric_total[LWM2_METRIC_L1_DATA_MISSES];

	local_hwc_ratios[HWC_L1] = l1_cache_hits / (double) lwm2_metric_total[LWM2_METRIC_L1_DATA_ACCESS] * 100;
	
#else
	lwm2_hwc_custom_convert_for_summary(lwm2_metric_total, local_custom_summary_hwc);
#endif
	
//	double total_mpi_p2p_time = lwm2_metric_total[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_sampling_rate;
//	if(total_mpi_p2p_time == 0)
//	{
//		total_mpi_p2p_time = 1 / (double) lwm2_sampling_rate;
//	}
	
//	local_mpi_trans_rate[MPI_TRANS_RATE_P2P] = (lwm2_metric_total[LWM2_METRIC_P2P_SENT_BYTES] + lwm2_metric_total[LWM2_METRIC_P2P_RECV_BYTES]) / total_mpi_p2p_time;
	
//	double total_mpi_coll_time = lwm2_metric_total[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_sampling_rate;
//	if(total_mpi_coll_time == 0)
//	{
//		total_mpi_coll_time = 1 / (double) lwm2_sampling_rate;
//	}
//	local_mpi_trans_rate[MPI_TRANS_RATE_COLL] = (lwm2_metric_total[LWM2_METRIC_COLL_SENT_BYTES] + lwm2_metric_total[LWM2_METRIC_COLL_RECV_BYTES]) / total_mpi_coll_time;

	double exec_time_till_now = util_get_timestamp() - lwm2_start_time;
	double sequential_signals = exec_time_till_now * lwm2_sampling_rate;
	
	local_mpi_trans_rate[TOTAL_EXEC_TIME] = exec_time_till_now;
//	local_mpi_trans_rate[MPI_FREQ_P2P] = local_metric_p2p_total_msg / exec_time_till_now;
//	local_mpi_trans_rate[MPI_FREQ_COLL] = lwm2_metric_total[LWM2_METRIC_COLL_CALL_COUNT] / exec_time_till_now;
	local_mpi_trans_rate[OMP_EFF_THRDS] = lwm2_metric_total[LWM2_SIGNAL_TOTAL] / sequential_signals;

	local_cuda_data[CUDA_DATA_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE];
	local_cuda_data[CUDA_DATA_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST];

	local_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS];

	local_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS];
	local_cuda_data_det[CUDA_RATE_MSG_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS] / exec_time_till_now;
	local_cuda_data_det[CUDA_RATE_MSG_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS] / exec_time_till_now;
}

void lwm2_output_summary_calculate_ratios()
{
	if(lwm2_output_should_print_summary() == 0)
	{
		return;
	}

	local_signal_ratios[SIGNAL_MPI] = lwm2_metric_total[LWM2_SIGNAL_MPI] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_FILE_IO] = lwm2_metric_total[LWM2_SIGNAL_FILE_IO] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_MPI_IO] = lwm2_metric_total[LWM2_SIGNAL_MPI_IO] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_MPI_P2P] = lwm2_metric_total[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_MPI_COLL] = lwm2_metric_total[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA] = lwm2_metric_total[LWM2_SIGNAL_CUDA] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA_TO_DEVICE] = lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_DEVICE] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	local_signal_ratios[SIGNAL_CUDA_TO_HOST] = lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_HOST] / (double) lwm2_metric_total[LWM2_SIGNAL_TOTAL];
	
	local_metric_p2p_total_msg = lwm2_metric_total[LWM2_METRIC_P2P_SENT_MSGS] + lwm2_metric_total[LWM2_METRIC_P2P_RECV_MSGS];

	
#ifndef LWM2_HWC_EXTERN_DEF
	local_hwc_ratios[HWC_CPI] = lwm2_metric_total[LWM2_METRIC_CYCLES] / (double) lwm2_metric_total[LWM2_METRIC_INSTRUCTIONS];
	local_hwc_ratios[HWC_FLOPS] = (double) lwm2_metric_total[LWM2_METRIC_FLOPS];
	local_hwc_ratios[HWC_FLOINS] = lwm2_metric_total[LWM2_METRIC_FLOINS] / (double) lwm2_metric_total[LWM2_METRIC_INSTRUCTIONS] * 100;

	unsigned long long l1_cache_hits = lwm2_metric_total[LWM2_METRIC_L1_DATA_ACCESS] - lwm2_metric_total[LWM2_METRIC_L1_DATA_MISSES];

	local_hwc_ratios[HWC_L1] = l1_cache_hits / (double) lwm2_metric_total[LWM2_METRIC_L1_DATA_ACCESS] * 100;
	
#else
	lwm2_hwc_custom_convert_for_summary(lwm2_metric_total, local_custom_summary_hwc);
#endif
	
	double total_mpi_p2p_time = lwm2_metric_total[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_sampling_rate;
	if(total_mpi_p2p_time == 0)
	{
		total_mpi_p2p_time = 1 / (double) lwm2_sampling_rate;
	}
	
	local_mpi_trans_rate[MPI_TRANS_RATE_P2P] = (lwm2_metric_total[LWM2_METRIC_P2P_SENT_BYTES] + lwm2_metric_total[LWM2_METRIC_P2P_RECV_BYTES]) / total_mpi_p2p_time;
	
	double total_mpi_coll_time = lwm2_metric_total[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_sampling_rate;
	if(total_mpi_coll_time == 0)
	{
		total_mpi_coll_time = 1 / (double) lwm2_sampling_rate;
	}
	local_mpi_trans_rate[MPI_TRANS_RATE_COLL] = (lwm2_metric_total[LWM2_METRIC_COLL_SENT_BYTES] + lwm2_metric_total[LWM2_METRIC_COLL_RECV_BYTES]) / total_mpi_coll_time;

	double exec_time_till_now = util_get_timestamp() - lwm2_start_time;
	double sequential_signals = exec_time_till_now * lwm2_sampling_rate;
	
	local_mpi_trans_rate[TOTAL_EXEC_TIME] = exec_time_till_now;
	local_mpi_trans_rate[MPI_FREQ_P2P] = local_metric_p2p_total_msg / exec_time_till_now;
	local_mpi_trans_rate[MPI_FREQ_COLL] = lwm2_metric_total[LWM2_METRIC_COLL_CALL_COUNT] / exec_time_till_now;
	local_mpi_trans_rate[OMP_EFF_THRDS] = lwm2_metric_total[LWM2_SIGNAL_TOTAL] / sequential_signals;

	local_cuda_data[CUDA_DATA_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE];
	local_cuda_data[CUDA_DATA_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST];

	local_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS];

	local_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS];
	local_cuda_data_det[CUDA_RATE_MSG_TO_DEVICE] = lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS] / exec_time_till_now;
	local_cuda_data_det[CUDA_RATE_MSG_TO_HOST] = lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS] / exec_time_till_now;
}

void lwm2_output_summary_merge_global()
{
	if(lwm2_output_should_print_summary() == 0)
	{
		return;
	}
	memset(lwm2_metric_sum, 0 , sizeof(unsigned long long) * LWM2_TOTAL_METRIC_COUNT);
	memset(lwm2_metric_max, 0 , sizeof(unsigned long long) * LWM2_TOTAL_METRIC_COUNT);
	memset(lwm2_metric_min, 0 , sizeof(unsigned long long) * LWM2_TOTAL_METRIC_COUNT);
	
	PMPI_Reduce(&lwm2_metric_total, &lwm2_metric_sum, LWM2_TOTAL_METRIC_COUNT, 
			MPI_UNSIGNED_LONG_LONG, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&lwm2_metric_total, lwm2_metric_min, LWM2_TOTAL_METRIC_COUNT, 
			MPI_UNSIGNED_LONG_LONG, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&lwm2_metric_total, lwm2_metric_max, LWM2_TOTAL_METRIC_COUNT, 
			MPI_UNSIGNED_LONG_LONG, MPI_MAX, 0, MPI_COMM_WORLD);
	
	lwm2_output_summary_calculate_ratios();
	
	PMPI_Reduce(&local_signal_ratios, &sum_signal_ratios, RATIO_CNT, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_signal_ratios, &min_signal_ratios, RATIO_CNT, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_signal_ratios, &max_signal_ratios, RATIO_CNT, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
	
	PMPI_Reduce(&local_metric_p2p_total_msg, &sum_metric_p2p_total_msg, 1, MPI_UNSIGNED_LONG_LONG, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_metric_p2p_total_msg, &min_metric_p2p_total_msg, 1, MPI_UNSIGNED_LONG_LONG, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_metric_p2p_total_msg, &max_metric_p2p_total_msg, 1, MPI_UNSIGNED_LONG_LONG, MPI_MAX, 0, MPI_COMM_WORLD);
	
//	PMPI_Reduce(&local_exec_time_till_now, &sum_exec_time_till_now, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
//	PMPI_Reduce(&local_exec_time_till_now, &min_exec_time_till_now, 1, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
//	PMPI_Reduce(&local_exec_time_till_now, &max_exec_time_till_now, 1, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
//
//	PMPI_Reduce(&local_p2p_freq, &sum_p2p_freq, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
//	PMPI_Reduce(&local_p2p_freq, &min_p2p_freq, 1, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
//	PMPI_Reduce(&local_p2p_freq, &max_p2p_freq, 1, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
	
	PMPI_Reduce(&lwm2_mpi_comm_data, &min_mpi_comm_data, COMM_DATA_TOTAL, MPI_UNSIGNED_LONG_LONG, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&lwm2_mpi_comm_data, &max_mpi_comm_data, COMM_DATA_TOTAL, MPI_UNSIGNED_LONG_LONG, MPI_MAX, 0, MPI_COMM_WORLD);
	
#ifndef LWM2_HWC_EXTERN_DEF
	PMPI_Reduce(&local_hwc_ratios, &sum_hwc_ratios, HWC_TOTAL, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_hwc_ratios, &min_hwc_ratios, HWC_TOTAL, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_hwc_ratios, &max_hwc_ratios, HWC_TOTAL, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
	
#else
	PMPI_Reduce(&local_custom_summary_hwc, &sum_custom_summary_hwc, LWM2_CUSTOM_SUMMARY_HWC_COUNT, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_custom_summary_hwc, &min_custom_summary_hwc, LWM2_CUSTOM_SUMMARY_HWC_COUNT, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_custom_summary_hwc, &max_custom_summary_hwc, LWM2_CUSTOM_SUMMARY_HWC_COUNT, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);

#endif
	
	PMPI_Reduce(&local_mpi_trans_rate, &sum_mpi_trans_rate, MPI_RATIO_TOTAL, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_mpi_trans_rate, &min_mpi_trans_rate, MPI_RATIO_TOTAL, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_mpi_trans_rate, &max_mpi_trans_rate, MPI_RATIO_TOTAL, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
	
	PMPI_Reduce(&local_cuda_data, &sum_cuda_data, CUDA_DATA_COUNT, MPI_UNSIGNED_LONG_LONG, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_cuda_data, &min_cuda_data, CUDA_DATA_COUNT, MPI_UNSIGNED_LONG_LONG, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_cuda_data, &max_cuda_data, CUDA_DATA_COUNT, MPI_UNSIGNED_LONG_LONG, MPI_MAX, 0, MPI_COMM_WORLD);

	PMPI_Reduce(&local_cuda_data_det, &sum_cuda_data_det, CUDA_DATA_DETAIL_COUNT, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_cuda_data_det, &min_cuda_data_det, CUDA_DATA_DETAIL_COUNT, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
	PMPI_Reduce(&local_cuda_data_det, &max_cuda_data_det, CUDA_DATA_DETAIL_COUNT, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
}

void lwm2_output_summary_print_summary_non_mpi()
{
	timestamp_t lwm2_total_exe_time = lwm2_end_time - lwm2_start_time;
	
	if(lwm2_output_should_print_summary() == 0)
	{
		return;
	}
	
	lwm2_output_summary_calculate_ratios_non_mpi();

	printf("\n");
	printf("********************************************************************************************************************\n");
	printf("\t\t\t\t\t\t\tJob Digest\n");
	printf("********************************************************************************************************************\n");
	
	printf("\t%045s \t%d\n", "Job id:", lwm2_job_id);
	printf("\t%045s \t%.2f\n", "Wall clock time [s]:", lwm2_total_exe_time);
	printf("\t%045s \t%d\n", "Nr. of Processes:", lwm2_mpi_comm_size);
	printf("\t%045s \t%ld\n", "Sampling rate [Hz]:", lwm2_sampling_rate);

	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s\n", "Time spent:"); 
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	printf("\t%045s %16.2f%%\n", 
			"Time spent in POSIX I/O:",
			((double)lwm2_metric_total[LWM2_SIGNAL_FILE_IO] / lwm2_metric_total[LWM2_SIGNAL_TOTAL] * 100)); 
	
if(lwm2_output_should_print_cuda())
{
	printf("\t%045s %16.2f%%\n", 
			"Time spent in CUDA:",
			((double)lwm2_metric_total[LWM2_SIGNAL_CUDA] / lwm2_metric_total[LWM2_SIGNAL_TOTAL] * 100)); 
	
	printf("\t%045s %16.2f%%\n", 
			"Time spent in CUDA memcpy to device:",
			((double)lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_DEVICE] / lwm2_metric_total[LWM2_SIGNAL_TOTAL] * 100)); 
	
	printf("\t%045s %16.2f%%\n", 
			"Time spent in CUDA memcpy to host:",
			((double)lwm2_metric_total[LWM2_SIGNAL_CUDA_TO_HOST] / lwm2_metric_total[LWM2_SIGNAL_TOTAL] * 100)); 
}

if(lwm2_output_should_print_cuda())
{
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s\n", "CUDA:");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	printf("\t%045s %16llu\n", 
				"Data transfered to device [Bytes]:", 
				lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE]);
	
	printf("\t%045s %16llu\n",
					"Data transfered to host [Bytes]:", 
					lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST]); 
		
	printf("\t%045s %16.2f\n",
					"Data transfered to device [per msg]:", 
					(lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_DEVICE] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS]));
		
	printf("\t%045s %16.2f\n", 
					"Data transfered to host [per msg]:", 
					(lwm2_metric_total[LWM2_METRIC_CUDA_BYTES_TO_HOST] / (double) lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS]));
		
	printf("\t%045s %16.2f\n", 
					"Frequency of Memcpy calls to device [/sec]:", 
					(lwm2_metric_total[LWM2_METRIC_CUDA_TO_DEVICE_CALLS]/ local_mpi_trans_rate[TOTAL_EXEC_TIME]));
		
	printf("\t%045s %16.2f\n", 
					"Frequency of Memcpy calls to host [/sec]:", 
					(lwm2_metric_total[LWM2_METRIC_CUDA_TO_HOST_CALLS]/ local_mpi_trans_rate[TOTAL_EXEC_TIME]));
		
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s\n", "Multithreading performance:");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	printf("\t%045s %16.2f\n", 
			"OMP effective threads:",
			local_mpi_trans_rate[OMP_EFF_THRDS]);

	printf("\t%045s %13llu\n", 
				"Max. thread count:",
				lwm2_metric_total[LWM2_METRIC_MAX_NUM_THREADS]);
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s\n", "Sequential performance:");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
#ifndef LWM2_HWC_EXTERN_DEF
	if((lwm2_hwc_registered & LWM2_HWC_CPI) != 0)
	{
		printf("\t%045s %16.2f\n", 
				"CPI:",
				local_hwc_ratios[HWC_CPI]); 
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOPS) != 0)
	{
		printf("\t%045s %16.2f\n", 
				"FLOPS",
				local_hwc_ratios[HWC_FLOPS]); 
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOPS) != 0 && (lwm2_hwc_registered & LWM2_HWC_FLOINS) == 0)
	{
		printf("\t%045s %16.2f\n", 
				"FP Operations:", 
				local_hwc_ratios[HWC_FLOINS]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOINS) != 0)
	{
		printf("\t%045s %16.2f\n", 
				"FP Instructions",
				local_hwc_ratios[HWC_FLOINS]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_L1_RATIO) != 0)
	{
		printf("\t%045s %16.2f\n", 
				"L1 data hit ratio:",
				local_hwc_ratios[HWC_L1]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_LOAD_STORE_RATIO) != 0)
	{
		printf("\t%045s %16.2f\n", 
				"Load/Store L1 hit ratio:",
				local_hwc_ratios[HWC_L1]);
	}
	
#else
	
	int ind;
	for(ind = 0; ind < LWM2_CUSTOM_SUMMARY_HWC_COUNT; ind++)
	{
		printf("\t%045s: %16.2f\n", 
				lwm2_custom_summary_hwc_names[ind],
				local_custom_summary_hwc[ind]); 
	}
	
#endif
}
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
}
void lwm2_output_summary_print_summary()
{
	timestamp_t lwm2_total_exe_time = lwm2_end_time - lwm2_start_time;
	
	if(lwm2_output_should_print_summary() == 0)
	{
		return;
	}
	
	printf("\n");
	printf("********************************************************************************************************************\n");
	printf("\t\t\t\t\t\t\tJob Digest\n");
	printf("********************************************************************************************************************\n");
	
	printf("\t%045s \t%d\n", "Job id:", lwm2_job_id);
	printf("\t%045s \t%.2f\n", "Wall clock time [s]:", lwm2_total_exe_time);
	printf("\t%045s \t%d\n", "Nr. of Processes:", lwm2_mpi_comm_size);
	printf("\t%045s \t%ld\n", "Sampling rate [Hz]:", lwm2_sampling_rate);

if(lwm2_output_should_print_verbose())
{
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s \t%.2f\n", "Average total signals [per process]:",   
			lwm2_metric_sum[LWM2_SIGNAL_TOTAL] / (double) lwm2_mpi_comm_size);
	printf("\t%045s \t%.2f\n", "Average MPI signals [per process]:", 
			lwm2_metric_sum[LWM2_SIGNAL_MPI] / (double) lwm2_mpi_comm_size);
	printf("\t%045s \t%.2f\n", "Average MPI P2P signals [per process]:",  
			lwm2_metric_sum[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_mpi_comm_size);
	printf("\t%045s \t%.2f\n", "Average MPI Coll. signals [per process]:", 
			lwm2_metric_sum[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_mpi_comm_size);
	printf("\t%045s \t%.2f\n", "Average Max. number of threads [per process]:",
			lwm2_metric_sum[LWM2_METRIC_MAX_NUM_THREADS] / (double) lwm2_mpi_comm_size);
}
	if(lwm2_metric_sum[LWM2_SIGNAL_TOTAL] == 0)
	{
		lwm2_metric_sum[LWM2_SIGNAL_TOTAL] = 1;
	}
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s %14s   \t%14s   \t%14s\n", "Time spent:", "Average", "Minimum", "Maximum");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	double mpi_spent_time = ((double)lwm2_metric_sum[LWM2_SIGNAL_MPI] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100);
	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
			"Time spent in MPI:",
			mpi_spent_time, 
			min_signal_ratios[SIGNAL_MPI] * 100, 
			max_signal_ratios[SIGNAL_MPI] * 100);

	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
				"Time spent in MPI P2P:",
				((double)lwm2_metric_sum[LWM2_SIGNAL_MPI_P2P] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
				min_signal_ratios[SIGNAL_MPI_P2P] * 100, 
				max_signal_ratios[SIGNAL_MPI_P2P] * 100);
	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
				"Time spent in MPI Coll:",
				((double)lwm2_metric_sum[LWM2_SIGNAL_MPI_COLL] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
				min_signal_ratios[SIGNAL_MPI_COLL] * 100, 
				max_signal_ratios[SIGNAL_MPI_COLL] * 100);
	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
			"Time spent in MPI I/O:",
			((double)lwm2_metric_sum[LWM2_SIGNAL_MPI_IO] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
			min_signal_ratios[SIGNAL_MPI_IO] * 100, 
			max_signal_ratios[SIGNAL_MPI_IO] * 100);

	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
			"Time spent in POSIX I/O:",
			((double)lwm2_metric_sum[LWM2_SIGNAL_FILE_IO] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
			min_signal_ratios[SIGNAL_FILE_IO] * 100, 
			max_signal_ratios[SIGNAL_FILE_IO] * 100);
	

if(lwm2_output_should_print_cuda())
{
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
#ifdef TREAT_CUDA_AS_MPI
			"Time spent in CUDA and MPI:",
#else
			"Time spent in CUDA:",
#endif
			((double)lwm2_metric_sum[LWM2_SIGNAL_CUDA] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
			min_signal_ratios[SIGNAL_CUDA] * 100, 
			max_signal_ratios[SIGNAL_CUDA] * 100);
	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
			"Time spent in CUDA memcpy to device:",
			((double)lwm2_metric_sum[LWM2_SIGNAL_CUDA_TO_DEVICE] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
			min_signal_ratios[LWM2_SIGNAL_CUDA_TO_DEVICE] * 100, 
			max_signal_ratios[LWM2_SIGNAL_CUDA_TO_DEVICE] * 100);
	
	printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", 
			"Time spent in CUDA memcpy to host:",
			((double)lwm2_metric_sum[LWM2_SIGNAL_CUDA_TO_HOST] / lwm2_metric_sum[LWM2_SIGNAL_TOTAL] * 100), 
			min_signal_ratios[LWM2_SIGNAL_CUDA_TO_DEVICE] * 100, 
			max_signal_ratios[LWM2_SIGNAL_CUDA_TO_DEVICE] * 100);
}
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s %14s   \t%14s   \t%14s\n", "MPI Communication:", "Average", "Minimum", "Maximum");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
		
	unsigned long long total_p2p_bytes = lwm2_metric_sum[LWM2_METRIC_P2P_SENT_BYTES] + lwm2_metric_sum[LWM2_METRIC_P2P_RECV_BYTES];
	unsigned long long total_p2p_msgs = lwm2_metric_sum[LWM2_METRIC_P2P_SENT_MSGS] + lwm2_metric_sum[LWM2_METRIC_P2P_RECV_MSGS];
	unsigned long long total_coll_bytes = lwm2_metric_sum[	LWM2_METRIC_COLL_SENT_BYTES] + lwm2_metric_sum[LWM2_METRIC_COLL_RECV_BYTES];
	
	double avg_coll_trans_rate, avg_p2p_trans_rate;
	
	avg_coll_trans_rate = total_coll_bytes / ( lwm2_metric_sum[LWM2_SIGNAL_MPI_COLL] / (double) lwm2_sampling_rate);
	avg_p2p_trans_rate = total_p2p_bytes / (lwm2_metric_sum[LWM2_SIGNAL_MPI_P2P] / (double) lwm2_sampling_rate);
	
	if(avg_coll_trans_rate < min_mpi_trans_rate[MPI_TRANS_RATE_COLL] || avg_coll_trans_rate > max_mpi_trans_rate[MPI_TRANS_RATE_COLL])
	{
		avg_coll_trans_rate = sum_mpi_trans_rate[MPI_TRANS_RATE_COLL] / (double) lwm2_mpi_comm_size;
	}
	
	if(avg_p2p_trans_rate < min_mpi_trans_rate[MPI_TRANS_RATE_P2P] || avg_p2p_trans_rate > max_mpi_trans_rate[MPI_TRANS_RATE_P2P])
	{
		avg_p2p_trans_rate = sum_mpi_trans_rate[MPI_TRANS_RATE_P2P] / (double) lwm2_mpi_comm_size;
	}

	double p2p_msg_size = 0;
	if(sum_metric_p2p_total_msg != 0)
	{
		p2p_msg_size = total_p2p_bytes / (double) sum_metric_p2p_total_msg;
	}
	else
	{
		min_mpi_comm_data[MIN_P2P_MSG_SIZE] = 0;
		avg_p2p_trans_rate = 0;
	}
	printf("\t%045s %16.2f\t%16llu\t%16llu\n", 
			"Size of P2P messages [Bytes]:", 
			p2p_msg_size,
//			total_p2p_bytes / (double) sum_metric_p2p_total_msg,
			min_mpi_comm_data[MIN_P2P_MSG_SIZE],
			max_mpi_comm_data[MAX_P2P_MSG_SIZE]);
	
	double coll_msg_size_sent = 0, coll_msg_size_recv = 0;
	if(lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT] != 0)
	{
		coll_msg_size_sent = lwm2_metric_sum[LWM2_METRIC_COLL_SENT_BYTES] / (double) lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT];
		coll_msg_size_recv = lwm2_metric_sum[LWM2_METRIC_COLL_RECV_BYTES] / (double) lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT];
	}
	else
	{
		min_mpi_comm_data[MIN_COLL_MSG_SENT] = 0;
		min_mpi_comm_data[MIN_COLL_MSG_RECV] = 0;
		avg_coll_trans_rate = 0;
	}

	printf("\t%045s %16.2f\t%16llu\t%16llu\n", 
			"Size of collective messages sent [Bytes]:", 
			coll_msg_size_sent,
//			lwm2_metric_sum[LWM2_METRIC_COLL_SENT_BYTES] / (double) lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT], 
			min_mpi_comm_data[MIN_COLL_MSG_SENT], 
			max_mpi_comm_data[MAX_COLL_MSG_SENT] );
	
	printf("\t%045s %16.2f\t%16llu\t%16llu\n", 
			"Size of collective messages recv [Bytes]:", 
			coll_msg_size_recv,
//			lwm2_metric_sum[LWM2_METRIC_COLL_RECV_BYTES] / (double) lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT], 
			min_mpi_comm_data[MIN_COLL_MSG_RECV], 
			max_mpi_comm_data[MAX_COLL_MSG_RECV] );
	
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n",
			"P2P message frequency [/s]:",
			(sum_metric_p2p_total_msg / sum_mpi_trans_rate[TOTAL_EXEC_TIME]),
			min_mpi_trans_rate[MPI_FREQ_P2P], 
			max_mpi_trans_rate[MPI_FREQ_P2P] );
	
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
				"Collective invocation frequency [/s]:",
				(lwm2_metric_sum[LWM2_METRIC_COLL_CALL_COUNT] / sum_mpi_trans_rate[TOTAL_EXEC_TIME]),
				min_mpi_trans_rate[MPI_FREQ_COLL] ,
				max_mpi_trans_rate[MPI_FREQ_COLL]);
	
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
					"P2P bytes transfer rate [/s]:",
					avg_p2p_trans_rate,
					min_mpi_trans_rate[MPI_TRANS_RATE_P2P],
					max_mpi_trans_rate[MPI_TRANS_RATE_P2P]);
	
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
						"Coll bytes transfer rate [/s]:",
						avg_coll_trans_rate,
						min_mpi_trans_rate[MPI_TRANS_RATE_COLL],
						max_mpi_trans_rate[MPI_TRANS_RATE_COLL]);
	
if(lwm2_output_should_print_cuda())
{
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s %14s   \t%14s   \t%14s\n", "CUDA:", "Average", "Minimum", "Maximum");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	printf("\t%045s %16.2f\t%16llu\t%16llu\n", 
				"Data transfered to device [Bytes]:", 
				(sum_cuda_data[CUDA_DATA_TO_DEVICE] / (double) lwm2_mpi_comm_size),
				min_cuda_data[CUDA_DATA_TO_DEVICE],
				max_cuda_data[CUDA_DATA_TO_DEVICE]);
	
	printf("\t%045s %16.2f\t%16llu\t%16llu\n", 
					"Data transfered to host [Bytes]:", 
					(sum_cuda_data[CUDA_DATA_TO_HOST] / (double) lwm2_mpi_comm_size),
					min_cuda_data[CUDA_DATA_TO_HOST],
					max_cuda_data[CUDA_DATA_TO_HOST]);

	double cuda_data_to_device = 0, cuda_data_to_host = 0;
	if(lwm2_metric_sum[LWM2_METRIC_CUDA_TO_DEVICE_CALLS] != 0)
	{
		cuda_data_to_device = sum_cuda_data[CUDA_DATA_TO_DEVICE] / (double) lwm2_metric_sum[LWM2_METRIC_CUDA_TO_DEVICE_CALLS];
	}
	else
	{
		min_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE] = 0;
		max_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE] = 0;
	}

	if(lwm2_metric_sum[LWM2_METRIC_CUDA_TO_HOST_CALLS] != 0)
	{
		cuda_data_to_host = sum_cuda_data[CUDA_DATA_TO_HOST] / (double) lwm2_metric_sum[LWM2_METRIC_CUDA_TO_HOST_CALLS];
	}
	else
	{
		min_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST] = 0;
		max_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST] = 0;
	}
		
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
					"Data transfered to device [per msg]:", 
					cuda_data_to_device,
//					(sum_cuda_data[CUDA_DATA_TO_DEVICE] / (double) lwm2_metric_sum[LWM2_METRIC_CUDA_TO_DEVICE_CALLS]),
					min_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE],
					max_cuda_data_det[CUDA_DATA_PER_MSG_TO_DEVICE]);
		
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
					"Data transfered to host [per msg]:", 
					cuda_data_to_host,
//					(sum_cuda_data[CUDA_DATA_TO_HOST] / (double) lwm2_metric_sum[LWM2_METRIC_CUDA_TO_HOST_CALLS]),
					min_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST],
					max_cuda_data_det[CUDA_DATA_PER_MSG_TO_HOST]);
		
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
					"Frequency of Memcpy calls to device [/sec]:", 
					(lwm2_metric_sum[LWM2_METRIC_CUDA_TO_DEVICE_CALLS]/ sum_mpi_trans_rate[TOTAL_EXEC_TIME]),
					min_cuda_data_det[CUDA_RATE_MSG_TO_DEVICE],
					max_cuda_data_det[CUDA_RATE_MSG_TO_DEVICE]);
		
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
					"Frequency of Memcpy calls to host [/sec]:", 
					(lwm2_metric_sum[LWM2_METRIC_CUDA_TO_HOST_CALLS]/ sum_mpi_trans_rate[TOTAL_EXEC_TIME]),
					min_cuda_data_det[CUDA_RATE_MSG_TO_HOST],
					max_cuda_data_det[CUDA_RATE_MSG_TO_HOST]);
}
		
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s %14s   \t%14s   \t%14s\n", "Multithreading performance:", "Average", "Minimum", "Maximum");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
	printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
			"OMP effective threads:",
			sum_mpi_trans_rate[OMP_EFF_THRDS] / lwm2_mpi_comm_size,
			min_mpi_trans_rate[OMP_EFF_THRDS],
			max_mpi_trans_rate[OMP_EFF_THRDS]);

	printf("\t%045s %13llu   \t%13llu   \t%13llu\n", 
				"Max. thread count:",
				lwm2_metric_sum[LWM2_METRIC_MAX_NUM_THREADS] / lwm2_mpi_comm_size,
				lwm2_metric_min[LWM2_METRIC_MAX_NUM_THREADS],
				lwm2_metric_max[LWM2_METRIC_MAX_NUM_THREADS]);
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	printf("\t%045s %14s   \t%14s   \t%14s\n", "Sequential performance:", "Average", "Minimum", "Maximum");
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
#ifndef LWM2_HWC_EXTERN_DEF
	if((lwm2_hwc_registered & LWM2_HWC_CPI) != 0)
	{
		printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", 
				"CPI:",
				sum_hwc_ratios[HWC_CPI] / lwm2_mpi_comm_size,
				min_hwc_ratios[HWC_CPI],
				max_hwc_ratios[HWC_CPI]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOPS) != 0)
	{
		printf("\t%045s %16.2f\t%16.2f\t%16.2f\n", "FLOPS:", 
				sum_hwc_ratios[HWC_FLOPS] / lwm2_mpi_comm_size, 
				min_hwc_ratios[HWC_FLOPS], 
				max_hwc_ratios[HWC_FLOPS]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOPS) != 0 && (lwm2_hwc_registered & LWM2_HWC_FLOINS) == 0)
	{
		printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", "FP Operations:", 
				sum_hwc_ratios[HWC_FLOINS] / lwm2_mpi_comm_size, 
				min_hwc_ratios[HWC_FLOINS], 
				max_hwc_ratios[HWC_FLOINS]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_FLOINS) != 0)
	{
		printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", "FP Instructions:", 
				sum_hwc_ratios[HWC_FLOINS] / lwm2_mpi_comm_size, 
				min_hwc_ratios[HWC_FLOINS], 
				max_hwc_ratios[HWC_FLOINS]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_L1_RATIO) != 0)
	{
		printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", "L1 data hit ratio:", 
				sum_hwc_ratios[HWC_L1] / lwm2_mpi_comm_size, 
				min_hwc_ratios[HWC_L1], 
				max_hwc_ratios[HWC_L1]);
	}
	
	if((lwm2_hwc_registered & LWM2_HWC_LOAD_STORE_RATIO) != 0)
	{
		printf("\t%045s %16.2f%%\t%16.2f%%\t%16.2f%%\n", "Load/Store L1 hit ratio:", 
				sum_hwc_ratios[HWC_L1] / lwm2_mpi_comm_size, 
				min_hwc_ratios[HWC_L1], 
				max_hwc_ratios[HWC_L1]);
	}
	
#else
	
	int ind;
	for(ind = 0; ind < LWM2_CUSTOM_SUMMARY_HWC_COUNT; ind++)
	{
		printf("\t%045s: %16.2f\t%16.2f\t%16.2f\n", 
				lwm2_custom_summary_hwc_names[ind],
				sum_custom_summary_hwc[ind] / lwm2_mpi_comm_size,
				min_custom_summary_hwc[ind],
				max_custom_summary_hwc[ind]);
	}
	
#endif
	
	printf("--------------------------------------------------------------------------------------------------------------------\n");
	
//	if(mpi_spent_time > MPI_TIME_THRESHOLD)
//	{
//		printf("\t**** The application spent a significant amount of time in MPI. It is recommended \n\t**** to analyze this application with a specialized tool.\n");
//		printf("--------------------------------------------------------------------------------------------------------------------\n");
//	}

}
